@@ -636,18 +636,18 @@ struct cpe_language *cpe_language_new()
  * More info in representive header file.
  * returns the type of <structure>
  */
-struct cpe_dict_model *cpe_dict_model_parse_xml(const struct oscap_import_source *source)
+struct cpe_dict_model *cpe_dict_model_parse_xml(const char *file)
 {
 
-	__attribute__nonnull__(source);
+	__attribute__nonnull__(file);
 
 	xmlTextReaderPtr reader;
 	struct cpe_dict_model *dict = NULL;
 
-	if (!cpe_validate_xml(oscap_import_source_get_name(source)))
+	if (!cpe_validate_xml(file))
 		return NULL;
 
-	reader = xmlReaderForFile(oscap_import_source_get_name(source), NULL, 0);
+	reader = xmlReaderForFile(file, NULL, 0);
 	if (reader != NULL) {
 		xmlTextReaderNextNode(reader);
 		dict = cpe_dict_model_parse(reader);
@@ -1065,30 +1065,25 @@ struct cpe_vendor *cpe_vendor_parse(xmlTextReaderPtr reader)
  * More info in representive header file.
  * returns the type of <structure>
  */
-void cpe_dict_model_export_xml(const struct cpe_dict_model *dict, const struct oscap_export_target *target)
+void cpe_dict_model_export_xml(const struct cpe_dict_model *dict, const char *file)
 {
 
 	__attribute__nonnull__(dict);
-	__attribute__nonnull__(target);
+	__attribute__nonnull__(file);
 
 	// TODO: add macro to check return value from xmlTextWriter* functions
 	xmlTextWriterPtr writer;
 
-	writer = xmlNewTextWriterFilename(oscap_export_target_get_name(target), 0);
+	writer = xmlNewTextWriterFilename(file, 0);
 	if (writer == NULL) {
 		oscap_setxmlerr(xmlGetLastError());
 		return;
 	}
-	// Set properties of writer TODO: make public function to edit this ??
-	// Yes - there will be structure oscap_export_target & oscap_parse_target
-	xmlTextWriterSetIndent(writer, oscap_export_target_get_indent(target));
-	xmlTextWriterSetIndentString(writer, BAD_CAST oscap_export_target_get_indent_string(target));
 
-	if (xmlFindCharEncodingHandler(oscap_export_target_get_encoding(target)) == NULL)
-		// forced default encoding
-		xmlTextWriterStartDocument(writer, NULL, NULL, NULL);
-	else
-		xmlTextWriterStartDocument(writer, NULL, oscap_export_target_get_encoding(target), NULL);
+	xmlTextWriterSetIndent(writer, 1);
+	xmlTextWriterSetIndentString(writer, BAD_CAST "    ");
+
+	xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
 
 	cpe_dict_export(dict, writer);
 	xmlTextWriterEndDocument(writer);
