@@ -271,18 +271,18 @@ static bool cpe_validate_xml(const char *filename)
 	return ret;
 }
 
-struct cpe_lang_model *cpe_lang_model_parse_xml(const struct oscap_import_source *source)
+struct cpe_lang_model *cpe_lang_model_parse_xml(const char *file)
 {
 
-	__attribute__nonnull__(source);
+	__attribute__nonnull__(file);
 
 	xmlTextReaderPtr reader;
 	struct cpe_lang_model *ret = NULL;
 
-	if (!cpe_validate_xml(oscap_import_source_get_name(source)))
+	if (!cpe_validate_xml(file))
 		return NULL;
 
-	reader = xmlReaderForFile(oscap_import_source_get_name(source), NULL, 0);
+	reader = xmlReaderForFile(file, NULL, 0);
 	if (reader != NULL) {
 		xmlTextReaderNextNode(reader);
 		ret = cpe_lang_model_parse(reader);
@@ -536,29 +536,25 @@ static char *parse_text_element(xmlTextReaderPtr reader, char *name)
  * More info in representive header file.
  * returns the type of <structure>
  */
-void cpe_lang_model_export_xml(const struct cpe_lang_model *spec, struct oscap_export_target *target)
+void cpe_lang_model_export_xml(const struct cpe_lang_model *spec, const char *file)
 {
 
 	__attribute__nonnull__(spec);
-	__attribute__nonnull__(target);
+	__attribute__nonnull__(file);
 
 	// TODO: ad macro to check return value from xmlTextWriter* functions
 	xmlTextWriterPtr writer;
 
-	writer = xmlNewTextWriterFilename(oscap_export_target_get_name(target), 0);
+	writer = xmlNewTextWriterFilename(file, 0);
 	if (writer == NULL) {
 		oscap_setxmlerr(xmlGetLastError());
 		return;
 	}
 	// Set properties of writer TODO: make public function to edit this ??
-	xmlTextWriterSetIndent(writer, oscap_export_target_get_indent(target));
-	xmlTextWriterSetIndentString(writer, BAD_CAST oscap_export_target_get_indent_string(target));
+	xmlTextWriterSetIndent(writer, 1);
+	xmlTextWriterSetIndentString(writer, BAD_CAST "    ");
 
-	if (xmlFindCharEncodingHandler(oscap_export_target_get_encoding(target)) == NULL)
-		// forced default encoding
-		xmlTextWriterStartDocument(writer, NULL, NULL, NULL);
-	else
-		xmlTextWriterStartDocument(writer, NULL, oscap_export_target_get_encoding(target), NULL);
+	xmlTextWriterStartDocument(writer, NULL, "UTF-8", NULL);
 
 	cpe_lang_export(spec, writer);
 	xmlTextWriterEndDocument(writer);
