@@ -122,6 +122,7 @@ struct oval_affected *oval_affected_new(struct oval_definition_model *model)
 
 bool oval_affected_is_valid(struct oval_affected * affected)
 {
+        oscap_dprintf("WARNING: NOOP.\n");
 	return true;		//TODO
 }
 
@@ -230,12 +231,13 @@ static int _oval_affected_parse_tag(xmlTextReaderPtr reader, struct oval_parser_
 			oscap_free(product);
 		}
 	} else {
-		int linno = xmlTextReaderGetParserLineNumber(reader);
 		int depth = xmlTextReaderDepth(reader);
+
 		if (depth == -1)
 			oscap_setxmlerr(xmlGetLastError());
 
-		oscap_dprintf("NOTICE::(oval_affected)skipping <%s> depth = %d line = %d\n", tagname, depth, linno);
+		oscap_dprintf("NOTICE::(oval_affected)skipping <%s> depth = %d line = %d\n", tagname, depth,
+                              xmlTextReaderGetParserLineNumber(reader));
 		return_code = oval_parser_skip_tag(reader, context);
 	}
 	oscap_free(tagname);
