@@ -37,9 +37,15 @@
 #include "oval_string_map_impl.h"
 #include "oval_agent_api_impl.h"
 #include "oval_parser_impl.h"
+#include "oval_probe_session.h"
 #include "common/util.h"
 #include "common/debug_priv.h"
 #include "common/_error.h"
+#if defined USE_REGEX_PCRE
+#include <pcre.h>
+#elif defined USE_REGEX_POSIX
+#include <regex.h>
+#endif
 
 /***************************************************************************/
 /* Variable definitions
@@ -86,6 +92,17 @@ static oval_syschar_collection_flag_t _flag_agg_map[7][7] = {
 #define _COMP_TYPE(comp) oval_component_type_get_text(oval_component_get_type(comp))
 #define _FLAG_TYPE(flag) oval_syschar_collection_flag_get_text(flag)
 
+typedef struct {
+	enum {
+		OVAL_MODE_COMPUTE,
+		OVAL_MODE_QUERY
+	} mode;
+	union {
+		struct oval_syschar_model *sysmod;
+		oval_probe_session_t *sess;
+	} u;
+} oval_argu_t;
+
 typedef struct oval_component {
 	struct oval_definition_model *model;
 	oval_component_type_t type;
@@ -668,6 +685,7 @@ struct oval_component *oval_component_new(struct oval_definition_model *model, o
 
 bool oval_component_is_valid(struct oval_component * component)
 {
+        oscap_dprintf("WARNING: NOOP.\n");
 	return true;		//TODO
 }
 
@@ -948,7 +966,7 @@ static int _oval_component_parse_BEGEND_tag(xmlTextReaderPtr reader,
 	__attribute__nonnull__(component);
 
 	oval_component_BEGEND_t *begend = (oval_component_BEGEND_t *) component;
-	begend->character = oscap_strdup((char *)xmlTextReaderGetAttribute(reader, BAD_CAST "character"));
+	begend->character = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "character");
 
 	return _oval_component_parse_FUNCTION_tag(reader, context, component);
 }
@@ -960,7 +978,7 @@ static int _oval_component_parse_SPLIT_tag(xmlTextReaderPtr reader,
 	__attribute__nonnull__(component);
 
 	oval_component_SPLIT_t *split = (oval_component_SPLIT_t *) component;
-	split->delimiter = oscap_strdup((char *)xmlTextReaderGetAttribute(reader, BAD_CAST "delimiter"));
+	split->delimiter = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "delimiter");
 
 	return _oval_component_parse_FUNCTION_tag(reader, context, component);
 }
@@ -972,8 +990,8 @@ static int _oval_component_parse_SUBSTRING_tag(xmlTextReaderPtr reader,
 	__attribute__nonnull__(component);
 
 	oval_component_SUBSTRING_t *substring = (oval_component_SUBSTRING_t *) component;
-	char *start_text = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "start");
-	char *length_text = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "length");
+	char *start_text = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "substring_start");
+	char *length_text = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "substring_length");
 	int start = (start_text == NULL) ? 0 : atoi(start_text);
 	int length = (length_text == NULL) ? 0 : atoi(length_text);
 	if (start_text != NULL)
@@ -1010,10 +1028,8 @@ static int _oval_component_parse_REGEX_CAPTURE_tag(xmlTextReaderPtr reader,
 	__attribute__nonnull__(component);
 
 	oval_component_REGEX_CAPTURE_t *regex = (oval_component_REGEX_CAPTURE_t *) component;
-	char *pattern = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "pattern");
-	regex->pattern = oscap_strdup(pattern);
-	if (pattern)
-		oscap_free(pattern);
+
+	regex->pattern = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "pattern");
 
 	return _oval_component_parse_FUNCTION_tag(reader, context, component);
 }
@@ -1067,16 +1083,15 @@ int oval_component_parse_tag(xmlTextReaderPtr reader,
 		component = oval_component_new(model, OVAL_FUNCTION_REGEX_CAPTURE);
 		return_code = _oval_component_parse_REGEX_CAPTURE_tag(reader, context, component);
 	} else {
-		int line = xmlTextReaderGetParserLineNumber(reader);
-		oscap_dprintf("NOTICE::oval_component_parse_tag::<%s> not handled (line = %d)", tagname, line);
+		oscap_dprintf("NOTICE::oval_component_parse_tag::<%s> not handled (line = %d)", tagname,
+                              xmlTextReaderGetParserLineNumber(reader));
 		return_code = oval_parser_skip_tag(reader, context);
 	}
 	if (component != NULL)
 		(*consumer) (component, user);
 	if (return_code != 1) {
-		int line = xmlTextReaderGetParserLineNumber(reader);
-		oscap_dprintf
-		    ("NOTICE: oval_component_parse_tag::parse of <%s> terminated on error at line %d", tagname, line);
+		oscap_dprintf("NOTICE: oval_component_parse_tag::parse of <%s> terminated on error at line %d", tagname,
+                            xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
 	return return_code;
@@ -1296,7 +1311,11 @@ xmlNode *oval_component_to_dom(struct oval_component *component, xmlDoc * doc, x
 	return component_node;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_LITERAL(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t oval_component_eval_common(oval_argu_t *argu,
+								 struct oval_component *component,
+								 struct oval_collection *value_collection);
+
+static oval_syschar_collection_flag_t _oval_component_evaluate_LITERAL(oval_argu_t *argu,
 								       struct oval_component *component,
 								       struct oval_collection *value_collection)
 {
@@ -1307,7 +1326,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_LITERAL(struct ov
 	return SYSCHAR_FLAG_COMPLETE;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(oval_argu_t *argu,
 									 struct oval_component *component,
 									 struct oval_collection *value_collection)
 {
@@ -1315,48 +1334,59 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_OBJECTREF(struct
 	__attribute__nonnull__(component);
 
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
+
 	struct oval_component_OBJECTREF *objref = (struct oval_component_OBJECTREF *)component;
 	struct oval_object *object = objref->object;
-	if (object) {
-		oval_probe_session_t *sess;
-		struct oval_syschar *syschar;
+	char *obj_id;
+	struct oval_syschar_model *sysmod;
+	struct oval_syschar *syschar;
 
-		sess = oval_probe_session_new(sysmod);
-		syschar = oval_probe_object_eval(sess, object, 0);
-		oval_probe_session_destroy(sess);
+	if (!object)
+		return flag;
 
-		if (syschar) {
-			flag = oval_syschar_get_flag(syschar);
-			char *field_name = objref->object_field;
-			struct oval_sysdata_iterator *sysdatas = oval_syschar_get_sysdata(syschar);
-			while (oval_sysdata_iterator_has_more(sysdatas)) {
-				struct oval_sysdata *sysdata = oval_sysdata_iterator_next(sysdatas);
-				struct oval_sysitem_iterator *items = oval_sysdata_get_items(sysdata);
-				while (oval_sysitem_iterator_has_more(items)) {
-					struct oval_sysitem *item = oval_sysitem_iterator_next(items);
-					char *item_name = oval_sysitem_get_name(item);
-					if (strcmp(field_name, item_name) == 0) {
-						char *text = oval_sysitem_get_value(item);
-						oval_datatype_t datatype = oval_sysitem_get_datatype(item);
-						struct oval_value *value = oval_value_new(datatype, text);
-						oval_collection_add(value_collection, value);
-					}
-				}
-				oval_sysitem_iterator_free(items);
-			}
-			oval_sysdata_iterator_free(sysdatas);
-		}
+	if (argu->mode == OVAL_MODE_QUERY) {
+		if (oval_probe_session_query_object(argu->u.sess, object) != 0)
+			return flag;
+		sysmod = oval_probe_session_getmodel(argu->u.sess);
+	} else {
+		sysmod = argu->u.sysmod;
 	}
+
+	obj_id = oval_object_get_id(object);
+	syschar = oval_syschar_model_get_syschar(sysmod, obj_id);
+
+	if (syschar) {
+		flag = oval_syschar_get_flag(syschar);
+		char *field_name = objref->object_field;
+		struct oval_sysdata_iterator *sysdatas = oval_syschar_get_sysdata(syschar);
+		while (oval_sysdata_iterator_has_more(sysdatas)) {
+			struct oval_sysdata *sysdata = oval_sysdata_iterator_next(sysdatas);
+			struct oval_sysitem_iterator *items = oval_sysdata_get_items(sysdata);
+			while (oval_sysitem_iterator_has_more(items)) {
+				struct oval_sysitem *item = oval_sysitem_iterator_next(items);
+				char *item_name = oval_sysitem_get_name(item);
+				if (strcmp(field_name, item_name) == 0) {
+					char *text = oval_sysitem_get_value(item);
+					oval_datatype_t datatype = oval_sysitem_get_datatype(item);
+					struct oval_value *value = oval_value_new(datatype, text);
+					oval_collection_add(value_collection, value);
+				}
+			}
+			oval_sysitem_iterator_free(items);
+		}
+		oval_sysdata_iterator_free(sysdatas);
+	}
+
 	return flag;
 #else
-	oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented, compiled without probes support.");	
+	oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented, compiled without probes support.");
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
 	return flag;
 #endif
 }
 
 #define _HAS_VALUES(flag) (flag==SYSCHAR_FLAG_COMPLETE || flag==SYSCHAR_FLAG_INCOMPLETE)
-static oval_syschar_collection_flag_t _oval_component_evaluate_VARREF(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_VARREF(oval_argu_t *argu,
 								      struct oval_component *component,
 								      struct oval_collection *value_collection)
 {
@@ -1365,24 +1395,35 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_VARREF(struct ova
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
 	struct oval_component_VARREF *varref = (struct oval_component_VARREF *)component;
 	struct oval_variable *variable = varref->variable;
-	if (variable) {
-		flag = oval_syschar_model_get_variable_collection_flag(sysmod, variable);
-		if (_HAS_VALUES(flag)) {
-			struct oval_value_iterator *values = oval_syschar_model_get_variable_values(sysmod, variable);
-			while (oval_value_iterator_has_more(values)) {
-				struct oval_value *value = oval_value_iterator_next(values);
-				oval_collection_add(value_collection, oval_value_clone(value));
-			}
-			oval_value_iterator_free(values);
-		}
-	} else {
-		oscap_dprintf("ERROR: No variable bound to VARREF Component");
+
+	if (!variable) {
+		oscap_dprintf("ERROR: No variable bound to VARREF Component.\n");
 		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "No variable bound to VARREF componenet");
+		return flag;
 	}
+
+	if (argu->mode == OVAL_MODE_QUERY) {
+		if (oval_probe_session_query_variable(argu->u.sess, variable) != 0)
+			return flag;
+	} else {
+		if (oval_syschar_model_compute_variable(argu->u.sysmod, variable) != 0)
+			return flag;
+	}
+
+	flag = oval_variable_get_collection_flag(variable);
+	if (_HAS_VALUES(flag)) {
+		struct oval_value_iterator *values = oval_variable_get_values(variable);
+		while (oval_value_iterator_has_more(values)) {
+			struct oval_value *value = oval_value_iterator_next(values);
+			oval_collection_add(value_collection, oval_value_clone(value));
+		}
+		oval_value_iterator_free(values);
+	}
+
 	return flag;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_BEGIN(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_BEGIN(oval_argu_t *argu,
 								     struct oval_component *component,
 								     struct oval_collection *value_collection)
 {
@@ -1394,7 +1435,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_BEGIN(struct oval
 		if (oval_component_iterator_has_more(subcomps)) {	/*only the first component is processed */
 			struct oval_collection *subcoll = oval_collection_new();
 			struct oval_component *subcomp = oval_component_iterator_next(subcomps);
-			flag = oval_component_evaluate(sysmod, subcomp, subcoll);
+			flag = oval_component_eval_common(argu, subcomp, subcoll);
 			struct oval_value_iterator *values =
 			    (struct oval_value_iterator *)oval_collection_iterator(subcoll);
 			while (oval_value_iterator_has_more(values)) {
@@ -1419,8 +1460,9 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_BEGIN(struct oval
 	return flag;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_END
-    (struct oval_syschar_model *sysmod, struct oval_component *component, struct oval_collection *value_collection) {
+static oval_syschar_collection_flag_t _oval_component_evaluate_END(oval_argu_t *argu,
+								   struct oval_component *component,
+								   struct oval_collection *value_collection) {
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
 	char *suffix = oval_component_get_suffix(component);
 	if (suffix) {
@@ -1429,7 +1471,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_END
 		if (oval_component_iterator_has_more(subcomps)) {	/*only the first component is processed */
 			struct oval_collection *subcoll = oval_collection_new();
 			struct oval_component *subcomp = oval_component_iterator_next(subcomps);
-			flag = oval_component_evaluate(sysmod, subcomp, subcoll);
+			flag = oval_component_eval_common(argu, subcomp, subcoll);
 			struct oval_value_iterator *values =
 			    (struct oval_value_iterator *)oval_collection_iterator(subcoll);
 			while (oval_value_iterator_has_more(values)) {
@@ -1455,7 +1497,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_END
 	return flag;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(oval_argu_t *argu,
 								      struct oval_component *component,
 								      struct oval_collection *value_collection)
 {
@@ -1469,7 +1511,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(struct ova
 	for (idx0 = 0; oval_component_iterator_has_more(subcomps); idx0++) {
 		struct oval_collection *subcoll = oval_collection_new();
 		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
-		oval_syschar_collection_flag_t subflag = oval_component_evaluate(sysmod, subcomp, subcoll);
+		oval_syschar_collection_flag_t subflag = oval_component_eval_common(argu, subcomp, subcoll);
 		flag = _AGG_FLAG(flag, subflag);
 		component_colls[idx0] = subcoll;
 	}
@@ -1542,7 +1584,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_CONCAT(struct ova
 	return flag;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_SPLIT(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_SPLIT(oval_argu_t *argu,
 								     struct oval_component *component,
 								     struct oval_collection *value_collection)
 {
@@ -1554,7 +1596,7 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_SPLIT(struct oval
 	if (oval_component_iterator_has_more(subcomps)) {	/* Only first component is considered */
 		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
 		struct oval_collection *subcoll = oval_collection_new();
-		flag = oval_component_evaluate(sysmod, subcomp, subcoll);
+		flag = oval_component_eval_common(argu, subcomp, subcoll);
 		struct oval_value_iterator *values = (struct oval_value_iterator *)oval_collection_iterator(subcoll);
 		struct oval_value *value;
 		while (oval_value_iterator_has_more(values)) {
@@ -1590,36 +1632,54 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_SPLIT(struct oval
 	return flag;
 };
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_SUBSTRING(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_SUBSTRING(oval_argu_t *argu,
 									 struct oval_component *component,
 									 struct oval_collection *value_collection)
 {
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
 	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
 	int start = oval_component_get_substring_start(component) - 1;
-	int len = oval_component_get_substring_length(component);
-	start = (start < 0) ? 0 : start;
+	size_t len = (size_t)oval_component_get_substring_length(component);
+        size_t beg;
+
+	beg = (start < 0) ? 0 : (size_t)start;
+
 	if (oval_component_iterator_has_more(subcomps)) {	/*Only first component is considered */
 		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
 		struct oval_collection *subcoll = oval_collection_new();
-		flag = oval_component_evaluate(sysmod, subcomp, subcoll);
+		flag = oval_component_eval_common(argu, subcomp, subcoll);
 		struct oval_value_iterator *values = (struct oval_value_iterator *)oval_collection_iterator(subcoll);
 		struct oval_value *value;
+
 		while (oval_value_iterator_has_more(values)) {
 			char *text = oval_value_get_text(oval_value_iterator_next(values));
 			char substr[len + 1];
-			text += (start < (int)strlen(text)) ? start : (int)strlen(text);
-			strncpy(substr, text, len);
+                        size_t txtlen, sublen;
+
+                        substr[0] = '\0';
+                        txtlen    = strlen(text);
+
+                        if (beg < txtlen) {
+                                sublen = (beg + len) >= txtlen ? 0 : txtlen - (beg + len);
+
+                                if (sublen < len)
+                                        len = sublen;
+
+                                strncpy(substr, text + beg, len);
+                                substr[len] = '\0';
+                        }
+
 			value = oval_value_new(OVAL_DATATYPE_STRING, substr);
 			oval_collection_add(value_collection, value);
 		}
 		oval_value_iterator_free(values);
 		oval_collection_free_items(subcoll, (oscap_destruct_func) oval_value_free);
 	}
+	oval_component_iterator_free(subcomps);
 	return flag;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_TIMEDIF(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_TIMEDIF(oval_argu_t *argu,
 								       struct oval_component *component,
 								       struct oval_collection *value_collection)
 {
@@ -1628,62 +1688,146 @@ static oval_syschar_collection_flag_t _oval_component_evaluate_TIMEDIF(struct ov
 
 static bool _isEscape(char chr)
 {
-	return false;		/*TODO: implement this function */
+	const char *regex_chars = "^$\\.[](){}*+?|";
+
+	return (strchr(regex_chars, chr) == NULL) ? false : true;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_ESCAPE_REGEX(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_ESCAPE_REGEX(oval_argu_t *argu,
 									    struct oval_component *component,
 									    struct oval_collection *value_collection)
 {
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
 	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
-	int start = oval_component_get_substring_start(component) - 1;
-	/* int len = oval_component_get_substring_length(component); */
-	start = (start < 0) ? 0 : start;
+
 	if (oval_component_iterator_has_more(subcomps)) {	//Only first component is considered
 		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
 		struct oval_collection *subcoll = oval_collection_new();
-		flag = oval_component_evaluate(sysmod, subcomp, subcoll);
+		flag = oval_component_eval_common(argu, subcomp, subcoll);
 		struct oval_value_iterator *values = (struct oval_value_iterator *)oval_collection_iterator(subcoll);
-		struct oval_value *value;
 		while (oval_value_iterator_has_more(values)) {
-			char *text = oval_value_get_text(oval_value_iterator_next(values));
-			/* TODO: this len shadows the above one in comment, which one is right ? */
+			struct oval_value *value = oval_value_iterator_next(values);
+			char *text = oval_value_get_text(value);
 			int len = strlen(text);
 			char string[2 * len + 1], *insert = string;
 			while (*text) {
-				if (_isEscape(*text)) {
-					*insert = '\\';
-					insert += 1;
-				}
-				*insert = *text;
-				insert += 1;
+				if (_isEscape(*text))
+					*insert++ = '\\';
+				*insert++ = *text++;
 			}
+			*insert = '\0';
 			value = oval_value_new(OVAL_DATATYPE_STRING, string);
 			oval_collection_add(value_collection, value);
 		}
 		oval_value_iterator_free(values);
 		oval_collection_free_items(subcoll, (oscap_destruct_func) oval_value_free);
 	}
+	oval_component_iterator_free(subcomps);
 	return flag;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE
-    (struct oval_syschar_model *sysmod, struct oval_component *component, struct oval_collection *value_collection) {
-	return SYSCHAR_FLAG_UNKNOWN;
-	//TODO: Missing implementation   
+static oval_syschar_collection_flag_t _oval_component_evaluate_REGEX_CAPTURE(oval_argu_t *argu,
+									     struct oval_component *component,
+									     struct oval_collection *value_collection)
+{
+	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_UNKNOWN;
+	struct oval_component_iterator *subcomps = oval_component_get_function_components(component);
+	int rc;
+	char *pattern;
+#if defined USE_REGEX_PCRE
+	int erroffset = -1;
+	pcre *re = NULL;
+	const char *error;
+
+	pattern = oval_component_get_regex_pattern(component);
+	re = pcre_compile(pattern, PCRE_UTF8, &error, &erroffset, NULL);
+	if (re == NULL) {
+		oscap_dprintf("ERROR: pcre_compile() failed: \"%s\".\n", error);
+		return SYSCHAR_FLAG_ERROR;
+	}
+#elif defined USE_REGEX_POSIX
+	regex_t re;
+
+	pattern = oval_component_get_regex_pattern(component);
+	if ((rc = regcomp(&re, pattern, REG_EXTENDED | REG_NEWLINE)) != 0) {
+		oscap_dprintf("ERROR: regcomp() failed: %d.\n", rc);
+		return SYSCHAR_FLAG_ERROR;
+	}
+#endif
+
+	if (oval_component_iterator_has_more(subcomps)) {	//Only first component is considered
+		struct oval_component *subcomp = oval_component_iterator_next(subcomps);
+		struct oval_collection *subcoll = oval_collection_new();
+		flag = oval_component_eval_common(argu, subcomp, subcoll);
+		struct oval_value_iterator *values = (struct oval_value_iterator *)oval_collection_iterator(subcoll);
+		while (oval_value_iterator_has_more(values)) {
+			struct oval_value *value = oval_value_iterator_next(values);
+			char *text = oval_value_get_text(value);
+			char *nval;
+#if defined USE_REGEX_PCRE
+			int i, ovector[60], ovector_len = sizeof (ovector) / sizeof (ovector[0]);
+
+			for (i = 0; i < ovector_len; ++i)
+				ovector[i] = -1;
+
+			rc = pcre_exec(re, NULL, text, strlen(text), 0, 0, ovector, ovector_len);
+			if (rc < -1) {
+				oscap_dprintf("ERROR: pcre_exec() failed: %d.\n", rc);
+				flag = SYSCHAR_FLAG_ERROR;
+				break;
+			}
+
+			if (rc > 1 && ovector[2] != -1) {
+				int substr_len = ovector[3] - ovector[2];
+
+				nval = oscap_alloc(substr_len + 1);
+				memcpy(nval, text + ovector[2], substr_len);
+				nval[substr_len] = '\0';
+			} else {
+				nval = NULL;
+			}
+#elif defined USE_REGEX_POSIX
+			regmatch_t pmatch[40];
+			int pmatch_len = sizeof (pmatch) / sizeof (pmatch[0]);
+
+			rc = regexec(&re, text, pmatch_len, pmatch, 0);
+			if (rc != REG_NOMATCH && pmatch[1].rm_so != -1) {
+				int substr_len = pmatch[1].rm_eo - pmatch[1].rm_so;
+
+				nval = oscap_alloc(substr_len + 1);
+				memcpy(nval, text + pmatch[1].rm_so, substr_len);
+				nval[substr_len] = '\0';
+			} else {
+				nval = NULL;
+			}
+#endif
+			flag = SYSCHAR_FLAG_COMPLETE;
+
+			if (nval != NULL) {
+				value = oval_value_new(OVAL_DATATYPE_STRING, nval);
+				oscap_free(nval);
+			} else {
+				value = oval_value_new(OVAL_DATATYPE_STRING, "");
+			}
+			oval_collection_add(value_collection, value);
+		}
+		oval_value_iterator_free(values);
+		oval_collection_free_items(subcoll, (oscap_destruct_func) oval_value_free);
+	}
+	oval_component_iterator_free(subcomps);
+	return flag;
 }
 
-static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(struct oval_syschar_model *sysmod,
+static oval_syschar_collection_flag_t _oval_component_evaluate_ARITHMETIC(oval_argu_t *argu,
 									  struct oval_component *component,
 									  struct oval_collection *value_collection)
 {
 	return SYSCHAR_FLAG_UNKNOWN;
-	//TODO: Missing implementation   
+	//TODO: Missing implementation
 }
 
 typedef oval_syschar_collection_flag_t(_oval_component_evaluator)
- (struct oval_syschar_model *, struct oval_component *, struct oval_collection *);
+ (oval_argu_t *, struct oval_component *, struct oval_collection *);
 
 static _oval_component_evaluator *_component_evaluators[15] = {
 	NULL,
@@ -1703,9 +1847,9 @@ static _oval_component_evaluator *_component_evaluators[15] = {
 	NULL
 };
 
-oval_syschar_collection_flag_t oval_component_evaluate(struct oval_syschar_model *sysmod,
-						       struct oval_component *component,
-						       struct oval_collection *value_collection)
+static oval_syschar_collection_flag_t oval_component_eval_common(oval_argu_t *argu,
+								 struct oval_component *component,
+								 struct oval_collection *value_collection)
 {
 	__attribute__nonnull__(component);
 
@@ -1715,9 +1859,34 @@ oval_syschar_collection_flag_t oval_component_evaluate(struct oval_syschar_model
 	    ? _component_evaluators[evidx] : NULL;
 	oval_syschar_collection_flag_t flag = SYSCHAR_FLAG_ERROR;
 	if (evaluator) {
-		flag = (*evaluator) (sysmod, component, value_collection);
-	} else
-		oscap_dprintf("ERROR component type %d not supported (%s:%d)", __FILE__, __LINE__);
-	oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Component type not supported");
+		flag = (*evaluator) (argu, component, value_collection);
+	} else {
+		oscap_dprintf("ERROR component type %d not supported.\n", type);
+		oscap_seterr(OSCAP_EFAMILY_OVAL, OVAL_EOVALINT, "Component type not supported");
+	}
 	return flag;
 }
+
+oval_syschar_collection_flag_t oval_component_compute(struct oval_syschar_model *sysmod,
+						      struct oval_component *component,
+						      struct oval_collection *value_collection)
+{
+	oval_argu_t argu;
+
+	argu.mode = OVAL_MODE_COMPUTE;
+	argu.u.sysmod = sysmod;
+
+	return oval_component_eval_common(&argu, component, value_collection);
+}
+
+oval_syschar_collection_flag_t oval_component_query(oval_probe_session_t *sess,
+						    struct oval_component *component,
+						    struct oval_collection *value_collection)
+{
+	oval_argu_t argu;
+
+	argu.mode = OVAL_MODE_QUERY;
+	argu.u.sess = sess;
+
+	return oval_component_eval_common(&argu, component, value_collection);
+}
