@@ -35,9 +35,7 @@
 #include "common/util.h"
 #include "common/debug_priv.h"
 
-#define OVAL_ENUMERATION_INVALID (-1)
-
-static const char _invalid[] = "**INVALID**";
+const char _invalid[] = "**INVALID**";
 
 static int oval_enumeration_attr(xmlTextReaderPtr reader, char *attname, const struct oscap_string_map *map, int defval)
 {
@@ -49,15 +47,10 @@ static int oval_enumeration_attr(xmlTextReaderPtr reader, char *attname, const s
 	return ret == OVAL_ENUMERATION_INVALID ? defval : ret;
 }
 
-static const char *oval_enumeration_get_text(const struct oscap_string_map *map, int idx)
+static const char *oval_enumeration_get_text(const struct oscap_string_map *map, int val)
 {
+	return oscap_enum_to_string(map, val);
 
-	if (idx) {
-		return map[idx - 1].string;
-	} else {
-		oscap_dprintf("WARNING: ZERO ENUMERATION INDEX (%s:%d)\n", __FILE__, __LINE__);
-		return _invalid;
-	}
 }
 
 static const struct oscap_string_map OVAL_SYSCHAR_FLAG_MAP[] = {
@@ -266,7 +259,7 @@ static const struct oscap_string_map OVAL_OPERATOR_MAP[] = {
 	{OVAL_OPERATOR_ONE, "ONE"},
 	{OVAL_OPERATOR_OR, "OR"},
 	{OVAL_OPERATOR_XOR, "XOR"},
-	{-1, NULL}
+	{OVAL_ENUMERATION_INVALID, NULL}
 };
 
 oval_operator_t oval_operator_parse(xmlTextReaderPtr reader, char *attname, oval_operator_t defval)
@@ -348,8 +341,7 @@ oval_family_t oval_family_parse(xmlTextReaderPtr reader)
 
 const char *oval_family_get_text(oval_family_t family)
 {
-	int family_idx = family / 1000;
-	return oval_enumeration_get_text(OVAL_FAMILY_MAP, family_idx);
+	return oval_enumeration_get_text(OVAL_FAMILY_MAP, family);
 }
 
 static const struct oscap_string_map OVAL_SUBTYPE_AIX_MAP[] = {
@@ -623,8 +615,7 @@ const char *oval_subtype_get_text(oval_subtype_t subtype)
 	}
 
 	if (map) {
-		int subidx = subtype % 1000;
-		return oval_enumeration_get_text(map, subidx);
+		return oval_enumeration_get_text(map, subtype);
 	} else {
 		fprintf(stderr, "WARNING: ZERO FAMILY INDEX\n    %s(%d)\n", __FILE__, __LINE__);
 		return _invalid;
@@ -651,83 +642,4 @@ const char *oval_result_get_text(oval_result_t result)
 	return oval_enumeration_get_text(OVAL_RESULT_MAP, result);
 }
 
-static const struct oscap_string_map *maps[] = {
-	OVAL_SYSCHAR_FLAG_MAP,
-	OVAL_SYSCHAR_STATUS_MAP,
-	OVAL_MESSAGE_LEVEL_MAP,
-	OVAL_ARITHMETIC_OPERATION_MAP,
-	OVAL_DATETIME_FORMAT_MAP,
-	OVAL_SET_OPERATION_MAP,
-	OVAL_OPERATION_MAP,
-	OVAL_CHECK_MAP,
-	OVAL_DATATYPE_MAP,
-	OVAL_EXISTENCE_MAP,
-	OVAL_OPERATOR_MAP,
-	OVAL_FAMILY_MAP,
-	OVAL_SUBTYPE_AIX_MAP,
-	OVAL_SUBTYPE_APACHE_MAP,
-	OVAL_SUBTYPE_CATOS_MAP,
-	OVAL_SUBTYPE_ESX_MAP,
-	OVAL_SUBTYPE_FREEBSD_MAP,
-	OVAL_SUBTYPE_HPUX_MAP,
-	OVAL_SUBTYPE_INDEPENDENT_MAP,
-	OVAL_SUBTYPE_IOS_MAP,
-	OVAL_SUBTYPE_LINUX_MAP,
-	OVAL_SUBTYPE_MACOS_MAP,
-	OVAL_SUBTYPE_PIXOS_MAP,
-	OVAL_SUBTYPE_SOLARIS_MAP,
-	OVAL_SUBTYPE_UNIX_MAP,
-	OVAL_SUBTYPE_WINDOWS_MAP,
-	OVAL_RESULT_MAP,
-	NULL
-};
 
-typedef const char *(*_textfunc) (int idx);
-
-static _textfunc textfuncs[] = {
-	(_textfunc) oval_syschar_collection_flag_get_text,
-	(_textfunc) oval_syschar_status_get_text,
-	(_textfunc) oval_message_level_text,
-	(_textfunc) oval_arithmetic_operation_get_text,
-	(_textfunc) oval_datetime_format_get_text,
-	(_textfunc) oval_set_operation_get_text,
-	(_textfunc) oval_operation_get_text,
-	(_textfunc) oval_check_get_text,
-	(_textfunc) oval_datatype_get_text,
-	(_textfunc) oval_existence_get_text,
-	(_textfunc) oval_operator_get_text,
-	(_textfunc) oval_family_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_subtype_get_text,
-	(_textfunc) oval_result_get_text,
-	NULL
-};
-
-/*void oval_enumerations_test_driver(){
-
-	int i;
-        for(i=0; maps[i]; i++){
-		oscap_dprintf("Testing enumeration #%d", i);
-		const struct oscap_string_map *map = maps[i];
-		_textfunc textfunc = textfuncs[i];
-		oscap_dprintf("    INVALID INDEX: %s", (*textfunc)(0));
-		int j;
-                for(j=0;map[j].string;j++){
-			int value = map[j].value;
-			const char *text = (*textfunc)(value);
-			oscap_dprintf("    [%d]: %s", value, text);
-		}
-	}
-}*/
