@@ -31,6 +31,7 @@
 #include "common/util.h"
 #include "common/bfind.h"
 #include "common/debug_priv.h"
+#include "probes/public/probe-api.h"
 #include "oval_probe_ext.h"
 #include "oval_sexp.h"
 
@@ -225,7 +226,7 @@ static SEXP_t *oval_probe_cmd_obj_eval(SEXP_t *sexp, void *arg)
 	}
 
 	oscap_clearerr();
-	oval_probe_object_eval(pext->sess_ptr, obj, OVAL_PDFLAG_NOREPLY); /* DO NOT PUT THIS INSIDE ASSERT! */
+	oval_probe_object_query(pext->sess_ptr, obj, OVAL_PDFLAG_NOREPLY); /* DO NOT PUT THIS INSIDE ASSERT! */
 
 	if (oscap_err()) {
 		oscap_dprintf("FAIL: obj eval failed: id=%s, err: %d, %d, %s\n",
@@ -238,7 +239,7 @@ static SEXP_t *oval_probe_cmd_obj_eval(SEXP_t *sexp, void *arg)
 
 	oscap_free(id_str);
 
-	return (sexp);
+	return SEXP_ref(sexp);
 }
 
 static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
@@ -268,6 +269,7 @@ static SEXP_t *oval_probe_cmd_ste_fetch(SEXP_t *sexp, void *arg)
 
 			ste_sexp = oval_state2sexp(ste);
 			SEXP_list_add(ste_list, ste_sexp);
+                        SEXP_free(ste_sexp);
 
 			oscap_free(id_str);
 		}
@@ -704,7 +706,7 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
                                 pd = oval_pdtbl_get(pext->pdtbl, oval_object_get_subtype(obj));
                 }
 
-                *sys = oval_probe_ext_eval(pext->pdtbl->ctx, pd, *(pext->model), obj, flags);
+                *sys = oval_probe_ext_eval(pext->pdtbl->ctx, pd, pext, obj, flags);
                 ret  = (*sys == NULL ? -1 : 0);
                 break;
         }
@@ -713,8 +715,41 @@ int oval_probe_ext_handler(oval_subtype_t type, void *ptr, int act, ...)
         case PROBE_HANDLER_ACT_INIT:
                 ret = oval_probe_ext_init(pext);
                 break;
-        case PROBE_HANDLER_ACT_FREE:
         case PROBE_HANDLER_ACT_RESET:
+        {
+                size_t i;
+
+                if (type == OVAL_SUBTYPE_ALL) {
+                        /*
+                         * Iterate trhu probe descriptor table and execute the reset operation
+                         * for each probe descriptor.
+                         */
+                        for (i = 0; i < pext->pdtbl->count; ++i) {
+                                pd  = pext->pdtbl->memb + i;
+
+                                //fprintf(stderr, "Sending reset to %p(%s)...\n", pd, oval_subtype2str(pd->subtype));
+
+                                ret = oval_probe_ext_reset(pext->pdtbl->ctx, pd, pext);
+
+                                if (ret != 0)
+                                        return(ret);
+                        }
+
+                        return(0);
+                } else {
+                        /*
+                         * Reset only the probe of specified subtype.
+                         */
+                        pd = oval_pdtbl_get(pext->pdtbl, type);
+
+                        if (pd == NULL)
+                                return(0);
+
+                        return oval_probe_ext_reset(pext->pdtbl->ctx, pd, pext);
+                }
+                break;
+        }
+        case PROBE_HANDLER_ACT_FREE:
         case PROBE_HANDLER_ACT_CLOSE:
         default:
                 va_end(ap);
@@ -746,12 +781,13 @@ int oval_probe_ext_init(oval_pext_t *pext)
         return(ret);
 }
 
-struct oval_syschar *oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, struct oval_syschar_model *model, struct oval_object *object, int flags)
+struct oval_syschar *oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext, struct oval_object *object, int flags)
 {
         SEXP_t *s_obj, *s_sys;
         struct oval_syschar *o_sys;
+        struct oval_syschar_model *model = *(pext->model);
 
-        s_obj = oval_object2sexp(oval_subtype2str(oval_object_get_subtype(object)), object, model);
+        s_obj = oval_object2sexp(oval_subtype2str(oval_object_get_subtype(object)), object, model, pext->sess_ptr);
 
         if (s_obj == NULL) {
                 oscap_dprintf("Can't translate OVAL object to S-exp\n");
@@ -807,3 +843,12 @@ struct oval_syschar *oval_probe_ext_eval(SEAP_CTX_t *ctx, oval_pd_t *pd, struct
 
 	return (o_sys);
 }
+
+int oval_probe_ext_reset(SEAP_CTX_t *ctx, oval_pd_t *pd, oval_pext_t *pext)
+{
+        void *res;
+
+        res = SEAP_cmd_exec(ctx, pd->sd, SEAP_EXEC_RECV, PROBECMD_RESET, NULL, SEAP_CMDTYPE_SYNC, NULL, NULL);
+
+        return (0);
+}
