@@ -118,7 +118,7 @@ struct oval_result_criteria_node *oval_result_criteria_node_new
 	}
 	node->sys = sys;
 	node->negate = negate;
-	node->result = OVAL_RESULT_INVALID;
+	node->result = OVAL_RESULT_NOT_EVALUATED;
 	node->type = type;
 	va_end(ap);
 	return node;
@@ -129,8 +129,10 @@ bool oval_result_criteria_node_is_valid(struct oval_result_criteria_node * resul
 	bool is_valid = true;
 	oval_criteria_node_type_t type;
 
-	if (result_criteria_node == NULL)
+	if (result_criteria_node == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
 		return false;
+        }
 
 	type = oval_result_criteria_node_get_type(result_criteria_node);
 	switch (type) {
@@ -175,6 +177,7 @@ bool oval_result_criteria_node_is_valid(struct oval_result_criteria_node * resul
 		}
 		break;
 	default:
+                oscap_dprintf("WARNING: argument is not valid: wrong node type: %d.\n", type);
 		return false;
 	}
 
@@ -285,13 +288,13 @@ struct oval_result_criteria_node *make_result_criteria_node_from_oval_criteria_n
 		case OVAL_NODETYPE_CRITERION:{
 				struct oval_test *oval_test = oval_criteria_node_get_test(oval_node);
 				struct oval_result_test *rslt_test = get_oval_result_test_new(sys, oval_test);
-				rslt_node = oval_result_criteria_node_new(sys, type, negate, rslt_test, 0);
+				rslt_node = oval_result_criteria_node_new(sys, type, negate, rslt_test, 1);
 			} break;
 		case OVAL_NODETYPE_EXTENDDEF:{
 				struct oval_definition *oval_definition = oval_criteria_node_get_definition(oval_node);
 				struct oval_result_definition *rslt_definition
 				    = oval_result_system_get_new_definition(sys, oval_definition);
-				rslt_node = oval_result_criteria_node_new(sys, type, negate, rslt_definition, 0);
+				rslt_node = oval_result_criteria_node_new(sys, type, negate, rslt_definition, 1);
 			} break;
 		default:
 			rslt_node = NULL;
@@ -339,89 +342,27 @@ static oval_result_t _oval_result_negate(bool negate, oval_result_t result)
 	    (negate && result == OVAL_RESULT_FALSE) ? OVAL_RESULT_TRUE : result;
 }
 
-#define INVALID   (counts[0])
-#define _CT  (counts[1])
-#define _CF  (counts[2])
-#define _CU  (counts[3])
-#define _CE  (counts[4])
-#define _CNE (counts[5])
-#define _CNA (counts[6])
-
-// todo: already implemented elsewhere; consolidate
-static oval_result_t _oval_result_binary_op(oval_operator_t operator, int *counts) {
-	oval_result_t result = OVAL_RESULT_INVALID;
-	if (INVALID == 0) {
-		switch (operator) {
-		case OVAL_OPERATOR_AND:{
-				result =
-				    (_CT > 0 && _CF == 0 && _CU == 0 && _CE == 0 && _CNE == 0) ? OVAL_RESULT_TRUE :
-				    (_CF > 0) ? OVAL_RESULT_FALSE :
-				    (_CF == 0 && _CE > 0) ? OVAL_RESULT_ERROR :
-				    (_CF == 0 && _CE == 0 && _CU > 0) ? OVAL_RESULT_UNKNOWN :
-				    (_CF == 0 && _CE == 0 && _CU == 0 && _CNE > 0) ? OVAL_RESULT_NOT_EVALUATED :
-				    (_CT == 0 && _CF == 0 && _CE == 0 && _CU == 0 && _CNE == 0
-				     && _CNA > 0) ? OVAL_RESULT_NOT_APPLICABLE : OVAL_RESULT_INVALID;
-			};
-			break;
-		case OVAL_OPERATOR_ONE:{
-				result =
-				    (_CT == 1 && _CE == 0 && _CU == 0 && _CNE == 0) ? OVAL_RESULT_TRUE :
-				    (_CT > 1) ? OVAL_RESULT_FALSE :
-				    (_CT == 0 && _CF > 0 && _CE == 0 && _CU == 0 && _CNE == 0) ? OVAL_RESULT_FALSE :
-				    (_CT < 2 && _CE > 0) ? OVAL_RESULT_ERROR :
-				    (_CT < 2 && _CE == 0 && _CU > 0) ? OVAL_RESULT_UNKNOWN :
-				    (_CT < 2 && _CE == 0 && _CU == 0 && _CNE > 0) ? OVAL_RESULT_NOT_EVALUATED :
-				    (_CT == 0 && _CF == 0 && _CE == 0 && _CU == 0 && _CNE == 0
-				     && _CNA > 0) ? OVAL_RESULT_NOT_APPLICABLE : OVAL_RESULT_INVALID;
-			};
-			break;
-		case OVAL_OPERATOR_OR:{
-				result =
-				    (_CT > 0) ? OVAL_RESULT_TRUE :
-				    (_CT == 0 && _CF > 0 && _CE == 0 && _CU == 0 && _CNE == 0) ? OVAL_RESULT_FALSE :
-				    (_CT == 0 && _CE > 0) ? OVAL_RESULT_ERROR :
-				    (_CT == 0 && _CE == 0 && _CU > 0) ? OVAL_RESULT_UNKNOWN :
-				    (_CT == 0 && _CE == 0 && _CU == 0 && _CNE > 0) ? OVAL_RESULT_NOT_EVALUATED :
-				    (_CT == 0 && _CF == 0 && _CE == 0 && _CU == 0 && _CNE == 0
-				     && _CNA > 0) ? OVAL_RESULT_NOT_APPLICABLE : OVAL_RESULT_INVALID;
-			};
-			break;
-		case OVAL_OPERATOR_XOR:{
-				result =
-				    (_CT % 2 == 1 && _CE == 0 && _CU == 0 && _CNE == 0) ? OVAL_RESULT_TRUE :
-				    (_CT % 2 == 0 && _CE == 0 && _CU == 0 && _CNE == 0) ? OVAL_RESULT_FALSE :
-				    (_CE > 0) ? OVAL_RESULT_ERROR :
-				    (_CE == 0 && _CU > 0) ? OVAL_RESULT_UNKNOWN :
-				    (_CE == 0 && _CU == 0 && _CNE > 0) ? OVAL_RESULT_NOT_EVALUATED :
-				    (_CT == 0 && _CF == 0 && _CE == 0 && _CU == 0 && _CNE == 0
-				     && _CNA > 0) ? OVAL_RESULT_NOT_APPLICABLE : OVAL_RESULT_INVALID;
-			};
-			break;
-		default:
-			break;
-		}
-	}
-	return result;
-}
 
 static oval_result_t _oval_result_criteria_node_result(struct oval_result_criteria_node *node) {
 	__attribute__nonnull__(node);
 
-	oval_result_t result = OVAL_RESULT_INVALID;
+	oval_result_t result;
 	switch (node->type) {
 	case OVAL_NODETYPE_CRITERIA:{
 			struct oval_result_criteria_node_iterator *subnodes
 			    = oval_result_criteria_node_get_subnodes(node);
 			oval_operator_t operator = oval_result_criteria_node_get_operator(node);
-			int counts[] = { 0, 0, 0, 0, 0, 0, 0 };	//result counts
+			struct oresults node_res;
+			ores_clear(&node_res);
 			while (oval_result_criteria_node_iterator_has_more(subnodes)) {
 				struct oval_result_criteria_node *subnode
 				    = oval_result_criteria_node_iterator_next(subnodes);
 				oval_result_t subres = oval_result_criteria_node_eval(subnode);
-				counts[subres]++;
+				ores_add_res(&node_res, subres);
 			}
 			oval_result_criteria_node_iterator_free(subnodes);
-			result = _oval_result_binary_op(operator, counts);
+			result = ores_get_result_byopr(&node_res, operator);
+
 		} break;
 	case OVAL_NODETYPE_CRITERION:{
 			struct oval_result_test *test = oval_result_criteria_node_get_test(node);
@@ -435,8 +376,7 @@ static oval_result_t _oval_result_criteria_node_result(struct oval_result_criter
 		break;
 	}
 
-	if ( result != OVAL_RESULT_INVALID ) 
-		result =  _oval_result_negate(node->negate, result);
+	result = _oval_result_negate(node->negate, result);
 
 	return result;
 }
@@ -445,7 +385,7 @@ oval_result_t oval_result_criteria_node_eval(struct oval_result_criteria_node *
 {
 	__attribute__nonnull__(node);
 
-	if (node->result == OVAL_RESULT_INVALID) {
+	if (node->result == OVAL_RESULT_NOT_EVALUATED) {
 		node->result = _oval_result_criteria_node_result(node);
 	}
 	return node->result;
@@ -672,6 +612,13 @@ static xmlNode *_oval_result_CRITERION_to_dom(struct oval_result_criteria_node *
 	snprintf(version, sizeof(version), "%d", oval_test_get_version(oval_test));
 	xmlNewProp(node_root, BAD_CAST "version", BAD_CAST version);
 
+        int instance = ((struct oval_result_criteria_node_CRITERION *) node)->variable_instance;
+        if (instance != 1) {
+                char instance_att[10] = "";
+                snprintf(instance_att, sizeof (instance_att), "%d", instance);
+                xmlNewProp(node_root, BAD_CAST "variable_instance", BAD_CAST instance_att);
+        }
+
 	return node_root;
 
 }
@@ -685,6 +632,13 @@ static xmlNode *_oval_result_EXTENDDEF_to_dom(struct oval_result_criteria_node *
 	char *definition_ref = oval_definition_get_id(oval_definition);
 	xmlNewProp(node_root, BAD_CAST "definition_ref", BAD_CAST definition_ref);
 
+        int instance = ((struct oval_result_criteria_node_EXTENDDEF *) node)->variable_instance;
+        if (instance != 1) {
+                char instance_att[10] = "";
+                snprintf(instance_att, sizeof (instance_att), "%d", instance);
+                xmlNewProp(node_root, BAD_CAST "variable_instance", BAD_CAST instance_att);
+        }
+
 	return node_root;
 
 }
