@@ -45,9 +45,6 @@ typedef struct oval_result_definition {
 	int instance;
 } oval_result_definition_t;
 
-static char *_oval_result_definition_results[] = {
-	NULL, "true", "false", "unknown", "error", "not evaluated", "not applicable"
-};
 
 bool oval_result_definition_iterator_has_more(struct oval_result_definition_iterator *definitions) {
 	return oval_collection_iterator_has_more((struct oval_iterator *)definitions);
@@ -72,7 +69,7 @@ struct oval_result_definition *oval_result_definition_new(struct oval_result_sys
 	struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
 	struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(syschar_model);
 	definition->definition = oval_definition_get_new(definition_model, definition_id);
-	definition->result = OVAL_RESULT_INVALID;
+	definition->result = OVAL_RESULT_NOT_EVALUATED;
 	definition->criteria = NULL;
 	definition->messages = oval_collection_new();
 	definition->instance = 1;
@@ -84,11 +81,15 @@ bool oval_result_definition_is_valid(struct oval_result_definition * result_defi
 	struct oval_definition *definition;
 	struct oval_result_criteria_node *rslt_criteria_node;
 
-	if (result_definition == NULL)
+	if (result_definition == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
 		return false;
+        }
 
-	if (oval_result_definition_get_system(result_definition) == NULL)
+	if (oval_result_definition_get_system(result_definition) == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: system == NULL.\n");
 		return false;
+        }
 
 	/* validate definition */
 	definition = oval_result_definition_get_definition(result_definition);
@@ -147,8 +148,8 @@ void oval_result_definition_free(struct oval_result_definition *definition)
 	definition->criteria = NULL;
 	definition->definition = NULL;
 	definition->messages = NULL;
-	definition->result = 0;
-	definition->instance = 0;
+	definition->result = OVAL_RESULT_NOT_EVALUATED;
+	definition->instance = 1;
 	oscap_free(definition);
 }
 
@@ -157,12 +158,10 @@ struct oval_result_definition *make_result_definition_from_oval_definition
 	char *defid = oval_definition_get_id(oval_definition);
 	struct oval_result_definition *rslt_definition = oval_result_definition_new(sys, defid);
 	struct oval_criteria_node *oval_criteria = oval_definition_get_criteria(oval_definition);
-	struct oval_result_criteria_node *rslt_criteria
-	    = make_result_criteria_node_from_oval_criteria_node(sys, oval_criteria);
+	struct oval_result_criteria_node *rslt_criteria = 
+		make_result_criteria_node_from_oval_criteria_node(sys, oval_criteria);
 	if (rslt_criteria)
 		oval_result_definition_set_criteria(rslt_definition, rslt_criteria);
-	else
-		oval_result_definition_set_result(rslt_definition, OVAL_RESULT_NOT_EVALUATED);
 	return rslt_definition;
 }
 
@@ -188,11 +187,11 @@ oval_result_t oval_result_definition_eval(struct oval_result_definition * defini
 {
 	__attribute__nonnull__(definition);
 
-	if (definition->result == OVAL_RESULT_INVALID) {
+	if (definition->result == OVAL_RESULT_NOT_EVALUATED) {
 		struct oval_result_criteria_node *criteria = oval_result_definition_get_criteria(definition);
 
 		definition->result = (criteria == NULL)
-		    ? OVAL_RESULT_NOT_EVALUATED : oval_result_criteria_node_eval(criteria);
+		    ? OVAL_RESULT_ERROR : oval_result_criteria_node_eval(criteria);
 	}
 	return definition->result;
 }
@@ -288,7 +287,9 @@ int oval_result_definition_parse
 	xmlChar *definition_id = xmlTextReaderGetAttribute(reader, BAD_CAST "definition_id");
 	xmlChar *definition_version = xmlTextReaderGetAttribute(reader, BAD_CAST "version");
 	int resvsn = atoi((char *)definition_version);
-	xmlChar *definition_result = xmlTextReaderGetAttribute(reader, BAD_CAST "result");
+
+	oval_result_t result = oval_result_parse(reader, "result", OVAL_ENUMERATION_INVALID);
+
 	int instance = oval_parser_int_attribute(reader, "variable_instance", 1);
 
 	struct oval_result_definition *definition = oval_result_definition_new(sys, (char *)definition_id);
@@ -303,28 +304,26 @@ int oval_result_definition_parse
 	}
 	oval_definition_set_version(definition->definition, resvsn);
 	oval_result_definition_set_instance(definition, instance);
-	oval_result_t result = 0;
-	int i;
-	for (i = 1; i < 7 && result == 0; i++) {
-		result = strcmp(_oval_result_definition_results[i], (const char *)definition_result) == 0 ? i : 0;
-	}
-	if (result) {
+	
+
+	if ((int)result != OVAL_ENUMERATION_INVALID) {
 		oval_result_definition_set_result(definition, result);
 	} else {
 		oscap_dprintf("WARNING: oval_result_definition_parse: can't resolve result attribute\n"
-			      "    definition id = %s\n" "    result attribute = %s", definition_id, definition_result);
+			      "    definition id = %s\n", definition_id);
+		oval_result_definition_set_result(definition, OVAL_RESULT_UNKNOWN);
 	}
 
 	//Process tag contents
 	oscap_dprintf("DEBUG: oval_result_definition_parse: processing <definition> contents\n"
 		      "    definition id = %s vsn = %d\n"
-		      "    definition result = %s(%d)", definition_id, defvsn, definition_result, result);
+		      "    definition result = (%d)", definition_id, defvsn, result);
+
 	return_code = oval_parser_parse_tag
 	    (reader, context, (oval_xml_tag_parser) _oval_result_definition_parse, definition);
 
 	oscap_free(definition_id);
 	oscap_free(definition_version);
-	oscap_free(definition_result);
 
 	(*consumer) (definition, client);
 	oscap_dprintf("DEBUG: oval_result_definition_parse: END");
