@@ -53,7 +53,7 @@ struct oval_result_item *oval_result_item_new(struct oval_result_system *sys, ch
 
 	item->sysdata = sysdata;
 	item->messages = oval_collection_new();
-	item->result = 0;
+	item->result = OVAL_RESULT_NOT_EVALUATED;
 	item->sys = sys;
 
 	return item;
@@ -63,8 +63,10 @@ bool oval_result_item_is_valid(struct oval_result_item * result_item)
 {
 	struct oval_sysdata *sysdata;
 
-	if (result_item == NULL)
+	if (result_item == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
 		return false;
+        }
 
 	sysdata = oval_result_item_get_sysdata(result_item);
 	if (oval_sysdata_is_valid(sysdata) != true)
@@ -106,7 +108,7 @@ void oval_result_item_free(struct oval_result_item *item)
 	oval_collection_free_items(item->messages, (oscap_destruct_func) oval_message_free);
 
 	item->messages = NULL;
-	item->result = 0;
+	item->result = OVAL_RESULT_NOT_EVALUATED;
 	item->sysdata = NULL;
 
 	oscap_free(item);
