@@ -48,6 +48,50 @@ typedef struct oval_result_system {
 	bool definitions_initialized;
 } oval_result_system_t;
 
+
+static void _oval_result_system_scan_criteria_for_references(struct oval_result_criteria_node *node, 
+							     struct oval_string_map *testmap);
+
+static void _oval_result_system_scan_entity_for_references(struct oval_syschar_model *syschar_model, 
+							   struct oval_entity *entity,
+							   struct oval_string_map *objmap,
+							   struct oval_string_map *sttmap, 
+							   struct oval_string_map *varmap, 
+							   struct oval_string_map *sysmap);
+
+static void _oval_result_system_scan_set_for_references(struct oval_syschar_model *syschar_model, 
+							struct oval_setobject *set,
+							struct oval_string_map *objmap,
+							struct oval_string_map *sttmap, 
+							struct oval_string_map *varmap, 
+							struct oval_string_map *sysmap);
+
+static void _oval_result_system_scan_object_for_references(struct oval_syschar_model *syschar_model, 
+							struct oval_object *object,
+							struct oval_string_map *objmap,
+							struct oval_string_map *sttmap, 
+							struct oval_string_map *varmap, 
+							struct oval_string_map *sysmap);
+static void _oval_result_system_scan_state_for_references(struct oval_syschar_model *syschar_model, 
+							struct oval_state *state,
+							struct oval_string_map *objmap,
+							struct oval_string_map *sttmap, 
+							struct oval_string_map *varmap, 
+							struct oval_string_map *sysmap);
+
+static void _oval_result_system_scan_component_for_references(struct oval_syschar_model *syschar_model, 
+							struct oval_component *component,	
+							struct oval_string_map *objmap,
+							struct oval_string_map *sttmap, 
+							struct oval_string_map *varmap, 
+							struct oval_string_map *sysmap);
+
+static bool _oval_result_system_resolve_syschar(struct oval_syschar *syschar, struct oval_string_map *sysmap);
+
+
+
+
+
 struct oval_result_system *oval_result_system_new(struct oval_results_model *model,
 						  struct oval_syschar_model *syschar_model)
 {
@@ -70,8 +114,10 @@ bool oval_result_system_is_valid(struct oval_result_system * result_system)
 	struct oval_result_test_iterator *rslt_tests_itr;
 	struct oval_syschar_model *syschar_model;
 
-	if (result_system == NULL)
+	if (result_system == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
 		return false;
+        }
 
 	/* validate syschar_model */
 	syschar_model = oval_result_system_get_syschar_model(result_system);
@@ -182,8 +228,7 @@ static void _oval_result_system_initialize(struct oval_result_system *sys) {
 	struct oval_definition_model *definition_model = oval_syschar_model_get_definition_model(sys->syschar_model);
 
 	struct oval_definition_iterator *oval_definitions = oval_definition_model_get_definitions(definition_model);
-	int i;
-	for (i = 0; oval_definition_iterator_has_more(oval_definitions); i++) {
+	while (oval_definition_iterator_has_more(oval_definitions)) {
 		struct oval_definition *oval_definition = oval_definition_iterator_next(oval_definitions);
 		oval_result_system_get_new_definition(sys, oval_definition);
 	}
@@ -219,7 +264,7 @@ struct oval_result_test_iterator *oval_result_system_get_tests(struct oval_resul
 	return iterator;
 }
 
-struct oval_result_definition *oval_result_system_get_definition(struct oval_result_system *sys, char *id) {
+struct oval_result_definition *oval_result_system_get_definition(struct oval_result_system *sys, const char *id) {
 	__attribute__nonnull__(sys);
 
 	if (!sys->definitions_initialized) {
@@ -367,6 +412,154 @@ int oval_result_system_parse
 	return return_code;
 }
 
+int oval_result_system_eval(struct oval_result_system *sys)
+{
+	struct oval_results_model *res_model;
+	struct oval_definition_model *definition_model;
+	struct oval_definition_iterator *definitions_itr;
+
+	res_model = oval_result_system_get_results_model(sys);
+	definition_model = oval_results_model_get_definition_model(res_model);
+	definitions_itr = oval_definition_model_get_definitions(definition_model);
+
+	while (oval_definition_iterator_has_more(definitions_itr)) {
+		struct oval_definition *definition;
+		struct oval_result_definition *rslt_definition;
+
+		definition = oval_definition_iterator_next(definitions_itr);
+		rslt_definition = oval_result_system_get_new_definition(sys, definition);
+		oval_result_definition_eval(rslt_definition);
+	}
+
+	oval_definition_iterator_free(definitions_itr);
+	return 0;
+}
+
+oval_result_t oval_result_system_eval_definition(struct oval_result_system *sys, const char *id)
+{
+        struct oval_results_model *res_model;
+        struct oval_definition_model *definition_model;
+        struct oval_definition *oval_definition;
+	struct oval_result_definition *rslt_definition;
+
+	res_model = oval_result_system_get_results_model(sys);
+	definition_model = oval_results_model_get_definition_model(res_model);
+	oval_definition = oval_definition_model_get_definition(definition_model, id);
+	if (oval_definition == NULL) {
+		oscap_dprintf("%s:%d No definition with ID (%s) in definition model.", __FILE__, __LINE__, id);
+		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "No definition with such an ID in definition model.");
+		return OVAL_RESULT_ERROR;
+	}
+
+        rslt_definition = oval_result_system_get_definition(sys, id);
+        if (rslt_definition == NULL) {
+        	rslt_definition = make_result_definition_from_oval_definition(sys, oval_definition);
+                oval_result_system_add_definition(sys, rslt_definition);
+        }
+
+	return oval_result_definition_eval(rslt_definition);
+}
+
+
+xmlNode *oval_result_system_to_dom
+    (struct oval_result_system * sys,
+     struct oval_results_model * results_model,
+     struct oval_result_directives * directives, xmlDocPtr doc, xmlNode * parent) {
+	xmlNs *ns_results = xmlSearchNsByHref(doc, parent, OVAL_RESULTS_NAMESPACE);
+	xmlNode *system_node = xmlNewChild(parent, ns_results, BAD_CAST "system", NULL);
+
+	struct oval_string_map *tstmap = oval_string_map_new();
+
+	xmlNode *definitions_node = xmlNewChild(system_node, ns_results, BAD_CAST "definitions", NULL);
+	struct oval_definition_model *definition_model = oval_results_model_get_definition_model(results_model);
+	struct oval_definition_iterator *oval_definitions = oval_definition_model_get_definitions(definition_model);
+	while(oval_definition_iterator_has_more(oval_definitions)) {
+		struct oval_definition *oval_definition = oval_definition_iterator_next(oval_definitions);
+		struct oval_result_definition *rslt_definition
+		    = oval_result_system_get_new_definition(sys, oval_definition);
+		if (rslt_definition) {
+			oval_result_t result = oval_result_definition_get_result(rslt_definition);
+			if (oval_result_directives_get_reported(directives, result)) {
+				oval_result_directive_content_t content
+				    = oval_result_directives_get_content(directives, result);
+				/* report definition according to directives settings */
+				oval_result_definition_to_dom(rslt_definition, content, doc, definitions_node);
+				if (content == OVAL_DIRECTIVE_CONTENT_FULL) {
+					struct oval_result_criteria_node *criteria
+					    = oval_result_definition_get_criteria(rslt_definition);
+					/* collect the tests that are referenced from reported definitions */
+					if (criteria)
+						_oval_result_system_scan_criteria_for_references(criteria, tstmap);
+				}
+			}
+		}
+	}
+	oval_definition_iterator_free(oval_definitions);
+
+	struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
+	struct oval_string_map *sysmap = oval_string_map_new();
+	struct oval_string_map *objmap = oval_string_map_new();
+	struct oval_string_map *sttmap = oval_string_map_new();
+	struct oval_string_map *varmap = oval_string_map_new();
+
+	struct oval_result_test_iterator *result_tests = (struct oval_result_test_iterator *)  oval_string_map_values(tstmap);
+	if (oval_result_test_iterator_has_more(result_tests)) {
+		xmlNode *tests_node = xmlNewChild(system_node, ns_results, BAD_CAST "tests", NULL);
+		while (oval_result_test_iterator_has_more(result_tests)) {
+			struct oval_result_test *result_test = oval_result_test_iterator_next(result_tests);
+			/* report the test */
+			oval_result_test_to_dom(result_test, doc, tests_node);
+			struct oval_test *oval_test = oval_result_test_get_test(result_test);
+			/* collect the objects that are referenced from reported test */
+			/* look for objects in path: test->object ...  */
+			struct oval_object *object = oval_test_get_object(oval_test);
+			if (object) {
+				char *objid = oval_object_get_id(object);
+				/* is object already "collected" */
+				void *value = oval_string_map_get_value(objmap, objid);
+				if (value == NULL) {
+					/* is there a system characteristic for the object */
+					struct oval_syschar *syschar =
+					    oval_syschar_model_get_syschar(syschar_model, objid);
+					if (syschar) {
+						/* put object into map */
+						oval_string_map_put(objmap, objid, object);
+						_oval_result_system_scan_object_for_references
+						    (syschar_model, object, objmap, sttmap, varmap, sysmap);
+					}
+				}
+			}
+			/* look for objects in test->state->... */
+			struct oval_state *state = oval_test_get_state(oval_test);
+			if (state) {
+				char *sttid = oval_state_get_id(state);
+				void *value = oval_string_map_get_value(sttmap, sttid);
+				if (value == NULL) {
+					oval_string_map_put(sttmap, sttid, state);
+					_oval_result_system_scan_state_for_references(syschar_model, state, objmap, sttmap,
+										      varmap, sysmap);
+				}
+			}
+		}
+	}
+	oval_result_test_iterator_free(result_tests);
+
+	oval_syschar_model_to_dom(syschar_model, doc, system_node, 
+				  (oval_syschar_resolver *) _oval_result_system_resolve_syschar, sysmap);
+
+	oval_string_map_free(sysmap, NULL);
+	oval_string_map_free(objmap, NULL);
+	oval_string_map_free(sttmap, NULL);
+	oval_string_map_free(varmap, NULL);
+	oval_string_map_free(tstmap, NULL);
+
+	return system_node;
+}
+
+
+
+
+
 static void _oval_result_system_scan_criteria_for_references
     (struct oval_result_criteria_node *node, struct oval_string_map *testmap) {
 	struct oval_result_criteria_node_iterator *subnodes = oval_result_criteria_node_get_subnodes(node);
@@ -386,25 +579,18 @@ static void _oval_result_system_scan_criteria_for_references
 	}
 }
 
-static void _oval_result_system_scan_entity_for_references
-    (struct oval_syschar_model *syschar_model, struct oval_entity *entity,
-     struct oval_string_map *objmap,
-     struct oval_string_map *sttmap, struct oval_string_map *varmap, struct oval_string_map *sysmap);
-
-static void _oval_result_system_scan_set_for_references
-    (struct oval_syschar_model *syschar_model, struct oval_setobject *set,
-     struct oval_string_map *objmap,
-     struct oval_string_map *sttmap, struct oval_string_map *varmap, struct oval_string_map *sysmap);
-
 static void _oval_result_system_scan_object_for_references
     (struct oval_syschar_model *syschar_model, struct oval_object *object,
      struct oval_string_map *objmap,
-     struct oval_string_map *sttmap, struct oval_string_map *varmap, struct oval_string_map *sysmap) {
+     struct oval_string_map *sttmap, 
+     struct oval_string_map *varmap, 
+     struct oval_string_map *sysmap) {
 	char *objid = oval_object_get_id(object);
 	struct oval_syschar *syschar = oval_syschar_model_get_syschar(syschar_model, objid);
-	if (syschar)
+	if (syschar) /* put syschar of the object into map */
 		oval_string_map_put(sysmap, objid, syschar);
 	struct oval_object_content_iterator *contents = oval_object_get_object_contents(object);
+	/* see if there are more objects referenced from this object's content */ 
 	while (oval_object_content_iterator_has_more(contents)) {
 		struct oval_object_content *content = oval_object_content_iterator_next(contents);
 		struct oval_entity *entity = oval_object_content_get_entity(content);
@@ -537,134 +723,3 @@ static bool _oval_result_system_resolve_syschar(struct oval_syschar *syschar, st
 	return oval_string_map_get_value(sysmap, objid) != NULL;
 }
 
-int oval_result_system_eval(struct oval_result_system *sys)
-{
-	struct oval_results_model *res_model;
-	struct oval_definition_model *definition_model;
-	struct oval_definition_iterator *definitions_itr;
-	oval_result_t result;
-
-	res_model = oval_result_system_get_results_model(sys);
-	definition_model = oval_results_model_get_definition_model(res_model);
-	definitions_itr = oval_definition_model_get_definitions(definition_model);
-
-	while (oval_definition_iterator_has_more(definitions_itr)) {
-		struct oval_definition *definition;
-		struct oval_result_definition *rslt_definition;
-
-		definition = oval_definition_iterator_next(definitions_itr);
-		rslt_definition = oval_result_system_get_new_definition(sys, definition);
-		result = oval_result_definition_eval(rslt_definition);
-		if( result == OVAL_RESULT_INVALID ) {
-			oval_definition_iterator_free(definitions_itr);
-			return -1;
-		}
-	}
-
-	oval_definition_iterator_free(definitions_itr);
-	return 0;
-}
-
-oval_result_t oval_result_system_eval_definition(struct oval_result_system *sys, char *id)
-{
-        struct oval_results_model *res_model;
-        struct oval_definition_model *definition_model;
-        struct oval_definition *oval_definition;
-	struct oval_result_definition *rslt_definition;
-
-	res_model = oval_result_system_get_results_model(sys);
-	definition_model = oval_results_model_get_definition_model(res_model);
-	oval_definition = oval_definition_model_get_definition(definition_model, id);
-	if (oval_definition == NULL) {
-		oscap_dprintf("%s:%d No definition with ID (%s) in definition model.", __FILE__, __LINE__, id);
-		oscap_seterr(OSCAP_EFAMILY_OSCAP, OVAL_EOVALINT, "No definition with such an ID in definition model.");
-		return OVAL_RESULT_INVALID;
-	}
- 
-        rslt_definition = oval_result_system_get_definition(sys, id);
-        if (rslt_definition == NULL) {
-        	rslt_definition = make_result_definition_from_oval_definition(sys, oval_definition);
-                oval_result_system_add_definition(sys, rslt_definition);
-        }
-
-	return oval_result_definition_eval(rslt_definition);
-}
-
-
-xmlNode *oval_result_system_to_dom
-    (struct oval_result_system * sys,
-     struct oval_results_model * results_model,
-     struct oval_result_directives * directives, xmlDocPtr doc, xmlNode * parent) {
-	xmlNs *ns_results = xmlSearchNsByHref(doc, parent, OVAL_RESULTS_NAMESPACE);
-	xmlNode *system_node = xmlNewChild(parent, ns_results, BAD_CAST "system", NULL);
-
-	struct oval_string_map *tstmap = oval_string_map_new();
-
-	xmlNode *definitions_node = xmlNewChild(system_node, ns_results, BAD_CAST "definitions", NULL);
-	struct oval_definition_model *definition_model = oval_results_model_get_definition_model(results_model);
-	struct oval_definition_iterator *oval_definitions = oval_definition_model_get_definitions(definition_model);
-	int i;
-	for (i = 0; oval_definition_iterator_has_more(oval_definitions); i++) {
-		struct oval_definition *oval_definition = oval_definition_iterator_next(oval_definitions);
-		struct oval_result_definition *rslt_definition
-		    = oval_result_system_get_new_definition(sys, oval_definition);
-		if (rslt_definition) {
-			oval_result_t result = oval_result_definition_get_result(rslt_definition);
-			if (oval_result_directives_get_reported(directives, result)) {
-				oval_result_directive_content_t content
-				    = oval_result_directives_get_content(directives, result);
-				oval_result_definition_to_dom(rslt_definition, content, doc, definitions_node);
-				if (content == OVAL_DIRECTIVE_CONTENT_FULL) {
-					struct oval_result_criteria_node *criteria
-					    = oval_result_definition_get_criteria(rslt_definition);
-					if (criteria)
-						_oval_result_system_scan_criteria_for_references(criteria, tstmap);
-				}
-			}
-		}
-	}
-	oval_definition_iterator_free(oval_definitions);
-
-	struct oval_syschar_model *syschar_model = oval_result_system_get_syschar_model(sys);
-	struct oval_string_map *sysmap = oval_string_map_new();
-	struct oval_string_map *objmap = oval_string_map_new();
-	struct oval_string_map *sttmap = oval_string_map_new();
-	struct oval_string_map *varmap = oval_string_map_new();
-
-	struct oval_result_test_iterator *result_tests = (struct oval_result_test_iterator *)
-	    oval_string_map_values(tstmap);
-	if (oval_result_test_iterator_has_more(result_tests)) {
-		xmlNode *tests_node = xmlNewChild(system_node, ns_results, BAD_CAST "tests", NULL);
-		while (oval_result_test_iterator_has_more(result_tests)) {
-			struct oval_result_test *result_test = oval_result_test_iterator_next(result_tests);
-			oval_result_test_to_dom(result_test, doc, tests_node);
-			struct oval_test *oval_test = oval_result_test_get_test(result_test);
-			struct oval_object *object = oval_test_get_object(oval_test);
-			if (object) {
-				char *objid = oval_object_get_id(object);
-				void *value = oval_string_map_get_value(objmap, objid);
-				if (value == NULL) {
-					struct oval_syschar *syschar =
-					    oval_syschar_model_get_syschar(syschar_model, objid);
-					if (syschar) {
-						oval_string_map_put(objmap, objid, object);
-						_oval_result_system_scan_object_for_references
-						    (syschar_model, object, objmap, sttmap, varmap, sysmap);
-					}
-				}
-			}
-		}
-	}
-	oval_result_test_iterator_free(result_tests);
-
-	oval_syschar_model_to_dom
-	    (syschar_model, doc, system_node, (oval_syschar_resolver *) _oval_result_system_resolve_syschar, sysmap);
-
-	oval_string_map_free(sysmap, NULL);
-	oval_string_map_free(objmap, NULL);
-	oval_string_map_free(sttmap, NULL);
-	oval_string_map_free(varmap, NULL);
-	oval_string_map_free(tstmap, NULL);
-
-	return system_node;
-}
