@@ -142,8 +142,10 @@ bool oval_setobject_is_valid(struct oval_setobject * set_object)
 	bool is_valid = true;
 	oval_setobject_type_t type;
 
-	if (set_object == NULL)
+	if (set_object == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
 		return false;
+        }
 
 	type = oval_setobject_get_type(set_object);
 	switch (type) {
@@ -201,6 +203,7 @@ bool oval_setobject_is_valid(struct oval_setobject * set_object)
 		}
 		break;
 	default:
+                oscap_dprintf("WARNING: argument is not valid: wrong setobject type: %d.\n", type);
 		return false;
 	}
 
@@ -403,14 +406,14 @@ static int _oval_set_parse_tag(xmlTextReaderPtr reader, struct oval_parser_conte
 		} else if (strcmp(tagname, "filter") == 0) {
 			return_code = oval_parser_text_value(reader, context, &oval_consume_state_ref, &ctx);
 		} else {
-			int line = xmlTextReaderGetParserLineNumber(reader);
-			oscap_dprintf("NOTICE: oval_set_parse_tag::unhandled component <%s> %d", tagname, line);
+			oscap_dprintf("NOTICE: oval_set_parse_tag::unhandled component <%s> %d", tagname,
+                                      xmlTextReaderGetParserLineNumber(reader));
 			return_code = oval_parser_skip_tag(reader, context);
 		}
 	}
 	if (return_code != 1) {
-		int line = xmlTextReaderGetParserLineNumber(reader);
-		oscap_dprintf("NOTICE: oval_set_parse_tag::parse of <%s> terminated on error line %d", tagname, line);
+		oscap_dprintf("NOTICE: oval_set_parse_tag::parse of <%s> terminated on error line %d", tagname,
+                              xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
 	oscap_free(namespace);
