@@ -1,3 +1,13 @@
+/**
+ * @file   oval_sexp.c
+ * @brief  OVAL to/from S-exp conversion functions - implementation
+ * @author "Daniel Kopecek" <dkopecek@redhat.com>
+ * @author "Lukas Kuklinek" <lkukline@redhat.com>
+ * @author "Tomas Heinrich" <theinric@redhat.com>
+ *
+ * @addtogroup PROBEOVALSEXP
+ * @{
+ */
 /*
  * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
@@ -76,7 +86,6 @@ SEXP_t *oval_value_to_sexp(struct oval_value *val, oval_datatype_t dtype)
 		break;
 	case OVAL_DATATYPE_BOOLEAN:
 		val_sexp = SEXP_number_newb(oval_value_get_boolean(val));
-		SEXP_datatype_set(val_sexp, "bool");
 		break;
 	default:
 		val_sexp = NULL;
@@ -134,7 +143,7 @@ static SEXP_t *oval_entity_to_sexp(struct oval_entity *ent)
 	return (elm);
 }
 
-static SEXP_t *oval_varref_to_sexp(struct oval_entity *entity, struct oval_syschar_model *syschar_model)
+static SEXP_t *oval_varref_to_sexp(struct oval_entity *entity, struct oval_syschar_model *syschar_model, void *sess)
 {
 	unsigned int val_cnt = 0;
 	SEXP_t *val_lst, *val_sexp, *varref, *id_sexp, *val_cnt_sexp;
@@ -147,11 +156,11 @@ static SEXP_t *oval_varref_to_sexp(struct oval_entity *entity, struct oval_sysch
 	dt = oval_entity_get_datatype(entity);
 
 	var = oval_entity_get_variable(entity);
-	vit = oval_syschar_model_get_variable_values(syschar_model, var);
-	if (vit == NULL) {
+	if (oval_probe_session_query_variable(sess, var) != 0) {
 		SEXP_free(val_lst);
 		return NULL;
 	}
+	vit = oval_variable_get_values(var);
 
 	while (oval_value_iterator_has_more(vit)) {
 		val = oval_value_iterator_next(vit);
@@ -293,7 +302,7 @@ static SEXP_t *oval_behaviors_to_sexp(struct oval_behavior_iterator *bit)
 	return (r0);
 }
 
-SEXP_t *oval_object2sexp(const char *typestr, struct oval_object * object, struct oval_syschar_model * syschar_model)
+SEXP_t *oval_object2sexp(const char *typestr, struct oval_object *object, struct oval_syschar_model *syschar_model, void *sess)
 {
 	unsigned int ent_cnt, varref_cnt;
 	SEXP_t *obj_sexp, *obj_name, *elm, *varrefs, *ent_lst, *lst, *stmp;
@@ -350,7 +359,7 @@ SEXP_t *oval_object2sexp(const char *typestr, struct oval_object * object, struc
 			}
 
 			if (oval_entity_get_varref_type(entity) == OVAL_ENTITY_VARREF_ATTRIBUTE) {
-				stmp = oval_varref_to_sexp(entity, syschar_model);
+				stmp = oval_varref_to_sexp(entity, syschar_model, sess);
 
 				if (stmp == NULL) {
 					SEXP_free(elm);
@@ -362,6 +371,7 @@ SEXP_t *oval_object2sexp(const char *typestr, struct oval_object * object, struc
 					varrefs = SEXP_list_new(NULL);
 				}
 				SEXP_list_add(varrefs, stmp);
+                                SEXP_free(stmp);
 				// todo: don't add duplicates
 				++varref_cnt;
 
