@@ -45,6 +45,11 @@ typedef struct _oval_variable_model_frame {
 	oval_datatype_t datatype;
 } _oval_variable_model_frame_t;
 
+typedef struct oval_variable_model {
+	struct oval_string_map *varmap;
+	bool is_locked;
+} oval_variable_model_t;
+
 static _oval_variable_model_frame_t *_oval_variable_model_frame_new(char *id, const char *comm, oval_datatype_t datatype);
 static int _oval_variable_model_parse_tag(xmlTextReader * reader, struct oval_parser_context *context,
 					  struct oval_variable_model *model);
@@ -57,6 +62,7 @@ static int _oval_variable_model_parse_variables(xmlTextReader * reader, struct o
 						struct oval_variable_model *model);
 static int _oval_variable_model_parse(struct oval_variable_model *model, xmlTextReader * reader, void *user_param);
 static int _oval_generator_parse_tag(xmlTextReader * reader, struct oval_parser_context *context, char *label);
+
 static _oval_variable_model_frame_t *_oval_variable_model_frame_new(char *id, const char *comm, oval_datatype_t datatype)
 {
 	_oval_variable_model_frame_t *frame = (_oval_variable_model_frame_t *) oscap_alloc(sizeof(_oval_variable_model_frame_t));
@@ -69,29 +75,34 @@ static _oval_variable_model_frame_t *_oval_variable_model_frame_new(char *id, co
 	return frame;
 }
 
-typedef struct oval_variable_model {
-	struct oval_string_map *varmap;
-	bool is_locked;
-} oval_variable_model_t;
-
 bool oval_variable_model_is_valid(struct oval_variable_model *variable_model)
 {
+        bool is_valid = true;
+	struct oval_string_iterator *varids_itr;
 
-	struct oval_string_iterator *varids = oval_variable_model_get_variable_ids(variable_model);
-	while (oval_string_iterator_has_more(varids)) {
-		char *varid = oval_string_iterator_next(varids);
-		oval_datatype_t datatype = oval_variable_model_get_datatype(variable_model, varid);
-		if ((oval_variable_model_get_comment(variable_model, varid) == NULL) || 
-                    (oval_string_map_get_value(variable_model->varmap, varid) == NULL) ||
-                    (varid == NULL)) {
-                    oval_string_iterator_free(varids);
-                    return false;
+	if (variable_model == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+		return false;
+        }
+
+        varids_itr = oval_variable_model_get_variable_ids(variable_model);
+	while (oval_string_iterator_has_more(varids_itr)) {
+		char *varid = oval_string_iterator_next(varids_itr);
+
+                if (oval_variable_model_get_datatype(variable_model, varid) == OVAL_DATATYPE_UNKNOWN) {
+                        oscap_dprintf("WARNING: argument is not valid: variable (%s) datatype == OVAL_DATATYPE_UNKNOWN.\n", varid);
+                        is_valid = false;
+                        break;
+                }
+		if (oval_variable_model_get_comment(variable_model, varid) == NULL) {
+                        oscap_dprintf("WARNING: argument is not valid: variable (%s) comment == NULL.\n", varid);
+                        is_valid = false;
+                        break;
                 }
-
 	}
-        oval_string_iterator_free(varids);
+        oval_string_iterator_free(varids_itr);
 
-        return true;
+        return is_valid;
 }
 
 bool oval_variable_model_is_locked(struct oval_variable_model * variable_model)
@@ -320,24 +331,33 @@ static int _oval_variable_model_parse(struct oval_variable_model *model, xmlText
 	return return_code;
 }
 
-int oval_variable_model_import(struct oval_variable_model *model, struct oscap_import_source *source)
+struct oval_variable_model * oval_variable_model_import(const char *file)
 {
 	int return_code;
-	xmlDoc *doc = xmlParseFile(oscap_import_source_get_name(source));
+	struct oval_variable_model * model;
+	xmlDoc *doc = xmlParseFile(file);
 	if (doc == NULL) {
 		oscap_setxmlerr(xmlGetLastError());
-		return -1;
+		return NULL;
 	}
-	xmlTextReader *reader = xmlNewTextReaderFilename(oscap_import_source_get_name(source));
+	xmlTextReader *reader = xmlNewTextReaderFilename(file);
 	if (reader == NULL) {
 		oscap_setxmlerr(xmlGetLastError());
-		return -1;
+		return NULL;
 	}
 	xmlTextReaderRead(reader);
+
+	model = oval_variable_model_new();
 	return_code = _oval_variable_model_parse(model, reader, NULL);
+	if (return_code != 1) {
+		oval_variable_model_free(model);
+		model = NULL;
+	}
+		
 	xmlFreeTextReader(reader);
 	xmlFreeDoc(doc);
-	return return_code;
+
+	return model;
 }
 static int _generator_to_dom(xmlDocPtr doc, xmlNode * tag_generator)
 {
@@ -404,10 +424,10 @@ static xmlNode *oval_variable_model_to_dom(struct oval_variable_model * variable
 	return root_node;
 }
 
-int oval_variable_model_export(struct oval_variable_model *model, struct oscap_export_target *target)
+int oval_variable_model_export(struct oval_variable_model *model, const char *file)
 {
 
-	__attribute__nonnull__(target);
+	__attribute__nonnull__(model);
 
 	int retcode = 0;
 
@@ -423,9 +443,8 @@ int oval_variable_model_export(struct oval_variable_model *model, struct oscap_e
 	/*
 	 * Dumping document to stdio or file
 	 */
-	retcode =
-	    xmlSaveFormatFileEnc(oscap_export_target_get_name(target), doc, oscap_export_target_get_encoding(target),
-				 1);
+	retcode = xmlSaveFormatFileEnc(file, doc, "UTF-8", 1);
+
 	if (retcode < 1)
 		oscap_setxmlerr(xmlGetLastError());
 
@@ -433,6 +452,20 @@ int oval_variable_model_export(struct oval_variable_model *model, struct oscap_e
 	return retcode;
 }
 
+bool oval_variable_model_has_variable(struct oval_variable_model *model, const char * id)
+{
+        __attribute__nonnull__(model);
+        struct oval_string_iterator * str_it = (struct oval_string_iterator *)oval_string_map_keys(model->varmap);
+        while (oval_string_iterator_has_more(str_it)) {
+            if (!strcmp(oval_string_iterator_next(str_it), id)) {
+                oval_string_iterator_free(str_it);
+                return true;
+            }
+        }
+        oval_string_iterator_free(str_it);
+        return false;
+}
+
 struct oval_string_iterator *oval_variable_model_get_variable_ids(struct oval_variable_model *model)
 {
 	__attribute__nonnull__(model);
