@@ -135,34 +135,52 @@ struct oval_value_iterator *oval_variable_get_values(struct oval_variable *varia
 	return (value_collection) ? (struct oval_value_iterator *)oval_collection_iterator(variable->values) : NULL;
 }
 
-oval_syschar_collection_flag_t oval_syschar_model_get_variable_collection_flag
-    (struct oval_syschar_model *sysmod, struct oval_variable *variable) {
+oval_syschar_collection_flag_t oval_variable_get_collection_flag(struct oval_variable *variable) {
 	__attribute__nonnull__(variable);
 
-	if (variable->flag == SYSCHAR_FLAG_UNKNOWN) {
-		oval_syschar_model_get_variable_values(sysmod, variable);
-	}
 	return variable->flag;
 }
 
-struct oval_value_iterator *oval_syschar_model_get_variable_values
-    (struct oval_syschar_model *sysmod, struct oval_variable *variable) {
+int oval_syschar_model_compute_variable(struct oval_syschar_model *sysmod, struct oval_variable *variable) {
 	__attribute__nonnull__(variable);
 
-	if (variable->flag == SYSCHAR_FLAG_UNKNOWN) {
-		variable->values = oval_collection_new();
-		struct oval_component *component = oval_variable_get_component(variable);
-		if (component) {
-			variable->flag = oval_component_evaluate(sysmod, component, variable->values);
-		} else
-			oscap_dprintf("WARNING: NULL component bound to variable\n"
-				      "    variable type = %s\n"
-				      "    variable id   = %s\n"
-				      "    codeloc = %s(%d)\n",
-				      oval_variable_type_get_text(variable->type), oval_variable_get_id(variable),
-				      __FILE__, __LINE__);
-	}
-	return (variable->values) ? (struct oval_value_iterator *)oval_collection_iterator(variable->values) : NULL;
+	if (oval_variable_get_collection_flag(variable) != SYSCHAR_FLAG_UNKNOWN)
+                return 0;
+
+        variable->values = oval_collection_new();
+        struct oval_component *component = oval_variable_get_component(variable);
+        if (component) {
+                variable->flag = oval_component_compute(sysmod, component, variable->values);
+        } else {
+                oscap_dprintf("WARNING: NULL component bound to variable\n"
+                              "    variable type = %s\n"
+                              "    variable id   = %s\n",
+                              oval_variable_type_get_text(variable->type), oval_variable_get_id(variable));
+                return -1;
+        }
+
+        return 0;
+}
+
+int oval_probe_session_query_variable(oval_probe_session_t *sess, struct oval_variable *variable) {
+	__attribute__nonnull__(variable);
+
+	if (variable->flag != SYSCHAR_FLAG_UNKNOWN)
+		return 0;
+
+        variable->values = oval_collection_new();
+        struct oval_component *component = oval_variable_get_component(variable);
+        if (component) {
+		variable->flag = oval_component_query(sess, component, variable->values);
+	} else {
+                oscap_dprintf("WARNING: NULL component bound to variable\n"
+                              "    variable type = %s\n"
+                              "    variable id   = %s\n",
+                              oval_variable_type_get_text(variable->type), oval_variable_get_id(variable));
+		return -1;
+        }
+
+	return 0;
 }
 
 struct oval_component *oval_variable_get_component(struct oval_variable *variable)
@@ -181,7 +199,7 @@ struct oval_component *oval_variable_get_component(struct oval_variable *variabl
 /* failed   - NULL 
  * success  - oval_variable
  * */
-struct oval_variable *oval_variable_new(struct oval_definition_model *model, char *id, oval_variable_type_t type)
+struct oval_variable *oval_variable_new(struct oval_definition_model *model, const char *id, oval_variable_type_t type)
 {
 	oval_variable_t *variable;
 	switch (type) {
@@ -243,7 +261,40 @@ struct oval_variable *oval_variable_new(struct oval_definition_model *model, cha
 
 bool oval_variable_is_valid(struct oval_variable * variable)
 {
-	return true;		//TODO
+        oval_variable_type_t type;
+
+        if (variable == NULL) {
+                oscap_dprintf("WARNING: argument is not valid: NULL.\n");
+                return false;
+        }
+
+        if (oval_variable_get_datatype(variable) == OVAL_DATATYPE_UNKNOWN) {
+                oscap_dprintf("WARNING: argument is not valid: datatype == OVAL_DATATYPE_UNKNOWN.\n");
+                return false;
+        }
+
+        type = oval_variable_get_type(variable);
+        switch (type) {
+        case OVAL_VARIABLE_EXTERNAL:
+                break;
+        case OVAL_VARIABLE_CONSTANT:
+                break;
+        case OVAL_VARIABLE_LOCAL:
+                {
+                        oval_variable_LOCAL_t *var = (oval_variable_LOCAL_t *) variable;
+
+                        if (var->component == NULL) {
+                                oscap_dprintf("WARNING: argument is not valid: component == NULL.\n");
+                                return false;
+                        }
+                }
+                break;
+        default:
+                oscap_dprintf("WARNING: argument is not valid: wrong variable type: %d.\n", type);
+                return false;
+        }
+
+	return true;
 }
 
 bool oval_variable_is_locked(struct oval_variable * variable)
@@ -388,6 +439,25 @@ void oval_variable_add_value(struct oval_variable *variable, struct oval_value *
 		oscap_dprintf("WARNING: attempt to update locked content (%s:%d)", __FILE__, __LINE__);
 }
 
+void oval_variable_clear_values(struct oval_variable *variable)
+{
+	oval_variable_CONEXT_t *conext_var;
+
+	if (oval_variable_is_locked(variable)) {
+		oscap_dprintf("WARNING: attempt to update locked content.\n");
+		return;
+	}
+	if (variable->type != OVAL_VARIABLE_CONSTANT && variable->type != OVAL_VARIABLE_EXTERNAL) {
+		oscap_dprintf("WARNING: wrong variable type for this operation: %d.\n", variable->type);
+		return;
+        }
+
+	conext_var = (oval_variable_CONEXT_t *) variable;
+	oval_collection_free_items(conext_var->values, (oscap_destruct_func) oval_value_free);
+	conext_var->values = oval_collection_new();
+	variable->flag = SYSCHAR_FLAG_NOT_COLLECTED;
+}
+
 void oval_variable_set_component(struct oval_variable *variable, struct oval_component *component)
 {
 	if (variable && !oval_variable_is_locked(variable)) {
@@ -412,10 +482,9 @@ static int _oval_variable_parse_local_tag(xmlTextReaderPtr reader, struct oval_p
 	int return_code = oval_component_parse_tag(reader, context, &_oval_variable_parse_local_tag_component_consumer,
 						   variable);
 	if (return_code != 1) {
-		int line = xmlTextReaderGetParserLineNumber(reader);
 		oscap_dprintf
 		    ("NOTICE: oval_variable_parse_local_tag::parse of %s terminated on error at <%s> line %d",
-		     variable->id, tagname, line);
+		     variable->id, tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
 	oscap_free(tagname);
 	oscap_free(namespace);
@@ -440,10 +509,9 @@ static int _oval_variable_parse_constant_tag(xmlTextReaderPtr reader,
 	if (strcmp("value", (char *)tagname) == 0 && strcmp(DEFINITION_NAMESPACE, (char *)namespace) == 0) {
 		oval_parser_text_value(reader, context, (oval_xml_value_consumer) _oval_variable_parse_value, variable);
 	} else {
-		int line = xmlTextReaderGetParserLineNumber(reader);
 		oscap_dprintf("NOTICE: Invalid element <%s:%s> in constant variable"
 			      "    <constant_variable id = %s> at line %d"
-			      "    %s(%d)", namespace, tagname, variable->id, line, __FILE__, __LINE__);
+			      "    %s(%d)", namespace, tagname, variable->id, xmlTextReaderGetParserLineNumber(reader), __FILE__, __LINE__);
 	}
 	oscap_free(tagname);
 	oscap_free(namespace);
@@ -463,9 +531,8 @@ int oval_variable_parse_tag(xmlTextReaderPtr reader, struct oval_parser_context
 		type = OVAL_VARIABLE_LOCAL;
 	else {
 		type = OVAL_VARIABLE_UNKNOWN;
-		int line = xmlTextReaderGetParserLineNumber(reader);
 		oscap_dprintf("NOTICE::oval_variable_parse_tag: <%s> unhandled variable type::line = %d",
-			      tagname, line);
+			      tagname, xmlTextReaderGetParserLineNumber(reader));
 	}
 	char *id = (char *)xmlTextReaderGetAttribute(reader, BAD_CAST "id");
 	struct oval_variable *variable = oval_variable_get_new(model, id, type);
