@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -20,16 +20,19 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
-#ifndef __STUB_PROBE
+#include <config.h>
+
 #include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stddef.h>
-#include <sys/types.h>
-#include <sys/uio.h>
 #include <unistd.h>
 #include <errno.h>
-#include <config.h>
+#include <sys/types.h>
+#include <sys/uio.h>
+#include <sys/time.h>
+#include <sys/select.h>
+
 #include "public/sm_alloc.h"
 #include "public/strbuf.h"
 #include "_sexp-types.h"
@@ -50,22 +53,31 @@ int sch_generic_connect (SEAP_desc_t *desc, const char *uri, uint32_t flags)
 
 int sch_generic_openfd (SEAP_desc_t *desc, int fd, uint32_t flags)
 {
-        desc->scheme_data = sm_talloc (sch_genericdata_t); 
+        sch_genericdata_t *data;
+
+        data = sm_talloc (sch_genericdata_t);
+        data->ifd = -1;
+        data->ofd = -1;
 
         if (flags & SEAP_DESC_FDIN)
-                DATA(desc->scheme_data)->ifd = fd;
+                data->ifd = fd;
         if (flags & SEAP_DESC_FDOUT)
-                DATA(desc->scheme_data)->ofd = fd;
-        
+                data->ofd = fd;
+
+        desc->scheme_data = data;
+
         return (0);
 }
 
 int sch_generic_openfd2 (SEAP_desc_t *desc, int ifd, int ofd, uint32_t flags)
 {
-        desc->scheme_data = sm_talloc (sch_genericdata_t);
-        
-        DATA(desc->scheme_data)->ifd = ifd;
-        DATA(desc->scheme_data)->ofd = ofd;
+        sch_genericdata_t *data;
+
+        data = sm_talloc (sch_genericdata_t);
+        data->ifd = ifd;
+        data->ofd = ofd;
+        desc->scheme_data = data;
+
         return (0);
 }
 
@@ -83,24 +95,34 @@ ssize_t sch_generic_sendsexp (SEAP_desc_t *desc, SEXP_t *sexp, uint32_t flags)
 {
         ssize_t   ret;
         strbuf_t *sb;
-        
+
         _LOGCALL_;
-        
+
         ret = 0;
         sb  = strbuf_new (1024);
-        
+
         if (SEXP_sbprintf_t (sexp, sb) != 0)
                 ret = -1;
         else
                 ret = strbuf_write (sb, DATA(desc->scheme_data)->ofd);
-        
+
         strbuf_free (sb);
-                
+
         return (ret);
 }
 
 int sch_generic_close (SEAP_desc_t *desc, uint32_t flags)
 {
+        sch_genericdata_t *data;
+
+        data = (sch_genericdata_t *)desc->scheme_data;
+
+        if (data->ifd != -1)
+                close(data->ifd);
+        if (data->ofd != -1)
+                close(data->ofd);
+        sm_free(data);
+
         return (0);
 }
 
@@ -110,12 +132,12 @@ int sch_generic_select (SEAP_desc_t *desc, int ev, uint16_t timeout, uint32_t fl
         fd_set  fset;
         int fd;
         struct timeval *tv_ptr, tv;
-        
+
         FD_ZERO(&fset);
         tv_ptr = NULL;
         wptr   = NULL;
         rptr   = NULL;
-        
+
         switch (ev) {
         case SEAP_IO_EVREAD:
                 fd = DATA(desc->scheme_data)->ifd;
@@ -136,10 +158,10 @@ int sch_generic_select (SEAP_desc_t *desc, int ev, uint16_t timeout, uint32_t fl
                 tv.tv_usec = 0;
                 tv_ptr = &tv;
         }
-        
+
         _A(!(wptr == NULL && rptr == NULL));
         _A(!(wptr != NULL && rptr != NULL));
-        
+
         switch (select (fd + 1, rptr, wptr, NULL, tv_ptr)) {
         case -1:
                 protect_errno {
@@ -155,4 +177,3 @@ int sch_generic_select (SEAP_desc_t *desc, int ev, uint16_t timeout, uint32_t fl
         /* NOTREACHED */
         return (-1);
 }
-#endif
