@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +20,6 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
-#ifndef __STUB_PROBE
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -45,7 +44,7 @@ SEAP_packet_t *SEAP_packet_new (void)
         p = sm_talloc (SEAP_packet_t);
         memset (p, 0, sizeof (SEAP_packet_t));
         p->type = SEAP_PACKET_INV;
-        
+
         return (p);
 }
 
@@ -65,7 +64,7 @@ void *SEAP_packet_settype (SEAP_packet_t *packet, uint8_t type)
         _LOGCALL_;
 
         packet->type = type;
-        
+
         switch (type) {
         case SEAP_PACKET_MSG:
                 return ((void *)&(packet->data.msg));
@@ -76,7 +75,7 @@ void *SEAP_packet_settype (SEAP_packet_t *packet, uint8_t type)
         default:
                 abort ();
         }
-        
+
         /* not reached */
         return (NULL);
 }
@@ -110,41 +109,41 @@ static int SEAP_packet_sexp2msg (SEXP_t *sexp_msg, SEAP_msg_t *seap_msg)
 {
         size_t msg_icnt, msg_n, attr_i;
         SEXP_t *attr_name, *attr_val;
-        
+
         _A(sexp_msg != NULL);
         _A(seap_msg != NULL);
         _LOGCALL_;
-        
+
         msg_icnt = SEXP_list_length (sexp_msg);
-        
+
         seap_msg->attrs_cnt = msg_icnt - 4;
         seap_msg->attrs     = sm_alloc (sizeof (SEAP_attr_t) * seap_msg->attrs_cnt);
-        
+
         for (msg_n = 2, attr_i = 0; msg_n < msg_icnt; ++msg_n) {
-                
+
                 attr_name = SEXP_list_nth (sexp_msg, msg_n);
                 if (attr_name == NULL) {
                         _D("Unexpected error: No S-exp (attr_name) at position %u in the message (%p).\n",
                            msg_n, sexp_msg);
-                        
+
                         sm_free (seap_msg->attrs);
-                        
+
                         return (SEAP_EUNEXP);
                 }
-                
+
                 if (SEXP_string_nth (attr_name, 1) == ':') {
                         if (SEXP_strcmp (attr_name, ":id") == 0) {
-                                
+
                                 attr_val = SEXP_list_nth (sexp_msg, msg_n + 1);
                                 if (attr_val == NULL) {
                                         _D("Unexpected error: \"%s\": No attribute value at position %u in the message (%p).\n",
                                            "id", msg_n + 1, sexp_msg);
-                                        
+
                                         sm_free (seap_msg->attrs);
-                                        
+
                                         return (SEAP_EUNEXP);
                                 }
-                                
+
 #if SEAP_MSGID_BITS == 64
                                 seap_msg->id = SEXP_number_getu_64 (attr_val);
                                 if (seap_msg->id == UINT64_MAX)
@@ -157,48 +156,48 @@ static int SEAP_packet_sexp2msg (SEXP_t *sexp_msg, SEAP_msg_t *seap_msg)
                                         case EDOM:
                                                 _D("id: invalid value or type: s_exp=%p, type=%s\n",
                                                    (void *)attr_val, SEXP_strtype (attr_val));
-                                                
+
                                                 sm_free (seap_msg->attrs);
                                                 return (SEAP_EINVAL);
                                         case EFAULT:
                                                 _D("id: not found\n");
-                                                
+
                                                 sm_free (seap_msg->attrs);
                                                 return (SEAP_EINVAL);
                                         }
                                 }
-                                
+
                                 SEXP_free (attr_val);
                         } else {
                                 seap_msg->attrs[attr_i].name  = SEXP_string_subcstr (attr_name, 1, 0);
                                 seap_msg->attrs[attr_i].value = SEXP_list_nth (sexp_msg, msg_n + 1);
-                                
+
                                 if (seap_msg->attrs[attr_i].value == NULL) {
                                         _D("Unexpected error: \"%s\": No attribute value at position %u in the message (%p).\n",
                                            seap_msg->attrs[attr_i].name, msg_n + 1, sexp_msg);
-                                        
+
                                         sm_free (seap_msg->attrs[attr_i].name);
-                                        
+
                                         for (; attr_i > 0; --attr_i) {
                                                 sm_free (seap_msg->attrs[attr_i - 1].name);
-                                                
+
                                                 if (seap_msg->attrs[attr_i - 1].value != NULL)
                                                         SEXP_free (seap_msg->attrs[attr_i - 1].value);
                                         }
-                                        
+
                                         sm_free (seap_msg->attrs);
-                                        
+
                                         return (SEAP_EINVAL);
                                 }
-                                
+
                                 ++attr_i;
                         }
-                        
+
                         ++msg_n;
                 } else {
                         seap_msg->attrs[attr_i].name  = SEXP_string_cstr (attr_name);
                         seap_msg->attrs[attr_i].value = NULL;
-                        
+
                         ++attr_i;
                 }
 
@@ -207,11 +206,11 @@ static int SEAP_packet_sexp2msg (SEXP_t *sexp_msg, SEAP_msg_t *seap_msg)
 
         _A(attr_i <= (SEXP_list_length (sexp_msg) - 4));
         _A(attr_i >= (SEXP_list_length (sexp_msg) - 4)/2);
-        
+
         seap_msg->attrs_cnt = attr_i;
         seap_msg->attrs     = sm_realloc (seap_msg->attrs, sizeof (SEAP_attr_t) * seap_msg->attrs_cnt);
         seap_msg->sexp      = SEXP_list_last (sexp_msg);
-        
+
         return (0);
 }
 
@@ -219,7 +218,7 @@ static SEXP_t *SEAP_packet_msg2sexp (SEAP_msg_t *msg)
 {
         SEXP_t *sexp;
         uint16_t i;
-        
+
         SEXP_t *r0, *r1, *r2;
 
         _A(msg != NULL);
@@ -233,9 +232,9 @@ static SEXP_t *SEAP_packet_msg2sexp (SEAP_msg_t *msg)
                               r2 = SEXP_number_newu_32 (msg->id),
 #endif
                               NULL);
-        
+
         SEXP_vfree (r0, r1, r2, NULL);
-        
+
         /* Add message attributes */
         for (i = 0; i < msg->attrs_cnt; ++i) {
                 if (msg->attrs[i].value != NULL) {
@@ -248,20 +247,22 @@ static SEXP_t *SEAP_packet_msg2sexp (SEAP_msg_t *msg)
 
                 SEXP_free (r0);
         }
-        
+
         /* Add data */
         if (msg->sexp != NULL)
                 SEXP_list_add (sexp, msg->sexp);
-        else
+        else {
                 SEXP_list_add (sexp, r0 = SEXP_list_new (NULL)); /* FIXME */
+                SEXP_free(r0);
+        }
 
 #if !defined(NDEBUG)
         fprintf (stderr,   "--- pck out ---\n");
         SEXP_fprintfa (stderr, sexp);
         fprintf (stderr, "\n---------------\n");
 #endif
-        
-        return (sexp);        
+
+        return (sexp);
 }
 
 static int SEAP_packet_sexp2cmd (SEXP_t *sexp_cmd, SEAP_cmd_t *seap_cmd)
@@ -275,36 +276,36 @@ static int SEAP_packet_sexp2cmd (SEXP_t *sexp_cmd, SEAP_cmd_t *seap_cmd)
         memset (seap_cmd, 0, sizeof (SEAP_cmd_t));
         seap_cmd->class = SEAP_CMDCLASS_INT;
         seap_cmd->args  = NULL;
-        
+
         mattrs = 0;
         i = 2;
-        
+
         while ((item = SEXP_list_nth (sexp_cmd, i)) != NULL) {
                 if (SEXP_strncmp (item, ":", 1) == 0) {
                         char *attr_name;
-                        
+
                         attr_name = SEXP_string_cstr (item);
-                        
+
                         switch (*(attr_name + 1)) {
                         case 'i':
                                 if (strcmp (attr_name + 1, "id") == 0) {
                                         val = SEXP_list_nth (sexp_cmd, i + 1);
-                                        
+
                                         if (val == NULL)
                                                 break;
-                                        
+
                                         if (SEXP_numberp (val)) {
                                                 seap_cmd->id = SEXP_number_getu_16 (val);
                                                 ++mattrs;
                                         }
-                                        
+
                                         SEXP_free (val);
                                 }
                                 break;
                         case 'r':
                                 if (strcmp (attr_name + 1, "reply_id") == 0) {
                                         val = SEXP_list_nth (sexp_cmd, i + 1);
-                                        
+
                                         if (val == NULL)
                                                 break;
 
@@ -312,7 +313,7 @@ static int SEAP_packet_sexp2cmd (SEXP_t *sexp_cmd, SEAP_cmd_t *seap_cmd)
                                                 seap_cmd->rid = SEXP_number_getu_16 (val);
                                                 seap_cmd->flags |= SEAP_CMDFLAG_REPLY;
                                         }
-                                        
+
                                         SEXP_free (val);
                                 }
                                 break;
@@ -335,20 +336,20 @@ static int SEAP_packet_sexp2cmd (SEXP_t *sexp_cmd, SEAP_cmd_t *seap_cmd)
                         case 't':
                                 if (strcmp (attr_name + 1, "type") == 0) {
                                         val = SEXP_list_nth (sexp_cmd, i + 1);
-                                        
+
                                         if (val == NULL)
                                                 break;
-                                        
+
                                         if (SEXP_stringp (val)) {
                                                 if (SEXP_strcmp (val, "sync") == 0)
                                                         seap_cmd->flags |= SEAP_CMDFLAG_SYNC;
                                         }
-                                        
+
                                         SEXP_free (val);
                                 }
                                 break;
                         }
-                        
+
                         free (attr_name);
                         ++i; ++i;
                 } else {
@@ -357,7 +358,7 @@ static int SEAP_packet_sexp2cmd (SEXP_t *sexp_cmd, SEAP_cmd_t *seap_cmd)
 
                 SEXP_free (item);
         }
-        
+
         if (item == NULL || mattrs < 1) {
                 errno = EINVAL;
                 return (-1);
@@ -371,9 +372,9 @@ static int SEAP_packet_sexp2cmd (SEXP_t *sexp_cmd, SEAP_cmd_t *seap_cmd)
                 errno = EINVAL;
                 return (-1);
         }
-        
+
         seap_cmd->args = SEXP_list_nth (sexp_cmd, ++i);
-        
+
         return (0);
 }
 
@@ -381,7 +382,7 @@ static SEXP_t *SEAP_packet_cmd2sexp (SEAP_cmd_t *cmd)
 {
         SEXP_t *sexp;
         SEXP_t *r0, *r1, *r2;
-        
+
         _A(cmd != NULL);
         _LOGCALL_;
 
@@ -389,22 +390,22 @@ static SEXP_t *SEAP_packet_cmd2sexp (SEAP_cmd_t *cmd)
                               r1 = SEXP_string_new (":id", 3),
                               r2 = SEXP_number_newu_16 (cmd->id),
                               NULL);
-        
+
         SEXP_vfree (r0, r1, r2, NULL);
-        
+
         if (cmd->flags & SEAP_CMDFLAG_REPLY) {
                 SEXP_list_add (sexp,
                                r0 = SEXP_string_new (":reply_id", 9));
                 SEXP_list_add (sexp,
                                r1 = SEXP_number_newu_16 (cmd->rid));
-                
+
                 SEXP_vfree (r0, r1, NULL);
         }
-        
+
         SEXP_list_add (sexp,
                        r0 = SEXP_string_new (":class", 6));
         SEXP_free (r0);
-        
+
         switch (cmd->class) {
         case SEAP_CMDCLASS_USR:
                 SEXP_list_add (sexp,
@@ -417,25 +418,25 @@ static SEXP_t *SEAP_packet_cmd2sexp (SEAP_cmd_t *cmd)
         default:
                 abort ();
         }
-        
+
 
         SEXP_free (r0);
-        
+
         SEXP_list_add (sexp,
                        r0 = SEXP_string_new (":type", 5));
-        
+
         SEXP_list_add (sexp,
                        r1 = (cmd->flags & SEAP_CMDFLAG_SYNC ?
                              SEXP_string_new ("sync", 4) : SEXP_string_new ("async", 5)));
-        
+
         SEXP_list_add (sexp,
                        r2 = SEXP_number_newu_16 (cmd->code));
-        
+
         SEXP_vfree (r0, r1, r2, NULL);
-        
+
         if (cmd->args != NULL)
                 SEXP_list_add (sexp, cmd->args);
-        
+
         SEXP_VALIDATE(sexp);
         return (sexp);
 }
@@ -444,14 +445,14 @@ static int SEAP_packet_sexp2err (SEXP_t *sexp_err, SEAP_err_t *seap_err)
 {
         SEXP_t  *member;
         uint32_t n;
-        
+
         _LOGCALL_;
-        
+
         seap_err->id   = 0;
         seap_err->code = SEAP_EUNKNOWN;
         seap_err->type = SEAP_ETYPE_INT;
         seap_err->data = NULL;
-        
+
         n = 2;
 
         while ((member = SEXP_list_nth (sexp_err, n)) != NULL) {
@@ -463,7 +464,7 @@ static int SEAP_packet_sexp2err (SEXP_t *sexp_err, SEAP_err_t *seap_err)
                                 errno = EINVAL;
                                 return (-1);
                         }
-                        
+
                         if (SEXP_strcmp (member, ":orig_id") == 0) {
                                 SEXP_t *val;
 
@@ -476,18 +477,18 @@ static int SEAP_packet_sexp2err (SEXP_t *sexp_err, SEAP_err_t *seap_err)
                                         errno = EINVAL;
                                         return (-1);
                                 }
-                                
+
 #if SEAP_MSGID_BITS == 64
                                 seap_err->id = SEXP_number_getu_64 (val);
 #else
                                 seap_err->id = SEXP_number_getu_32 (val);
-#endif                    
+#endif
                                 SEXP_free (val);
                         } else if (SEXP_strcmp (member, ":type") == 0) {
                                 SEXP_t *val;
 
                                 val = SEXP_list_nth (member, ++n);
-                                
+
                                 if (SEXP_strcmp (val, "int") == 0)
                                         seap_err->type = SEAP_ETYPE_INT;
                                 else if (SEXP_strcmp (val, "usr") == 0)
@@ -499,7 +500,7 @@ static int SEAP_packet_sexp2err (SEXP_t *sexp_err, SEAP_err_t *seap_err)
                                         errno = EINVAL;
                                         return (-1);
                                 }
-                                
+
                                 SEXP_free (val);
                         } else {
                                 _D("Unknown packet attribute\n");
@@ -512,7 +513,7 @@ static int SEAP_packet_sexp2err (SEXP_t *sexp_err, SEAP_err_t *seap_err)
                 case SEXP_TYPE_NUMBER:
                         seap_err->code = SEXP_number_getu_32 (member);
                         SEXP_free (member);
-                        
+
                         goto loop_exit;
                 default:
                         _D("Unexpected type of packet member: list\n");
@@ -520,12 +521,12 @@ static int SEAP_packet_sexp2err (SEXP_t *sexp_err, SEAP_err_t *seap_err)
                         errno = EINVAL;
                         return (-1);
                 }
-                
+
                 ++n;
         }
 loop_exit:
         seap_err->data = SEXP_list_nth (sexp_err, n + 1);
-        
+
         return (0);
 }
 
@@ -533,7 +534,7 @@ static SEXP_t *SEAP_packet_err2sexp (SEAP_err_t *err)
 {
         SEXP_t *sexp;
         SEXP_t *r0, *r1, *r2, *r3, *r4, *r5;
-        
+
         _LOGCALL_;
 
         sexp = SEXP_list_new (r0 = SEXP_string_new (SEAP_SYM_ERR, strlen (SEAP_SYM_ERR)),
@@ -542,26 +543,26 @@ static SEXP_t *SEAP_packet_err2sexp (SEAP_err_t *err)
                               r2 = SEXP_number_newu_64 (err->id),
 #else
                               r2 = SEXP_number_newu_32 (err->id),
-#endif                    
+#endif
                               r3 = SEXP_string_new (":type", 5),
                               r4 = SEXP_number_newu (err->type),
                               r5 = SEXP_number_newu (err->code),
                               NULL);
-        
+
         SEXP_vfree (r1, r2, r3, r4, r5, NULL);
-        
+
         if (err->data != NULL)
                 SEXP_list_add (sexp, err->data);
-        
+
         return (sexp);
 }
 
 static SEXP_t *SEAP_packet2sexp (SEAP_packet_t *packet)
 {
         SEXP_t *sexp = NULL;
-        
+
         _LOGCALL_;
-        
+
         switch (packet->type) {
         case SEAP_PACKET_MSG:
                 sexp = SEAP_packet_msg2sexp (SEAP_packet_msg (packet));
@@ -575,7 +576,7 @@ static SEXP_t *SEAP_packet2sexp (SEAP_packet_t *packet)
         default:
                 errno = EINVAL;
         }
-        
+
         return (sexp);
 }
 
@@ -587,18 +588,18 @@ int SEAP_packet_recv (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet)
         void        *data_buffer;
         size_t       data_buflen;
         ssize_t      data_length;
-        
+
         SEXP_psetup_t *psetup;
         SEXP_pstate_t *pstate;
 
         SEXP_t     *psym_sexp;
         char        psym_cstr_b[16+1];
         char       *psym_cstr;
-        
+
         _LOGCALL_;
 
-        dsc = SEAP_desc_get (&(ctx->sd_table), sd);
-        
+        dsc = SEAP_desc_get (ctx->sd_table, sd);
+
         if (dsc == NULL) {
                 errno = EFAULT;
                 return (-1);
@@ -615,8 +616,11 @@ int SEAP_packet_recv (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet)
         if (dsc->sexpbuf != NULL) {
                 if (SEXP_list_length (dsc->sexpbuf) > 0)
                         goto sexp_buf_recv;
+                else {
+                        SEXP_free(dsc->sexpbuf);
+                        dsc->sexpbuf = NULL;
+                }
         }
-                        
         /*
          * Event loop
          * The read mutex is not locked during the wait for an event.
@@ -629,13 +633,13 @@ int SEAP_packet_recv (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet)
          */
 
 eloop_start:
-                
+
         for (;;) {
                 if (SCH_SELECT(dsc->scheme, dsc, SEAP_IO_EVREAD, 0, 0) != 0)
                         return (-1);
-                
+
                 _D("return from select\n");
-                
+
                 switch (DESC_TRYRLOCK (dsc)) {
                 case  1:
                         goto eloop_exit;
@@ -651,38 +655,44 @@ eloop_start:
                         }
                 case -1:
                         fail_rmutex:
-                        
+
                         protect_errno {
                                 _D("An error ocured while locking the read mutex: dsc=%p, errno=%u, %s.\n",
                                    dsc, errno, strerror (errno));
                         }
-                        
+
                         return (-1);
                 default:
                         abort ();
                 }
         }
 eloop_exit:
-        
+
         /*
          * Receive loop
          * The read mutex is locked during execution of this loop and
          * the execution stops if the received data forms a valid
          * S-expression.
          */
-        
+
         pstate = NULL;
         psetup = SEXP_psetup_new ();
 
+        /*
+         * All buffer passed to SEXP_parse will be freed by
+         * SEXP_pstate_free (i.e. after successful parsing)
+         */
+        SEXP_psetup_setflags(psetup, SEXP_PFLAG_FREEBUF);
+
         for (;;) {
                 data_buffer = sm_alloc (SEAP_RECVBUF_SIZE);
                 data_buflen = SEAP_RECVBUF_SIZE;
                 data_length = SCH_RECV(dsc->scheme, dsc, data_buffer, data_buflen, 0);
-                
+
                 if (data_length < 0) {
                         protect_errno {
                                 _D("FAIL: recv failed: dsc=%p, errno=%u, %s.\n", dsc, errno, strerror (errno));
-                                
+
                                 sm_free (data_buffer);
                                 SEXP_psetup_free (psetup);
 
@@ -694,7 +704,7 @@ eloop_exit:
                         _D("zero bytes received -> EOF\n");
                         sm_free (data_buffer);
                         SEXP_psetup_free (psetup);
-                        
+
                         if (pstate != NULL) {
                                 _D("FAIL: incomplete S-exp received\n");
                                 errno = ENETRESET;
@@ -704,19 +714,21 @@ eloop_exit:
                                 return (-1);
                         }
                 }
-                
+
                 _A(data_length > 0);
-                
-                if (data_buflen != (size_t)(data_length))
+
+                if (data_buflen != (size_t)(data_length)) {
                         data_buffer = sm_realloc (data_buffer, data_length);
-                
+			data_buflen = data_length;
+		}
+
                 sexp_buffer = SEXP_parse (psetup, data_buffer, data_length, &pstate);
-                
+
                 if (sexp_buffer != NULL) {
                         _A(pstate == NULL);
-                        
+
                         DESC_RUNLOCK(dsc);
-                        
+
                         if (SEXP_list_length (sexp_buffer) > 0) {
                                 break;
                         } else {
@@ -726,7 +738,7 @@ eloop_exit:
                                 goto eloop_start;
                         }
                 }
-                
+
                 if (SCH_SELECT(dsc->scheme, dsc, SEAP_IO_EVREAD, ctx->recv_timeout, 0) != 0) {
                         switch (errno) {
                         case ETIMEDOUT:
@@ -738,8 +750,7 @@ eloop_exit:
                                 protect_errno {
                                         _D("FAIL: recv failed: dsc=%p, errno=%u, %s.\n",
                                            dsc, errno, strerror (errno));
-                                        
-                                        sm_free (data_buffer);
+
                                         SEXP_psetup_free (psetup);
                                         SEXP_pstate_free (pstate);
                                 }
@@ -747,16 +758,16 @@ eloop_exit:
                         }
                 }
         }
-        
+
         SEXP_psetup_free (psetup);
         dsc->sexpbuf = sexp_buffer;
-        
+
 sexp_buf_recv:
         SEXP_VALIDATE(dsc->sexpbuf);
         sexp_packet = SEXP_list_pop (dsc->sexpbuf);
-        
+
         _A(sexp_packet != NULL);
-        
+
         if (!SEXP_listp(sexp_packet)) {
                 _D("Invalid SEAP packet received: %s.\n", "not a list");
 
@@ -772,20 +783,20 @@ sexp_buf_recv:
                 errno = EINVAL;
                 return (-1);
         }
-        
+
         psym_sexp = SEXP_list_first (sexp_packet);
-        
+
         if (!SEXP_stringp(psym_sexp)) {
                 _D("Invalid SEAP packet received: %s.\n", "first list item is not a string");
-                
+
                 SEXP_free (psym_sexp);
                 SEXP_free (sexp_packet);
-                
+
                 errno = EINVAL;
                 return (-1);
         } else if (SEXP_string_length (psym_sexp) != (strlen (SEAP_SYM_PREFIX) + 3)) {
                 _D("Invalid SEAP packet received: %s.\n", "invalid packet type symbol length");
-                
+
                 SEXP_free (psym_sexp);
                 SEXP_free (sexp_packet);
 
@@ -800,11 +811,11 @@ sexp_buf_recv:
                 errno = EINVAL;
                 return (-1);
         }
-        
+
         SEXP_string_cstr_r (psym_sexp, psym_cstr_b, sizeof psym_cstr_b);
         psym_cstr = psym_cstr_b + strlen (SEAP_SYM_PREFIX);
         SEXP_free (psym_sexp);
-        
+
         switch (psym_cstr[0]) {
         case 'm':
                 if (psym_cstr[1] == 's' &&
@@ -812,13 +823,13 @@ sexp_buf_recv:
                 {
                         (*packet) = SEAP_packet_new ();
                         (*packet)->type = SEAP_PACKET_MSG;
-                        
+
                         if (SEAP_packet_sexp2msg (sexp_packet, &((*packet)->data.msg)) != 0) {
                                 /* error */
                                 _D("Invalid SEAP packet received: %s.\n", "can't translate to msg struct");
-                                
+
                                 SEXP_free (sexp_packet);
-                                
+
                                 errno = EINVAL;
                                 return (-1);
                         }
@@ -836,7 +847,7 @@ sexp_buf_recv:
                                 /* error */
                                 _D("Invalid SEAP packet received: %s.\n", "can't translate to cmd struct");
                                 SEXP_free (sexp_packet);
-                                
+
                                 errno = EINVAL;
                                 return (-1);
                         }
@@ -849,12 +860,12 @@ sexp_buf_recv:
                 {
                         (*packet) = SEAP_packet_new ();
                         (*packet)->type = SEAP_PACKET_ERR;
-                        
+
                         if (SEAP_packet_sexp2err (sexp_packet, &((*packet)->data.err)) != 0) {
                                 /* error */
                                 _D("Invalid SEAP packet received: %s.\n", "can't translate to err struct");
                                 SEXP_free (sexp_packet);
-                                
+
                                 errno = EINVAL;
                                 return (-1);
                         }
@@ -873,9 +884,9 @@ sexp_buf_recv:
         SEXP_fprintfa (stderr, sexp_packet);
         fprintf (stderr, "\n--------------\n");
 #endif
-        
+
         SEXP_free (sexp_packet);
-        
+
         return (0);
 }
 
@@ -892,16 +903,16 @@ int SEAP_packet_recv_bytype (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet, ui
            type == SEAP_PACKET_ERR);
 
         _LOGCALL_;
-        
-        dsc = SEAP_desc_get (&(ctx->sd_table), sd);
-        
+
+        dsc = SEAP_desc_get (ctx->sd_table, sd);
+
         if (dsc == NULL)
                 return (-1);
-        
+
         for (;;) {
                 pck = NULL;
                 ret = SEAP_packet_recv (ctx, sd, &pck);
-        
+
                 if (ret != 0)
                         return (ret);
                 else if (pck->type == type)
@@ -909,7 +920,7 @@ int SEAP_packet_recv_bytype (SEAP_CTX_t *ctx, int sd, SEAP_packet_t **packet, ui
                 else if (pqueue_add (dsc->pck_queue, pck) != 0)
                         return (-1);
         }
-        
+
         *packet = pck;
 
         return (ret);
@@ -920,17 +931,17 @@ int SEAP_packet_send (SEAP_CTX_t *ctx, int sd, SEAP_packet_t *packet)
         SEXP_t *packet_sexp;
         SEAP_desc_t *dsc;
         int ret;
-                
+
         _LOGCALL_;
 
         ret = -1;
-        dsc = SEAP_desc_get (&(ctx->sd_table), sd);
-        
+        dsc = SEAP_desc_get (ctx->sd_table, sd);
+
         if (dsc == NULL)
                 return (-1);
-        
+
         packet_sexp = SEAP_packet2sexp (packet);
-        
+
         if (packet_sexp == NULL) {
                 _D("Can't convert S-exp to packet\n");
                 return (-1);
@@ -938,22 +949,22 @@ int SEAP_packet_send (SEAP_CTX_t *ctx, int sd, SEAP_packet_t *packet)
 
         if (DESC_WLOCK (dsc)) {
                 ret = 0;
-                
+
                 if (SCH_SENDSEXP(dsc->scheme, dsc, packet_sexp, 0) < 0) {
                         ret = -1;
-                        
+
                         protect_errno {
                                 _D("FAIL: errno=%u, %s.\n", errno, strerror (errno));
                         }
                 }
-                
+
                 DESC_WUNLOCK(dsc);
         }
-        
+
         protect_errno {
                 SEXP_free (packet_sexp);
         }
-        
+
         return (ret);
 }
 
@@ -962,4 +973,3 @@ int SEAP_packet_enqueue (SEAP_CTX_t *ctx, int sd, SEAP_packet_t *packet)
         _LOGCALL_;
         return (-1);
 }
-#endif
