@@ -38,6 +38,7 @@
 #include "_seap-command.h"
 #include "_seap-error.h"
 #include "_seap-packet.h"
+#include "_seap.h"
 
 static void SEAP_CTX_initdefault (SEAP_CTX_t *ctx)
 {
@@ -125,7 +126,11 @@ int SEAP_connect (SEAP_CTX_t *ctx, const char *uri, uint32_t flags)
         }
 
         dsc = SEAP_desc_get (ctx->sd_table, sd);
-        _A(dsc != NULL);
+
+        if (dsc == NULL) {
+                errno = ESRCH;
+                return(-1);
+        }
 
         if (SCH_CONNECT(scheme, dsc, uri + schstr_len + 1, flags) != 0) {
                 _D("FAIL: errno=%u, %s.\n", errno, strerror (errno));
@@ -166,7 +171,11 @@ int SEAP_openfd2 (SEAP_CTX_t *ctx, int ifd, int ofd, uint32_t flags)
         }
 
         dsc = SEAP_desc_get (ctx->sd_table, sd);
-        _A(dsc != NULL);
+
+        if (dsc == NULL) {
+                errno = ESRCH;
+                return(-1);
+        }
 
         if (SCH_OPENFD2(SCH_GENERIC, dsc, ifd, ofd, flags) != 0) {
                 _D("FAIL: errno=%u, %s.\n", errno, strerror (errno));
@@ -202,19 +211,15 @@ static int __SEAP_cmdexec_reply (SEAP_CTX_t *ctx, int sd, SEAP_cmd_t *cmd)
 
         _LOGCALL_;
 
+        dsc = SEAP_desc_get (ctx->sd_table, sd);
+
+        if (dsc == NULL)
+                return (-1);
+
         res = SEAP_cmd_exec (ctx, sd, SEAP_EXEC_LOCAL,
                              cmd->code, cmd->args,
                              SEAP_CMDCLASS_USR, NULL, NULL);
 
-        dsc = SEAP_desc_get (ctx->sd_table, sd);
-
-        if (dsc == NULL) {
-                protect_errno {
-                        SEXP_free (res);
-                }
-                return (-1);
-        }
-
         packet = SEAP_packet_new ();
         cmdrep = SEAP_packet_settype (packet, SEAP_PACKET_CMD);
 
@@ -233,6 +238,9 @@ static int __SEAP_cmdexec_reply (SEAP_CTX_t *ctx, int sd, SEAP_cmd_t *cmd)
                 return (-1);
         }
 
+        if (res != NULL)
+                SEXP_free(res);
+        
         SEAP_packet_free (packet);
 
         return (0);
@@ -257,7 +265,7 @@ static void *__SEAP_cmdexec_worker (void *arg)
         return (NULL);
 }
 
-static int __SEAP_recvmsg_process_cmd (SEAP_CTX_t *ctx, int sd, SEAP_cmd_t *cmd)
+int __SEAP_recvmsg_process_cmd (SEAP_CTX_t *ctx, int sd, SEAP_cmd_t *cmd)
 {
         _A(ctx != NULL);
         _A(cmd != NULL);
@@ -300,7 +308,12 @@ static int __SEAP_recvmsg_process_cmd (SEAP_CTX_t *ctx, int sd, SEAP_cmd_t *cmd)
                                               cmd->rid, cmd->args,
                                               SEAP_CMDCLASS_USR, NULL, NULL);
                 } else {
-                        return __SEAP_cmdexec_reply (ctx, sd, cmd);
+                        int ret;
+
+                        ret = __SEAP_cmdexec_reply (ctx, sd, cmd);
+                        
+                        if (cmd->args != NULL)
+                                SEXP_free(cmd->args);
                 }
         }
 
@@ -540,7 +553,7 @@ int SEAP_write (SEAP_CTX_t *ctx, int sd, SEXP_t *sexp)
 int SEAP_close (SEAP_CTX_t *ctx, int sd)
 {
         SEAP_desc_t *dsc;
-        int ret = 0;
+        int ret;
 
         _A(ctx != NULL);
         _LOGCALL_;
@@ -551,16 +564,21 @@ int SEAP_close (SEAP_CTX_t *ctx, int sd)
         }
 
         dsc = SEAP_desc_get (ctx->sd_table, sd);
+
+        if (dsc == NULL) {
+                errno = EBADF;
+                return (-1);
+        }
+
         ret = SCH_CLOSE(dsc->scheme, dsc, 0); /* TODO: Are flags usable here? */
 
         protect_errno {
                 if (SEAP_desc_del (ctx->sd_table, sd) != 0) {
                         /* something very bad happened */
                         _D("SEAP_desc_del failed\n");
-                        if (ret > 0)
-                                ret = -1;
+                        return(-1);
                 }
         }
 
-        return (ret);
+        return(ret);
 }
