@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat Inc., Durham, North Carolina.
+ * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -20,9 +20,9 @@
  *      "Daniel Kopecek" <dkopecek@redhat.com>
  */
 
-#ifndef __STUB_PROBE
 #include <config.h>
 #include <stdint.h>
+#include "_sexp-atomic.h"
 #include "_sexp-value.h"
 #include "public/sm_alloc.h"
 
@@ -35,9 +35,9 @@ int SEXP_val_new (SEXP_val_t *dst, size_t vmemsize, SEXP_type_t type)
         {
                 return (-1);
         }
-        
+
         SEXP_val_dsc (dst, (uintptr_t) s_val);
-        
+
         dst->hdr->refs = 1;
         dst->hdr->size = vmemsize;
         dst->type      = type;
@@ -49,7 +49,7 @@ int SEXP_val_new (SEXP_val_t *dst, size_t vmemsize, SEXP_type_t type)
            "                type = %hhu\n"
            "                 ptr = %p\n",
            dst->hdr->refs, dst->hdr->size, dst->type, (void *)dst->ptr);
-        
+
         return (0);
 }
 
@@ -73,15 +73,7 @@ uintptr_t SEXP_val_ptr (SEXP_val_t *dsc)
  */
 uintptr_t SEXP_rawval_incref (uintptr_t valp)
 {
-        uint32_t refs;
-
-#if defined(HAVE_ATOMIC_FUNCTIONS) || defined(HAVE_ATOMIC_BUILTINS)
-        refs = __sync_fetch_and_add (&(SEXP_VALP_HDR(valp)->refs), 1);
-#else
-        # warning "Atomic functions not available. SEXP_rawval_incref will return copies of values."
-        /* TODO: copy the value here */
-#endif
-        return (refs > 0 ? valp : (uintptr_t) NULL);
+        return SEXP_atomic_inc_u32 (&(SEXP_VALP_HDR(valp)->refs)) > 0 ? valp : (uintptr_t) NULL;
 }
 
 /*
@@ -91,12 +83,7 @@ uintptr_t SEXP_rawval_incref (uintptr_t valp)
  */
 int SEXP_rawval_decref (uintptr_t valp)
 {
-#if defined(HAVE_ATOMIC_FUNCTIONS) || defined(HAVE_ATOMIC_BUILTINS)
-        return (__sync_sub_and_fetch (&(SEXP_VALP_HDR(valp)->refs), 1) == 0);
-#else
-        # warning "Atomic functions not available. SEXP_rawval_decref will always signal zero refs."
-        return (1);
-#endif
+        return (SEXP_atomic_dec_u32 (&(SEXP_VALP_HDR(valp)->refs)) == 0);
 }
 
 SEXP_numtype_t SEXP_rawval_number_type (SEXP_val_t *dsc)
@@ -111,21 +98,21 @@ size_t SEXP_rawval_list_length (struct SEXP_val_list *list)
 
         length = 0;
         lblk   = SEXP_VALP_LBLK(list->b_addr);
-        
+
         while (lblk != NULL) {
                 length += lblk->real;
                 lblk    = SEXP_VALP_LBLK(lblk->nxsz);
         }
-        
+
         return (length - list->offset);
 }
 
 uintptr_t SEXP_rawval_lblk_new (uint8_t sz)
 {
         struct SEXP_val_lblk *lblk;
-                
+
         _A(sz < 16);
-        
+
         if (sm_memalign ((void **)(void *)&lblk, SEXP_LBLK_ALIGN,
                          sizeof (uintptr_t) + (2 * sizeof (uint16_t)) + (sizeof (SEXP_t) * (1 << sz))) != 0) {
                 /* TODO: handle this */
@@ -136,7 +123,7 @@ uintptr_t SEXP_rawval_lblk_new (uint8_t sz)
         lblk->nxsz = ((uintptr_t)(NULL) & SEXP_LBLKP_MASK) | ((uintptr_t)sz & SEXP_LBLKS_MASK);
         lblk->refs = 1;
         lblk->real = 0;
-        
+
         return ((uintptr_t)lblk);
 }
 
@@ -144,10 +131,9 @@ uintptr_t SEXP_rawval_lblk_incref (uintptr_t lblkp)
 {
         struct SEXP_val_lblk *lblk;
         uint16_t refs;
-        
+
         lblk = SEXP_VALP_LBLK(lblkp);
-        
-#if defined(HAVE_ATOMIC_FUNCTIONS) || defined(HAVE_ATOMIC_BUILTINS)
+
         /*
          * Atomicaly update the reference counter.
          * If the reference counter can't be updated
@@ -156,29 +142,20 @@ uintptr_t SEXP_rawval_lblk_incref (uintptr_t lblkp)
          */
         for (;;) {
                 refs = lblk->refs;
-                
+
                 if (refs < UINT16_MAX) {
-                        if (__sync_bool_compare_and_swap (&lblk->refs, refs, refs + 1))
+                        if (SEXP_atomic_cas_u16 (&lblk->refs, refs, refs + 1))
                                 break;
                 } else
-                        return SEXP_rawval_list_copy (lblkp, 0);
+                        return SEXP_rawval_lblk_copy (lblkp, 0);
         }
-        
+
         return (lblkp);
-#else
-        # warning "Atomic functions not available. SEXP_rawval_lblk_incref will return copies of values."
-        return SEXP_rawval_list_copy (lblkp, 0);
-#endif
 }
 
 int SEXP_rawval_lblk_decref (uintptr_t lblkp)
 {
-#if defined(HAVE_ATOMIC_FUNCTIONS) || defined(HAVE_ATOMIC_BUILTINS)
-        return (__sync_sub_and_fetch (&SEXP_VALP_LBLK(lblkp)->refs, 1) == 0);
-#else
-# warning "Atomic functions not available. SEXP_rawval_lblk_decref will always signal zero refs."
-        return (1);
-#endif
+        return (SEXP_atomic_dec_u16 (&SEXP_VALP_LBLK(lblkp)->refs) == 0);
 }
 
 uintptr_t SEXP_rawval_lblk_fill (uintptr_t lblkp, SEXP_t *s_exp[], uint16_t s_exp_count)
@@ -186,12 +163,12 @@ uintptr_t SEXP_rawval_lblk_fill (uintptr_t lblkp, SEXP_t *s_exp[], uint16_t s_ex
         struct SEXP_val_lblk *lblk;
 
         lblk = SEXP_VALP_LBLK(lblkp);
-        
+
         if (s_exp_count > (1 << (lblk->nxsz & 0xf)) - lblk->real)
                 return ((uintptr_t) NULL);
-        
+
         lblk->real = s_exp_count;
-        
+
         for (; s_exp_count > 0; --s_exp_count) {
                 lblk->memb[s_exp_count - 1].s_valp = SEXP_rawval_incref (s_exp[s_exp_count - 1]->s_valp);
                 lblk->memb[s_exp_count - 1].s_type = s_exp[s_exp_count - 1]->s_type;
@@ -201,7 +178,7 @@ uintptr_t SEXP_rawval_lblk_fill (uintptr_t lblkp, SEXP_t *s_exp[], uint16_t s_ex
                 lblk->memb[s_exp_count - 1].__magic1 = s_exp[s_exp_count - 1]->__magic1;
 #endif
         }
-        
+
         return (lblkp);
 }
 
@@ -226,17 +203,17 @@ uintptr_t SEXP_rawval_lblk_add (uintptr_t lblkp, const SEXP_t *s_exp)
                                 lblk    = SEXP_VALP_LBLK(lblk->nxsz);
                         } else {
                                 uintptr_t lb_ptr;
-                                
+
                                 /*
                                  * We've encountered a block which belongs to more
                                  * than one list so we have to create a copy of the
                                  * rest of the list.
                                  */
-                                lb_ptr = SEXP_rawval_list_copy (lblkp, 0);
-                                
+                                lb_ptr = SEXP_rawval_lblk_copy (lblkp, 0);
+
                                 if (lb_prev == 0)
                                         lb_head = lb_ptr;
-                                
+
                                 /*
                                  * Update pointer in the previous block (if there is one)
                                  * to point at the copy. Also decrement the reference
@@ -244,9 +221,9 @@ uintptr_t SEXP_rawval_lblk_add (uintptr_t lblkp, const SEXP_t *s_exp)
                                  */
                                 if (lb_prev != 0)
                                         SEXP_VALP_LBLK(lb_prev)->nxsz = (lb_ptr & SEXP_LBLKP_MASK) | (SEXP_VALP_LBLK(lb_prev)->nxsz & SEXP_LBLKS_MASK);
-                                
+
                                 SEXP_rawval_lblk_decref (lblkp);
-                                
+
                                 /*
                                  * Get the last block without checking refs
                                  * (we don't need to because it's our copy)
@@ -259,16 +236,16 @@ uintptr_t SEXP_rawval_lblk_add (uintptr_t lblkp, const SEXP_t *s_exp)
 
         _A(lb_prev != 0);
         _A(lb_head != 0);
-        
+
         (void)SEXP_rawval_lblk_add1 (lb_prev, s_exp);
-        
+
         return (lb_head);
 }
 
 uintptr_t SEXP_rawval_lblk_add1 (uintptr_t lblkp, const SEXP_t *s_exp)
 {
         struct SEXP_val_lblk *lblk = SEXP_VALP_LBLK(lblkp);
-        
+
         if (lblk->real < (1 << (lblk->nxsz & 0xf))) {
 
                 lblk->memb[lblk->real].s_valp = SEXP_rawval_incref (s_exp->s_valp);
@@ -279,7 +256,7 @@ uintptr_t SEXP_rawval_lblk_add1 (uintptr_t lblkp, const SEXP_t *s_exp)
                 lblk->memb[lblk->real].__magic1 = s_exp->__magic1;
 #endif
                 ++lblk->real;
-                
+
                 return (lblkp);
         } else {
                 uint8_t   new_sz = ((lblk->nxsz & SEXP_LBLKS_MASK) + 1) % 16;
@@ -287,7 +264,7 @@ uintptr_t SEXP_rawval_lblk_add1 (uintptr_t lblkp, const SEXP_t *s_exp)
 
                 new_lb     = SEXP_rawval_lblk_new (new_sz);
                 lblk->nxsz = (new_lb & SEXP_LBLKP_MASK) | (lblk->nxsz & SEXP_LBLKS_MASK);
-                
+
                 /*
                  * We don't need to check the return value
                  * here because we are adding the S-exp to
@@ -295,10 +272,10 @@ uintptr_t SEXP_rawval_lblk_add1 (uintptr_t lblkp, const SEXP_t *s_exp)
                  * least one free slot.
                  */
                 SEXP_rawval_lblk_add1 (new_lb, s_exp);
-                
+
                 return (lblkp);
         }
-        
+
         /* NOTREACHED */
         return ((uintptr_t) NULL);
 }
@@ -310,33 +287,33 @@ uintptr_t SEXP_rawval_lblk_last (uintptr_t lblkp)
 
         last = lblkp;
         lblk = SEXP_VALP_LBLK(last);
-        
+
         while ((lblk->nxsz & SEXP_LBLKP_MASK) != ((uintptr_t)NULL)) {
                 last = lblk->nxsz;
                 lblk = SEXP_VALP_LBLK(last);
         }
-        
+
         return (last);
 }
 
 SEXP_t *SEXP_rawval_lblk_nth (uintptr_t lblkp, uint32_t n)
 {
         struct SEXP_val_lblk *lblk;
-        
+
         _LOGCALL_;
         _D("n = %u\n", n);
-        
+
         lblk = SEXP_VALP_LBLK(lblkp);
-        
+
         while (lblk != NULL) {
                 if (lblk->real >= n) {
                         return (lblk->memb + (n - 1));
                 } else {
-                        n   -= lblk->real; 
+                        n   -= lblk->real;
                         lblk = SEXP_VALP_LBLK(lblk->nxsz);
                 }
         }
-        
+
         return (NULL);
 }
 
@@ -346,14 +323,14 @@ uintptr_t SEXP_rawval_lblk_replace (uintptr_t lblkp, uint32_t n, const SEXP_t *n
         uintptr_t lb_head;
         struct SEXP_val_lblk *lblk;
         SEXP_t   *memb;
-        
+
         lblk = SEXP_VALP_LBLK(lblkp);
 
         _A(lblk != NULL);
 
         lb_head = lblkp;
         lb_prev = 0;
-        
+
         while (n > lblk->real) {
                 if (lblk->refs < 2) {
                         n      -= lblk->real;
@@ -366,17 +343,17 @@ uintptr_t SEXP_rawval_lblk_replace (uintptr_t lblkp, uint32_t n, const SEXP_t *n
                         }
                 } else {
                         uintptr_t lb_ptr;
-                        
+
                         /*
                          * We've encountered a block which belongs to more
                          * than one list so we have to create a copy of the
                          * rest of the list.
                          */
-                        lb_ptr = SEXP_rawval_list_copy ((uintptr_t)lblk, 0);
-                        
+                        lb_ptr = SEXP_rawval_lblk_copy ((uintptr_t)lblk, 0);
+
                         if (lb_prev == 0)
                                 lb_head = lb_ptr;
-                        
+
                         /*
                          * Update pointer in the previous block (if there is one)
                          * to point at the copy. Also decrement the reference
@@ -384,22 +361,22 @@ uintptr_t SEXP_rawval_lblk_replace (uintptr_t lblkp, uint32_t n, const SEXP_t *n
                          */
                         if (lb_prev != 0)
                                 SEXP_VALP_LBLK(lb_prev)->nxsz = (lb_ptr & SEXP_LBLKP_MASK) | (SEXP_VALP_LBLK(lb_prev)->nxsz & SEXP_LBLKS_MASK);
-                        
+
                         SEXP_rawval_lblk_decref ((uintptr_t)lblk);
                         memb = SEXP_rawval_lblk_nth (lb_ptr, n);
-                        
+
                         goto replace;
                 }
         }
-        
+
         _A(n > 0);
-        
+
         memb = lblk->memb + (n - 1);
 replace:
-        
+
         _A(lb_head != 0);
         _A(memb != NULL);
-        
+
         (*o_val) = SEXP_new ();
         (*o_val)->s_valp = memb->s_valp;
         (*o_val)->s_type = memb->s_type;
@@ -423,40 +400,60 @@ int SEXP_rawval_lblk_cb (uintptr_t lblkp, int (*func) (SEXP_t *, void *), void *
 {
         struct SEXP_val_lblk *lblk;
         uint16_t bi;
-        
+
         lblk = SEXP_VALP_LBLK(lblkp);
-        
+
         while (lblk != NULL) {
                 if (lblk->real >= n) {
                         int ret;
 
                         bi = (uint16_t)n - 1;
-                        
+
                         do {
                                 while (bi < lblk->real) {
                                         ret = func (lblk->memb + bi, arg);
-                                        
+
                                         if (ret != 0)
                                                 return (ret);
                                         else
                                                 ++bi;
                                 }
-                                
+
                                 lblk = SEXP_VALP_LBLK(lblk->nxsz);
                                 bi   = 0;
                         } while (lblk != NULL);
-                        
+
                         break;
                 } else {
                         n   -= lblk->real;
                         lblk = SEXP_VALP_LBLK(lblk->nxsz);
                 }
         }
-        
+
         return (0);
 }
 
-uintptr_t SEXP_rawval_list_copy (uintptr_t lblkp, uint16_t n_skip)
+uintptr_t SEXP_rawval_list_copy (uintptr_t s_valp)
+{
+        SEXP_val_t v_dsc_o, v_dsc_c;
+
+        if (SEXP_val_new (&v_dsc_c, sizeof (void *) + sizeof (uint16_t),
+                          SEXP_VALTYPE_LIST) != 0)
+        {
+                /* TODO: handle this */
+                return ((uintptr_t)NULL);
+        }
+
+        SEXP_val_dsc (&v_dsc_o, s_valp);
+
+        SEXP_LCASTP(v_dsc_c.mem)->b_addr = (void *) SEXP_rawval_lblk_copy ((uintptr_t)SEXP_LCASTP(v_dsc_o.mem)->b_addr,
+                                                                           (uintptr_t)SEXP_LCASTP(v_dsc_o.mem)->offset);
+        SEXP_LCASTP(v_dsc_c.mem)->offset = 0;
+
+        return (SEXP_val_ptr (&v_dsc_c));
+}
+
+uintptr_t SEXP_rawval_lblk_copy (uintptr_t lblkp, uint16_t n_skip)
 {
         struct SEXP_val_lblk *lb_new, *lb_old;
         uintptr_t lb_next;
@@ -465,7 +462,7 @@ uintptr_t SEXP_rawval_list_copy (uintptr_t lblkp, uint16_t n_skip)
         uint16_t  off_o;  /* offset in the old block */
         uint8_t  cur_sz;  /* size of the new block */
         uint8_t  old_sz;  /* size of the old block */
-        
+
         lb_head = 0;
         off_n   = 0;
         off_o   = n_skip;
@@ -473,26 +470,26 @@ uintptr_t SEXP_rawval_list_copy (uintptr_t lblkp, uint16_t n_skip)
 
         if (lb_old == NULL)
                 return ((uintptr_t) NULL);
-        
+
         old_sz  = lb_old->nxsz & SEXP_LBLKS_MASK;
         cur_sz  = 0;
         lb_new  = (struct SEXP_val_lblk *)SEXP_rawval_lblk_new (cur_sz);
         lb_head = (uintptr_t)lb_new;
-        
+
         while (lb_old != NULL) {
                 if ((lb_old->real - off_o) == 0) {
                         /*
                          * move to the next old block
                          */
                         lb_old = SEXP_VALP_LBLK(lb_old->nxsz);
-                        
+
                         if (lb_old == NULL)
                                 break;
-                        
+
                         off_o  = 0;
                         old_sz = lb_old->nxsz & SEXP_LBLKS_MASK;
                 }
-                
+
                 /*
                  * allocate new block
                  */
@@ -502,11 +499,11 @@ uintptr_t SEXP_rawval_list_copy (uintptr_t lblkp, uint16_t n_skip)
                         lb_new  = SEXP_VALP_LBLK(lb_next);
                         off_n   = 0;
                 }
-                
+
                 /*
                  * copy list items
                  */
-                while (off_n < (1 << cur_sz) && off_o < (1 << old_sz)) {
+                while (off_n < (1 << cur_sz) && off_o < lb_old->real) {
                         lb_new->memb[off_n].s_valp = SEXP_rawval_incref (lb_old->memb[off_o].s_valp);
                         lb_new->memb[off_n].s_type = lb_old->memb[off_o].s_type;
                         lb_new->memb[off_n].s_flgs = lb_old->memb[off_o].s_flgs;
@@ -514,13 +511,13 @@ uintptr_t SEXP_rawval_list_copy (uintptr_t lblkp, uint16_t n_skip)
                         lb_new->memb[off_n].__magic0 = lb_old->memb[off_o].__magic0;
                         lb_new->memb[off_n].__magic1 = lb_old->memb[off_o].__magic1;
 #endif
-                        
+
                         ++off_n;
                         ++off_o;
                         ++lb_new->real;
                 }
         }
-        
+
         return (lb_head);
 }
 
@@ -528,17 +525,17 @@ void SEXP_rawval_lblk_free (uintptr_t lblkp, void (*func) (SEXP_t *))
 {
         if (SEXP_rawval_lblk_decref (lblkp)) {
                 struct SEXP_val_lblk *lblk, *next;
-                
+
                 lblk = SEXP_VALP_LBLK(lblkp);
                 next = SEXP_VALP_LBLK(lblk->nxsz);
-                
+
                 while (lblk->real > 0) {
                         --lblk->real;
                         func (lblk->memb + lblk->real);
                 }
-                
+
                 sm_free (lblk);
-                
+
                 if (next != NULL)
                         SEXP_rawval_lblk_free ((uintptr_t)next, func);
         }
@@ -550,17 +547,16 @@ void SEXP_rawval_lblk_free1 (uintptr_t lblkp, void (*func) (SEXP_t *))
 {
         if (SEXP_rawval_lblk_decref (lblkp)) {
                 struct SEXP_val_lblk *lblk;
-                
+
                 lblk = SEXP_VALP_LBLK(lblkp);
-                
+
                 while (lblk->real > 0) {
                         --lblk->real;
                         func (lblk->memb + lblk->real);
                 }
-                
+
                 sm_free (lblk);
         }
-        
+
         return;
 }
-#endif
