@@ -1,3 +1,13 @@
+/**
+ * @file   xmlfilecontent.c
+ * @brief  xmlfilecontent probe
+ * @author "Tomas Heinrich" <theinric@redhat.com>
+ *
+ * 2010/06/13 dkopecek@redhat.com
+ *  This probe is able to process an xmlfilecontent_object as defined in OVAL 5.4 and 5.5.
+ *
+ */
+
 /*
  * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
@@ -59,7 +69,7 @@ struct pfdata {
 	SEXP_t *item_list;
 };
 
-void dummy_err_func(void * ctx, const char * msg, ...)
+static void dummy_err_func(void * ctx, const char * msg, ...)
 {
 }
 
