@@ -1,8 +1,11 @@
 /**
- * @file
+ * @addtogroup PROBEAPI
+ * @{
+ * @file   probe-api.c
+ * @brief  Probe API implmentation
  * @author "Daniel Kopecek" <dkopecek@redhat.com>
+ * @author "Tomas Heinrich" <theinric@redhat.com>
  */
-
 /*
  * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
@@ -23,6 +26,7 @@
  *
  * Authors:
  *      Daniel Kopecek <dkopecek@redhat.com>
+ *      Tomas Heinrich <theinric@redhat.com>
  */
 
 #include <stdarg.h>
@@ -30,59 +34,9 @@
 #include <errno.h>
 #include "_probe-api.h"
 
-/*
-This file contains functions for manipulating with the S-exp representation of OVAL objects and items.
-Currently object and items have the same structure and the API distinction is just formal. However, the
-structure can diverge in the future and the API functions for manipulating with items should be used
-only with items and vice versa. The most recent description of the object and item structure can be found
-in this file and should be used as the main source of information for implementing new API functions.
-
-Object structure
-----------------
-
-The basic structure of an object is
-
-   (foo_object bar)
-
-where foo_object is the object's name and bar is the value of this object. The value is optional.
-In the case the object has some attributes the structure is
-
-   ((foo_object :attr1 val1) bar)
-
-where attr1 is the name of the attribute and val1 is the attribute's value. The colon in the
-attribute name signals that this attribute has a value. 
-
-*/
-
-#if 0				/* experimental */
-/*
- * Initialization
- */
-
-int probe_api_init(void)
-{
-	/* initialize name cache */
-	return (0);
-}
-
-int probe_api_deinit(void)
-{
-	/* destroy name cache */
-	return (0);
-}
-#endif
-
 /*
  * items
  */
-
-SEXP_t *probe_item_build(const char *fmt, ...)
-{
-	_LOGCALL_;
-	/* TBI */
-	return (NULL);
-}
-
 SEXP_t *probe_item_creat(const char *name, SEXP_t * attrs, ...)
 {
 	va_list ap;
@@ -215,26 +169,31 @@ SEXP_t *probe_item_ent_add(SEXP_t *item, const char *name, SEXP_t *attrs, SEXP_t
 
 int probe_item_setstatus(SEXP_t *obj, oval_syschar_status_t status)
 {
+        SEXP_t *r0;
+
 	_A(obj != NULL);
 	_LOGCALL_;
 
-	probe_item_attr_add(obj, "status", SEXP_number_newi_32((int) status));
+	probe_item_attr_add(obj, "status", r0 = SEXP_number_newi_32((int) status));
+        SEXP_free(r0);
+
 	return (0);
 }
 
 int probe_itement_setstatus(SEXP_t *obj, const char *name, uint32_t n, oval_syschar_status_t status)
 {
-        SEXP_t *ent_h, *ent_s;
+        SEXP_t *ent_h, *ent_s, *r0;
 	_LOGCALL_;
-	
+
         ent_h = probe_item_getent (obj, name, n);
         ent_s = SEXP_unref (ent_h);
-        
+
         _A(ent_s != NULL);
-        
-        probe_ent_attr_add (ent_s, "status", SEXP_number_newi_32 ((int) status));
+
+        probe_ent_attr_add (ent_s, "status", r0 = SEXP_number_newi_32 ((int) status));
         SEXP_free (ent_s);
-        
+        SEXP_free (r0);
+
         return (0);
 }
 
@@ -299,13 +258,6 @@ SEXP_t *probe_attr_creat(const char *name, const SEXP_t * val, ...)
  * objects
  */
 
-SEXP_t *probe_obj_build(const char *fmt, ...)
-{
-	_LOGCALL_;
-	/* TBI */
-	return (NULL);
-}
-
 SEXP_t *probe_obj_creat(const char *name, SEXP_t * attrs, ...)
 {
 	va_list ap;
@@ -540,18 +492,23 @@ bool probe_obj_attrexists(const SEXP_t * obj, const char *name)
 
 int probe_obj_setstatus(SEXP_t * obj, oval_syschar_status_t status)
 {
+        SEXP_t *r0;
 	_LOGCALL_;
 
-	probe_item_attr_add(obj, "status", SEXP_number_newi_32(status));
+	probe_item_attr_add(obj, "status", r0 = SEXP_number_newi_32(status));
+        SEXP_free(r0);
+
 	return (-1);
 }
 
+#if 0
 static int probe_obj_setentstatus(SEXP_t * obj, const char *name, uint32_t n, oval_syschar_status_t status)
 {
 	_LOGCALL_;
 	/* TBI */
 	return (-1);
 }
+#endif
 
 char *probe_obj_getname(const SEXP_t * obj)
 {
@@ -794,7 +751,6 @@ oval_syschar_collection_flag_t _probe_cobj_combine_flags(oval_syschar_collection
 /*
  * entities
  */
-
 SEXP_t *probe_ent_creat(const char *name, SEXP_t * attrs, SEXP_t * val, ...)
 {
 	va_list ap;
@@ -922,10 +878,9 @@ SEXP_t *probe_ent_getattrval(const SEXP_t * ent, const char *name)
 						}
 					}
 				}
-
-				SEXP_free(attr);
 			}
 
+                        SEXP_free(attr);
 			++i;
 		}
 	}
@@ -1067,9 +1022,12 @@ bool probe_ent_getmask(const SEXP_t * ent)
 
 int probe_ent_setstatus(SEXP_t * ent, oval_syschar_status_t status)
 {
+        SEXP_t *r0;
 	_LOGCALL_;
-        
-        probe_item_attr_add (ent, "status", SEXP_number_newi_32(status));
+
+        probe_item_attr_add (ent, "status", r0 = SEXP_number_newi_32(status));
+        SEXP_free(r0);
+
 	return (0);
 }
 
@@ -1077,18 +1035,18 @@ oval_syschar_status_t probe_ent_getstatus(const SEXP_t * ent)
 {
         SEXP_t *val;
         oval_syschar_status_t sta;
-        
+
 	_LOGCALL_;
-	
+
         val = probe_ent_getattrval (ent, "status");
-        
+
         if (val != NULL) {
                 sta = (oval_syschar_status_t) SEXP_number_geti_32 (val);
                 SEXP_free (val);
         } else {
                 sta = OVAL_STATUS_EXISTS;
         }
-        
+
 	return (sta);
 }
 
