@@ -1,19 +1,10 @@
 /**
- * @addtogroup OVAL
- * @{
- * @addtogroup PROBES
- * Probes and S-expression handling for probe writes.
- * @{
- * @addtogroup FINDFILE Find Files
- *  Auxiliary find_files() function 
- * @{
- * @{
- *
- * @file
- *
+ * @file   findfile.h
  * @author "Peter Vrabec" <pvrabec@redhat.com>
+ *
+ * @addtogroup PROBEAUXAPI
+ * @{
  */
-
 /*
  * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
@@ -44,7 +35,16 @@
 #include "fsdev.h"
 
 #define MTAB_PATH "/etc/mtab"
-#define LOCAL_FILESYSTEMS {"ext2", "ext3", "reiserfs", "xfs", NULL}
+#define LOCAL_FILESYSTEMS { "btrfs",            \
+                        "ext2", "ext3", "ext4", \
+                        "ffs",                  \
+                        "gfs",                  \
+                        "hpfs",                 \
+                        "jfs",                  \
+                        "reiser4", "reiserfs",  \
+                        "ufs",                  \
+                        "xfs",                  \
+                        "zfs", NULL}
 
 /**
  * Search the filesystem according to the specified path, filename and behaviors.
@@ -55,6 +55,6 @@
  * @param cb for each match, the callback function cb is called with first argument being the matched path and the second being the matched filename
  * @param arg an optional argument to the callback function
  */
-int find_files(SEXP_t * path, SEXP_t * filename, SEXP_t * behaviors,
-	       int (*cb) (const char *, const char *, void *arg), void *arg);
+int find_files(SEXP_t *spath, SEXP_t *sfilename, SEXP_t *behaviors,
+	       int (*cb) (const char *, const char *, void *), void *arg);
 #endif
