@@ -1,20 +1,37 @@
 /**
- * @addtogroup OVAL
- * @{
- * @addtogroup PROBES
- * Probes and S-expression handling for probe writes.
- * @{
- * @addtogroup PROBESAPI Main probe API
- * Most of the operations that need to be done in a probe, processing input object, 
- * manipulating entities, creating output items.
- * @{
- * @{
- *
- * @file
- *
+ * @file   probe-api.h
+ * @brief  Probe API public header
  * @author "Daniel Kopecek" <dkopecek@redhat.com>
+ * @author "Tomas Heinrich" <theinric@redhat.com>
+ *
+ * @addtogroup PROBEAPI
+ * This file contains functions for manipulating with the S-exp representation
+ * of OVAL objects and items. Currently object and items have the same structure
+ * and the API distinction is just formal. However, the structure can diverge in
+ * the future and the API functions for manipulating with items should be used
+ * only with items and vice versa. The most recent description of the object and
+ * item structure can be found in this file and should be used as the main source
+ * of information for implementing new API functions. In the following text, the
+ * term `element' referers to the general structure which is used to represent
+ * the various components of an OVAL document, particularly the OVAL objects,
+ * items and entities.
+ *
+ * \paragraph Element structure
+ * The basic structure of an element looks like this:\n
+ * \n
+ * (foo_object bar)\n
+ * \n
+ * \em foo_object is the element name and \em bar is the value of the element. There
+ * can be 0 to n values. In case the element has some attributes set the structure
+ * changes to:\n
+ * \n
+ * ((foo_object :attr1 val1) bar)\n
+ * \n
+ * where \em attr1 is the name of the attribute and \em val1 is the attribute's value.
+ * The colon in the attribute name signals that the attribute has a value.
+ *
+ * @{
  */
-
 /*
  * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
@@ -386,21 +403,21 @@ size_t probe_ent_getname_r(const SEXP_t * ent, char *buffer, size_t buflen);
  */
 void probe_free(SEXP_t * obj);
 
-#define PROBE_EINVAL     1	/* Invalid type/value/format */
-#define PROBE_ENOELM     2	/* Missing element OBSOLETE: use ENOENT */
-#define PROBE_ENOVAL     3	/* Missing value */
-#define PROBE_ENOATTR    4	/* Missing attribute */
-#define PROBE_EINIT      5	/* Initialization failed */
-#define PROBE_ENOMEM     6	/* No memory */
-#define PROBE_EOPNOTSUPP 7	/* Not supported */
-#define PROBE_ERANGE     8	/* Out of range */
-#define PROBE_EDOM       9	/* Out of domain */
-#define PROBE_EFAULT    10	/* Memory fault/NULL value */
-#define PROBE_EACCES    11	/* Operation not perimitted */
-#define PROBE_ESETEVAL  12	/* Set evaluation failed */
-#define PROBE_ENOENT    13      /* Missing entity */
-#define PROBE_EFATAL   254	/* Unrecoverable error */
-#define PROBE_EUNKNOWN 255	/* Unknown/Unexpected error */
+#define PROBE_EINVAL     1	/**< Invalid type/value/format */
+#define PROBE_ENOELM     2	/**< Missing element OBSOLETE: use ENOENT */
+#define PROBE_ENOVAL     3	/**< Missing value */
+#define PROBE_ENOATTR    4	/**< Missing attribute */
+#define PROBE_EINIT      5	/**< Initialization failed */
+#define PROBE_ENOMEM     6	/**< No memory */
+#define PROBE_EOPNOTSUPP 7	/**< Not supported */
+#define PROBE_ERANGE     8	/**< Out of range */
+#define PROBE_EDOM       9	/**< Out of domain */
+#define PROBE_EFAULT    10	/**< Memory fault/NULL value */
+#define PROBE_EACCES    11	/**< Operation not perimitted */
+#define PROBE_ESETEVAL  12	/**< Set evaluation failed */
+#define PROBE_ENOENT    13      /**< Missing entity */
+#define PROBE_EFATAL   254	/**< Unrecoverable error */
+#define PROBE_EUNKNOWN 255	/**< Unknown/Unexpected error */
 
 #define OVAL_STATUS_UNKNOWN      SYSCHAR_STATUS_UNKNOWN
 #define OVAL_STATUS_ERROR        SYSCHAR_STATUS_ERROR
@@ -408,8 +425,9 @@ void probe_free(SEXP_t * obj);
 #define OVAL_STATUS_DOESNOTEXIST SYSCHAR_STATUS_DOES_NOT_EXIST
 #define OVAL_STATUS_NOTCOLLECTED SYSCHAR_STATUS_NOT_COLLECTED
 
-#define PROBECMD_STE_FETCH 1
-#define PROBECMD_OBJ_EVAL  2
+#define PROBECMD_STE_FETCH 1 /**< State fetch command code */
+#define PROBECMD_OBJ_EVAL  2 /**< Object eval command code */
+#define PROBECMD_RESET     3 /**< Reset command code */
 
 void *probe_init(void) __attribute__ ((unused));
 void probe_fini(void *) __attribute__ ((unused));
