@@ -1,3 +1,12 @@
+/**
+ * @file   rpminfo.c
+ * @brief  rpminfo probe
+ * @author "Daniel Kopecek" <dkopecek@redhat.com>
+ *
+ * 2010/06/13 dkopecek@redhat.com
+ *  This probe is able to process a rpminfo_object as defined in OVAL 5.4 and 5.5.
+ *
+ */
 
 /*
  * Copyright 2009 Red Hat Inc., Durham, North Carolina.
@@ -47,6 +56,8 @@
 #include <rpm/rpmdb.h>
 #include <rpm/rpmlib.h>
 #include <rpm/rpmts.h>
+#include <rpm/rpmmacro.h>
+#include <rpm/rpmlog.h>
 
 /* SEAP */
 #include <seap.h>
@@ -247,6 +258,10 @@ void probe_fini (void *ptr)
 
         rpmdbClose (r->rpmdb);
 	rpmFreeCrypto();
+        rpmFreeRpmrc();
+        rpmFreeMacros(NULL);
+        rpmlogClose();
+        rpmFreeFilesystems();
         pthread_mutex_destroy (&(r->mutex));
 
         return;
