@@ -1,3 +1,12 @@
+/**
+ * @file   process.c
+ * @brief  process probe
+ * @author "Steve Grubb" <sgrubb@redhat.com>
+ *
+ * 2010/06/13 dkopecek@redhat.com
+ *  This probe is able to process an process_object as defined in OVAL 5.4 and 5.5.
+ *
+ */
 
 /*
  * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
@@ -293,6 +302,8 @@ static int read_process(SEXP_t *cmd_ent, SEXP_t *probe_out)
 		}
 		SEXP_free(cmd_sexp);
 	}
+        closedir(d);
+
 	return err;
 }
 
