@@ -1,3 +1,14 @@
+/**
+ * @file   runlevel.c
+ * @brief  runlevel probe
+ * @author "Daniel Kopecek" <dkopecek@redhat.com>
+ * @author "Tomas Heinrich" <theinric@redhat.com>
+ *
+ * 2010/06/13 dkopecek@redhat.com
+ *  This probe is able to process a runlevel_object as defined in OVAL 5.4 and 5.5.
+ *
+ */
+
 /*
  * Copyright 2009 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
@@ -18,6 +29,7 @@
  *
  * Authors:
  *      "Daniel Kopecek" <dkopecek@redhat.com>
+ *      "Tomas Heinrich" <theinric@redhat.com>
  */
 
 /*
@@ -97,7 +109,7 @@ static int get_runlevel_redhat (struct runlevel_req *req, struct runlevel_rep **
 
         while ((init_dp = readdir(init_dir)) != NULL) {
 		char *service_name;
-		int i;
+		unsigned int i;
 		SEXP_t *r0;
 
                 if (stat(init_dp->d_name, &init_st) != 0) {
@@ -393,6 +405,8 @@ SEXP_t *probe_main (SEXP_t *object, int *err, void *arg)
 		}
         }
 
+        SEXP_free(request_st.runlevel_ent);
+        SEXP_free(request_st.service_name_ent);
         *err = 0;
 
         return (probe_out);
