@@ -1,3 +1,13 @@
+/**
+ * @file   shadow.c
+ * @brief  shadow probe
+ * @author "Steve Grubb" <sgrubb@redhat.com>
+ *
+ * 2010/06/13 dkopecek@redhat.com
+ *  This probe is able to process a shadow_object as defined in OVAL 5.4 and 5.5.
+ *
+ */
+
 /*
  * Copyright 2009-2010 Red Hat Inc., Durham, North Carolina.
  * All Rights Reserved.
@@ -140,9 +150,12 @@ SEXP_t *probe_main(SEXP_t *object, int *err, void *arg)
 
 	// Now we check the file...
 	if (read_shadow(ent, probe_out)) {
-		*err = PROBE_EACCES;
-		SEXP_free(probe_out);
-		probe_out = NULL;
+                SEXP_t *eitm;
+
+                eitm = probe_item_creat ("shadow_item", NULL, NULL);
+                probe_item_setstatus (eitm, OVAL_STATUS_ERROR);
+                SEXP_list_add(probe_out, eitm);
+                SEXP_free(eitm);
 	}
 
 	SEXP_free(ent);
