@@ -188,12 +188,6 @@ struct oval_variable_binding_iterator;
 
 
 
-/**
- * Import the content from a specified XML stream into a oval_syschar_model, return -1 if an error occurred.
- * If the input_source specifies a model entity that is already registered within the model its content is overwritten.
- * @memberof oval_syschar_model
- */
-int oval_syschar_model_import(struct oval_syschar_model *model, struct oscap_import_source *source);
 /**
  * Create new oval_syschar_model.
  * The new model is bound to a specified oval_definition_model and variable bindings.
@@ -201,6 +195,16 @@ int oval_syschar_model_import(struct oval_syschar_model *model, struct oscap_imp
  * @memberof oval_syschar_model
  */
 struct oval_syschar_model *oval_syschar_model_new(struct oval_definition_model *definition_model);
+
+/**
+ * Import the content from the file into an oval_syschar_model.
+ * If imported content specifies a model entity that is already registered within the model its content is overwritten.
+ * @param model the merge target model
+ * @param file filename
+ * @return -1 if an error occurred
+ * @memberof oval_syschar_model
+ */
+int oval_syschar_model_import(struct oval_syschar_model *model, const char *file);
 /**
  * Copy an oval_syschar_model.
  * @return A copy of the specified @ref oval_syschar_model.
@@ -208,12 +212,12 @@ struct oval_syschar_model *oval_syschar_model_new(struct oval_definition_model *
  */
 struct oval_syschar_model *oval_syschar_model_clone(struct oval_syschar_model *);
 /**
- * Export system characteristics as a XML file.
+ * Export system characteristics into file.
  * @memberof oval_syschar_model
  */
-int oval_syschar_model_export(struct oval_syschar_model *, struct oscap_export_target *);
+int oval_syschar_model_export(struct oval_syschar_model *, const char *file);
 /**
- * free memory allocated to a specified syschar model.
+ * Free memory allocated to a specified syschar model.
  * @param model the specified syschar model
  * @memberof oval_syschar_model
  */
@@ -282,22 +286,17 @@ struct oval_sysinfo *oval_syschar_model_get_sysinfo(struct oval_syschar_model *m
  * @param object_id the specified object_id.
  * @memberof oval_syschar_model
  */
-struct oval_syschar *oval_syschar_model_get_syschar(struct oval_syschar_model *model, char *object_id);
-/**
- * Get the collection flag associated with a specified oval_variable
- * @memberof oval_syschar_model
- */
-oval_syschar_collection_flag_t oval_syschar_model_get_variable_collection_flag(struct oval_syschar_model *,
-									       struct oval_variable *);
+struct oval_syschar *oval_syschar_model_get_syschar(struct oval_syschar_model *model, const char *object_id);
 /**
  * Get the oval_values bound to a specified variable.
  * @memberof oval_syschar_model
  */
-struct oval_value_iterator *oval_syschar_model_get_variable_values(struct oval_syschar_model *, struct oval_variable *);
+int oval_syschar_model_compute_variable(struct oval_syschar_model *, struct oval_variable *);
+oval_syschar_collection_flag_t oval_variable_get_collection_flag(struct oval_variable *);
 /**
  * @memberof oval_syschar_model
  */
-struct oval_sysdata *oval_syschar_model_get_sysdata(struct oval_syschar_model *, char *);
+struct oval_sysdata *oval_syschar_model_get_sysdata(struct oval_syschar_model *, const char *);
 /**
  * Return <b>true</b> if the syschar_model instance is locked.
  * The state of a locked instance cannot be changed.
@@ -310,19 +309,6 @@ bool oval_syschar_model_is_locked(struct oval_syschar_model *syschar_model);
  * @name Evaluators
  * @{
  */
-/**
- * Probe all oval_objects from oval_definition model that was bound to specific oval_syschar_model. Sucessfull 
- * finish of this functions result in oval_syschar_model filled out by system characteristic to each object.
- * @memberof oval_syschar_model
- * return 0 on sucess and -1 on fail. Use \ref ERRORS mechanism to examine the error.
- */
-int oval_syschar_model_probe_objects(struct oval_syschar_model *);
-/**
- * Gather system information and add it to oval_syschar_model.
- * @memberof oval_syschar_model
- * @return 0 on sucess and -1 on fail. Use \ref ERRORS mechanism to examine the error.
- */
-int oval_syschar_model_probe_sysinfo(struct oval_syschar_model *);
 /**
  * Return <b>true</b> if the syschar_model instance is valid
  * An oval_syschar_model is valid if all the following are true
@@ -672,7 +658,7 @@ bool oval_sysint_is_valid(struct oval_sysint *sysint);
 /**
  * @memberof oval_sysdata
  */
-struct oval_sysdata *oval_sysdata_new(struct oval_syschar_model *, char *id);
+struct oval_sysdata *oval_sysdata_new(struct oval_syschar_model *, const char *id);
 /**
  * @return A copy of the specified @ref oval_sysdata.
  * @memberof oval_sysdata
