@@ -35,9 +35,9 @@ struct xccdf_backref {
 	char *id;		// id
 };
 
-struct xccdf_benchmark *xccdf_benchmark_import(struct oscap_import_source *source)
+struct xccdf_benchmark *xccdf_benchmark_import(const char *file)
 {
-	xmlTextReaderPtr reader = xmlReaderForFile(oscap_import_source_get_name(source), NULL, 0);
+	xmlTextReaderPtr reader = xmlReaderForFile(file, NULL, 0);
 	if (!reader)
 		return NULL;
 	while (xmlTextReaderRead(reader) == 1 && xmlTextReaderNodeType(reader) != 1) ;
@@ -69,10 +69,10 @@ struct xccdf_benchmark *xccdf_benchmark_clone(const struct xccdf_benchmark *old_
 {
 	struct xccdf_item *new_benchmark = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_benchmark_item));
 	struct xccdf_item *old = XITEM(old_benchmark);
-	new_benchmark->item = *(xccdf_item_base_clone(&(old->item)));
+    xccdf_item_base_clone(&new_benchmark->item, &old->item);
 	new_benchmark->type = old->type;
 	//second argument is a pointer to the benchmark being created which will be the parent of all of its sub elements.
-	new_benchmark->sub.benchmark = *(xccdf_benchmark_item_clone(&(old->sub.benchmark), new_benchmark));
+    xccdf_benchmark_item_clone(&new_benchmark->sub.benchmark, old_benchmark);
 	return XBENCHMARK(new_benchmark);
 }
 
@@ -145,9 +145,9 @@ bool xccdf_benchmark_parse(struct xccdf_item * benchmark, xmlTextReaderPtr reade
 	return true;
 }
 
-int xccdf_benchmark_export(struct xccdf_benchmark *benchmark, struct oscap_export_target *target)
+int xccdf_benchmark_export(struct xccdf_benchmark *benchmark, const char *file)
 {
-	__attribute__nonnull__(target);
+	__attribute__nonnull__(file);
 
 	int retcode = 0;
 
@@ -161,7 +161,7 @@ int xccdf_benchmark_export(struct xccdf_benchmark *benchmark, struct oscap_expor
 
 	xccdf_benchmark_to_dom(benchmark, doc, NULL, NULL);
 
-	retcode = xmlSaveFormatFileEnc(oscap_export_target_get_name(target), doc, oscap_export_target_get_encoding(target), 1);
+	retcode = xmlSaveFormatFileEnc(file, doc, "UTF-8", 1);
 	if (retcode < 1)
 		oscap_setxmlerr(xmlGetLastError());
 
@@ -212,12 +212,7 @@ xmlNode *xccdf_benchmark_to_dom(struct xccdf_benchmark *benchmark, xmlDocPtr doc
 	struct xccdf_notice_iterator *notices = xccdf_benchmark_get_notices(benchmark);
 	while (xccdf_notice_iterator_has_more(notices)) {
 		struct xccdf_notice *notice = xccdf_notice_iterator_next(notices);
-                struct oscap_text *text = xccdf_notice_get_text(notice);
-		xmlNode *notice_node = xmlNewChild(root_node, ns_xccdf, BAD_CAST "notice", BAD_CAST oscap_text_get_text(text));
-
-		const char *lang = oscap_text_get_lang(text);
-		if (lang)
-			xmlNewProp(notice_node, BAD_CAST "xml:lang", BAD_CAST lang);
+		xmlNode *notice_node = oscap_text_to_dom(xccdf_notice_get_text(notice), root_node, "notice");
 
 		const char *id = xccdf_notice_get_id(notice);
 		if (id)
@@ -225,31 +220,8 @@ xmlNode *xccdf_benchmark_to_dom(struct xccdf_benchmark *benchmark, xmlDocPtr doc
 	}
 	xccdf_notice_iterator_free(notices);
 
-	struct oscap_text_iterator *fronts = xccdf_benchmark_get_front_matter(benchmark);
-	while (oscap_text_iterator_has_more(fronts)) {
-		struct oscap_text *front = oscap_text_iterator_next(fronts);
-		xmlNode *front_node = xmlNewChild(root_node, ns_xccdf, BAD_CAST "front-matter", BAD_CAST oscap_text_get_text(front));
-
-		const char *lang = oscap_text_get_lang(front);
-		if (lang)
-			xmlNewProp(front_node, BAD_CAST "xml:lang", BAD_CAST lang);
-
-		/* TODO: Dublin Core */
-	}
-	oscap_text_iterator_free(fronts);
-
-	struct oscap_text_iterator *rears = xccdf_benchmark_get_rear_matter(benchmark);
-	while (oscap_text_iterator_has_more(rears)) {
-		struct oscap_text *rear = oscap_text_iterator_next(rears);
-		xmlNode *rear_node = xmlNewChild(root_node, ns_xccdf, BAD_CAST "rear-matter", BAD_CAST oscap_text_get_text(rear));
-
-		const char *lang = oscap_text_get_lang(rear);
-		if (lang)
-			xmlNewProp(rear_node, BAD_CAST "xml:lang", BAD_CAST lang);
-
-		/* TODO: Dublin Core */
-	}
-	oscap_text_iterator_free(rears);
+	xccdf_texts_to_dom(xccdf_benchmark_get_front_matter(benchmark), root_node, "front-matter");
+	xccdf_texts_to_dom(xccdf_benchmark_get_rear_matter(benchmark), root_node, "rear-matter");
 
 	struct oscap_string_iterator *platforms = xccdf_benchmark_get_platforms(benchmark);
 	while (oscap_string_iterator_has_more(platforms)) {
@@ -267,6 +239,11 @@ xmlNode *xccdf_benchmark_to_dom(struct xccdf_benchmark *benchmark, xmlDocPtr doc
 		xmlNewChild(metadata_node, ns_xccdf, NULL, BAD_CAST metadata);
 	}
 
+	OSCAP_FOR(xccdf_model, model, xccdf_benchmark_get_models(benchmark)) {
+		xmlNode *model_node = xmlNewChild(root_node, ns_xccdf, BAD_CAST "model", NULL);
+		xmlNewProp(model_node, BAD_CAST "system", BAD_CAST xccdf_model_get_system(model));
+	}
+
 	struct xccdf_profile_iterator *profiles = xccdf_benchmark_get_profiles(benchmark);
 	while (xccdf_profile_iterator_has_more(profiles)) {
 		struct xccdf_profile *profile = xccdf_profile_iterator_next(profiles);
@@ -445,23 +422,29 @@ const char * xccdf_benchmark_supported(void)
     return XCCDF_SUPPORTED;
 }
 
-struct xccdf_group *xccdf_benchmark_append_new_group(const struct xccdf_benchmark *benchmark, const char *id)
+struct xccdf_group *xccdf_benchmark_append_new_group(struct xccdf_benchmark *benchmark, const char *id)
 {
-    //TODO: not implemented
-    oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented");
-    return NULL;
+	if (benchmark == NULL) return NULL;
+	struct xccdf_group *group = xccdf_group_new();
+	xccdf_group_set_id(group, id);
+	xccdf_benchmark_add_group(benchmark, group);
+    return group;
 }
-struct xccdf_value *xccdf_benchmark_append_new_value(const struct xccdf_benchmark *benchmark, const char *id, xccdf_value_type_t type)
+struct xccdf_value *xccdf_benchmark_append_new_value(struct xccdf_benchmark *benchmark, const char *id, xccdf_value_type_t type)
 {
-    //TODO: not implemented
-    oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented");
-    return NULL;
+	if (benchmark == NULL) return NULL;
+	struct xccdf_value *value = xccdf_value_new(type);
+	xccdf_value_set_id(value, id);
+	xccdf_benchmark_add_value(benchmark, value);
+    return value;
 }
-struct xccdf_rule *xccdf_benchmark_append_new_rule(const struct xccdf_benchmark *benchmark, const char *id)
+struct xccdf_rule *xccdf_benchmark_append_new_rule(struct xccdf_benchmark *benchmark, const char *id)
 {
-    //TODO: not implemented
-    oscap_seterr(OSCAP_EFAMILY_OSCAP, OSCAP_ENOTIMPL, "This feature is not implemented");
-    return NULL;
+	if (benchmark == NULL) return NULL;
+	struct xccdf_rule *rule = xccdf_rule_new();
+	xccdf_rule_set_id(rule, id);
+	xccdf_benchmark_add_rule(benchmark, rule);
+    return rule;
 }
 
 static const size_t XCCDF_ID_SIZE = 32;
@@ -507,13 +490,12 @@ bool xccdf_add_item(struct oscap_list *list, struct xccdf_item *parent, struct x
 
 bool xccdf_benchmark_register_item(struct xccdf_benchmark *benchmark, struct xccdf_item *item)
 {
-	assert(benchmark != NULL);
-
-	if (item == NULL || xccdf_item_get_id(item) == NULL)
+	if (benchmark == NULL || item == NULL || xccdf_item_get_id(item) == NULL)
 		return false;
 
-	if (xccdf_item_get_benchmark(item) != NULL)
-		return xccdf_item_get_benchmark(item) == benchmark; // already registered
+	const char *id = xccdf_item_get_id(item);
+	struct xccdf_item *found = xccdf_benchmark_get_item(benchmark, id);
+	if (found != NULL) return found == item; // already registered
 
 	return oscap_htable_add(XITEM(benchmark)->sub.benchmark.dict, xccdf_item_get_id(item), item);
 }
