@@ -106,27 +106,29 @@ struct xccdf_item *xccdf_item_clone(const struct xccdf_item *old_item)
 {
 	struct xccdf_item *new_item = oscap_calloc(1, sizeof(struct xccdf_item));
 
-	new_item->item = *(xccdf_item_base_clone(&(old_item->item)));
+	//new_item->item = *(xccdf_item_base_clone(&(old_item->item)));
+    xccdf_item_base_clone(&new_item->item, &(old_item->item));
 	new_item->type = old_item->type;
 
 	switch (new_item->type) {
 	case XCCDF_BENCHMARK:
-		 new_item->sub.benchmark = *(xccdf_benchmark_item_clone(&(old_item->sub.benchmark), new_item));
+		 //new_item->sub.benchmark = *(xccdf_benchmark_item_clone(&(old_item->sub.benchmark), new_item));
+        xccdf_benchmark_item_clone(&new_item->sub.benchmark, XBENCHMARK(old_item));
 		break;
 	case XCCDF_RULE:
-		new_item->sub.rule = *(xccdf_rule_item_clone(&(old_item->sub.rule)));
+	    xccdf_rule_item_clone(&new_item->sub.rule, &old_item->sub.rule);
 		break;
 	case XCCDF_GROUP:
-		new_item->sub.group = *(xccdf_group_item_clone(&(old_item->sub.group), new_item));
+	    xccdf_group_item_clone(new_item, &old_item->sub.group);
 		break;
 	case XCCDF_VALUE:
-		new_item->sub.value = *(xccdf_value_item_clone(&(old_item->sub.value)));
+	    xccdf_value_item_clone(&new_item->sub.value, &old_item->sub.value);
 		break;
 	case XCCDF_RESULT:
-		new_item->sub.result = *(xccdf_result_item_clone(&(old_item->sub.result)));
+	    xccdf_result_item_clone(&new_item->sub.result, &old_item->sub.result);
 		break;
 	case XCCDF_PROFILE:
-		new_item->sub.profile = *(xccdf_profile_item_clone(&(old_item->sub.profile)));
+	    xccdf_profile_item_clone(&new_item->sub.profile, &old_item->sub.profile);
 		break;
 	default:
 		//dont initialize the sub item.
@@ -137,9 +139,8 @@ struct xccdf_item *xccdf_item_clone(const struct xccdf_item *old_item)
 }
 
 /* Performs a deep copy of xccdf_item_base and returns a pointer to that copy */
-struct xccdf_item_base *xccdf_item_base_clone(const struct xccdf_item_base *old_base)
+void xccdf_item_base_clone(struct xccdf_item_base *new_base, const struct xccdf_item_base *old_base)
 {
-	struct xccdf_item_base *new_base = oscap_calloc(1, sizeof(struct xccdf_item_base));
 	new_base->id = oscap_strdup(old_base->id);
 	new_base->cluster_id = oscap_strdup(old_base->cluster_id);
 	new_base->version = oscap_strdup(old_base->version);
@@ -161,15 +162,8 @@ struct xccdf_item_base *xccdf_item_base_clone(const struct xccdf_item_base *old_
 	new_base->platforms = oscap_list_clone(old_base->platforms, (oscap_clone_func) oscap_strdup);
 
 	/* Handling flags */
-	new_base->flags.selected = old_base->flags.selected;
-	new_base->flags.hidden = old_base->flags.hidden;
-	new_base->flags.resolved = old_base->flags.resolved;
-	new_base->flags.abstract = old_base->flags.abstract;
-	new_base->flags.prohibit_changes = old_base->flags.prohibit_changes;
-	new_base->flags.interactive = old_base->flags.interactive;
-	new_base->flags.multiple = old_base->flags.multiple;
-
-	return new_base;
+	new_base->flags = old_base->flags;
+	new_base->defined_flags = old_base->defined_flags;
 }
 
 
@@ -178,10 +172,11 @@ struct xccdf_item_base *xccdf_item_base_clone(const struct xccdf_item_base *old_
 struct xccdf_reference *xccdf_reference_clone(const struct xccdf_reference *old_reference)
 {
 	struct xccdf_reference *new_reference = oscap_calloc(1, sizeof(struct xccdf_reference));
-	new_reference->override = old_reference->override;
+	//new_reference->override = old_reference->override;
 	new_reference->href = oscap_strdup(old_reference->href);
-	new_reference->content = oscap_strdup(old_reference->content);
-	new_reference->lang = oscap_strdup(old_reference->lang);
+	new_reference->text = oscap_text_clone(old_reference->text);
+	//new_reference->content = oscap_strdup(old_reference->content);
+	//new_reference->lang = oscap_strdup(old_reference->lang);
 	return new_reference;
 }
 
@@ -308,6 +303,12 @@ void xccdf_item_print(struct xccdf_item *item, int depth)
 	}
 }
 
+void xccdf_texts_to_dom(struct oscap_text_iterator *texts, xmlNode *parent, const char *elname)
+{
+	OSCAP_FOR(oscap_text, text, texts)
+		oscap_text_to_dom(text, parent, elname);
+}
+
 xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent)
 {
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
@@ -335,21 +336,8 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 		xmlNewProp(item_node, BAD_CAST "abstract", BAD_CAST "True");
 
 	/* Handle generic item child nodes */
-        struct oscap_text_iterator *titles = xccdf_item_get_title(item);
-        while (oscap_text_iterator_has_more(titles)) {
-                struct oscap_text *title = oscap_text_iterator_next(titles);
-                xmlNode * child = xmlNewChild(item_node, ns_xccdf, BAD_CAST "title", BAD_CAST oscap_text_get_text(title));
-                if (oscap_text_get_lang(title) != NULL) xmlNewProp(child, BAD_CAST "xml:lang", BAD_CAST oscap_text_get_lang(title));
-        }
-        oscap_text_iterator_free(titles);
-
-        struct oscap_text_iterator *descriptions = xccdf_item_get_description(item);
-        while (oscap_text_iterator_has_more(descriptions)) {
-                struct oscap_text *description = oscap_text_iterator_next(descriptions);
-                xmlNode * child = xmlNewChild(item_node, ns_xccdf, BAD_CAST "description", BAD_CAST oscap_text_get_text(description));
-                if (oscap_text_get_lang(description) != NULL) xmlNewProp(child, BAD_CAST "xml:lang", BAD_CAST oscap_text_get_lang(description));
-        }
-        oscap_text_iterator_free(descriptions);
+	xccdf_texts_to_dom(xccdf_item_get_title(item), item_node, "title");
+	xccdf_texts_to_dom(xccdf_item_get_description(item), item_node, "description");
 
 	const char *version= xccdf_item_get_version(item);
 	if (version)
@@ -362,19 +350,14 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 	}
 	xccdf_status_iterator_free(statuses);
 
-        struct oscap_text_iterator *questions = xccdf_item_get_question(item);
-        while (oscap_text_iterator_has_more(questions)) {
-                struct oscap_text *question = oscap_text_iterator_next(questions);
-                xmlNode * child = xmlNewChild(item_node, ns_xccdf, BAD_CAST "question", BAD_CAST oscap_text_get_text(question));
-                if (oscap_text_get_lang(question) != NULL) xmlNewProp(child, BAD_CAST "xml:lang", BAD_CAST oscap_text_get_lang(question));
-        }
-        oscap_text_iterator_free(questions);
+	xccdf_texts_to_dom(xccdf_item_get_question(item), item_node, "question");
 
 	struct xccdf_reference_iterator *references = xccdf_item_get_references(item);
 	while (xccdf_reference_iterator_has_more(references)) {
 		struct xccdf_reference *ref = xccdf_reference_iterator_next(references);
 		xccdf_reference_to_dom(ref, doc, item_node);
 	}
+        xccdf_reference_iterator_free(references);
 
 	/* Handle type specific attributes and children */
 	switch (xccdf_item_get_type(item)) {
@@ -390,7 +373,8 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 			xccdf_profile_to_dom(XPROFILE(item), item_node, doc, parent);
 			break;
 		case XCCDF_RESULT:
-			xmlNodeSetName(item_node,BAD_CAST "Result");
+			xmlNodeSetName(item_node,BAD_CAST "TestResult");
+			if (parent) xccdf_result_to_dom(XRESULT(item), item_node, doc, parent);
 			break;
 		case XCCDF_GROUP:
 			xmlNodeSetName(item_node,BAD_CAST "Group");
@@ -415,14 +399,8 @@ xmlNode *xccdf_item_to_dom(struct xccdf_item *item, xmlDoc *doc, xmlNode *parent
 
 xmlNode *xccdf_reference_to_dom(struct xccdf_reference *ref, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
-	xmlNode *reference_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "reference", BAD_CAST xccdf_reference_get_content(ref));
-
-	const char *lang = xccdf_reference_get_lang(ref);
-	if (lang != NULL) xmlNewProp(reference_node, BAD_CAST "xml:lang", BAD_CAST lang);
-
-	const char *href = xccdf_reference_get_href(ref);
-	xmlNewProp(reference_node, BAD_CAST "href", BAD_CAST href);
+	xmlNode *reference_node = oscap_text_to_dom(xccdf_reference_get_text(ref), parent, "reference");
+	xmlNewProp(reference_node, BAD_CAST "href", BAD_CAST xccdf_reference_get_href(ref));
 
         /* TODO: Dublin Core Elements /XML spec p. 69/ */
 
@@ -431,18 +409,8 @@ xmlNode *xccdf_reference_to_dom(struct xccdf_reference *ref, xmlDoc *doc, xmlNod
 
 xmlNode *xccdf_profile_note_to_dom(struct xccdf_profile_note *note, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
-	xmlNode *note_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "profile-note", NULL);
-
-	// This is in the XCCDF Spec, but not implemented in OpenSCAP
-	//const char *lang = xccdf_profile_note_get_lang(note);
-	//xmlNewProp(note_node, BAD_CAST "xml:lang", BAD_CAST lang);
-
-	struct oscap_text *text = xccdf_profile_note_get_text(note);
-	xmlNewChild(note_node, ns_xccdf, BAD_CAST "sub", BAD_CAST oscap_text_get_text(text));
-
-	const char *reftag = xccdf_profile_note_get_reftag(note);
-	xmlNewChild(note_node, ns_xccdf, BAD_CAST "tag", BAD_CAST reftag);
+	xmlNode *note_node = oscap_text_to_dom(xccdf_profile_note_get_text(note), parent, "profile-note");
+	xmlNewProp(note_node, BAD_CAST "tag", BAD_CAST xccdf_profile_note_get_reftag(note));
 
 	return note_node;
 }
@@ -451,14 +419,16 @@ xmlNode *xccdf_status_to_dom(struct xccdf_status *status, xmlDoc *doc, xmlNode *
 {
 	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
 
-	xccdf_level_t level = xccdf_status_get_status(status);
-	xmlNode *status_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "status",
-					   BAD_CAST XCCDF_STATUS_MAP[level - 1].string);
+	xmlNode *status_node = NULL;
+	xccdf_status_type_t type = xccdf_status_get_status(status);
+	if (type != XCCDF_STATUS_NOT_SPECIFIED)
+		status_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "status",
+							BAD_CAST XCCDF_STATUS_MAP[type - 1].string);
 
 	time_t date_time = xccdf_status_get_date(status);
 	struct tm *date = localtime(&date_time);
 	char date_str[] = "YYYY-DD-MM";
-	snprintf(date_str, sizeof(date_str), "%d-%d-%d", date->tm_year + 1900, date->tm_mon + 1, date->tm_mday);
+	snprintf(date_str, sizeof(date_str), "%04d-%02d-%02d", date->tm_year + 1900, date->tm_mon + 1, date->tm_mday);
 	xmlNewProp(status_node, BAD_CAST "date", BAD_CAST date_str);
 
 	return status_node;
@@ -466,14 +436,7 @@ xmlNode *xccdf_status_to_dom(struct xccdf_status *status, xmlDoc *doc, xmlNode *
 
 xmlNode *xccdf_fixtext_to_dom(struct xccdf_fixtext *fixtext, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
-	xmlNode *fixtext_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "fixtext", NULL);
-
-	// This is in the XCCDF Spec, but not implemented in OpenSCAP
-	//const char *lang = xccdf_fixtext_get_lang(note);
-	//xmlNewProp(note_node, BAD_CAST "xml:lang", BAD_CAST lang);
-	//if (xccdf_fixtext_get_override(note))
-	//	xmlNewProp(fixtext_node, BAD_CAST "override", BAD_CAST "True");
+	xmlNode *fixtext_node = oscap_text_to_dom(xccdf_fixtext_get_text(fixtext), parent, "fixtext");
 	
 	if (xccdf_fixtext_get_reboot(fixtext))
 		xmlNewProp(fixtext_node, BAD_CAST "reboot", BAD_CAST "True");
@@ -493,17 +456,12 @@ xmlNode *xccdf_fixtext_to_dom(struct xccdf_fixtext *fixtext, xmlDoc *doc, xmlNod
         if (strategy != 0)
 	        xmlNewProp(fixtext_node, BAD_CAST "strategy", BAD_CAST XCCDF_STRATEGY_MAP[strategy].string);
 
-	const char *content = fixtext->content;
-	xmlNewChild(fixtext_node, ns_xccdf, BAD_CAST "sub", BAD_CAST content);
-
 	return fixtext_node;
 }
 
 xmlNode *xccdf_fix_to_dom(struct xccdf_fix *fix, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
-	const char *content = xccdf_fix_get_content(fix);
-	xmlNode *fix_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "fix", BAD_CAST content);
+	xmlNode *fix_node = oscap_xmlstr_to_dom(parent, "fix", xccdf_fix_get_content(fix));
 
 	const char *id = xccdf_fix_get_id(fix);
 	if (id != NULL) xmlNewProp(fix_node, BAD_CAST "id", BAD_CAST id);
@@ -515,9 +473,8 @@ xmlNode *xccdf_fix_to_dom(struct xccdf_fix *fix, xmlDoc *doc, xmlNode *parent)
 		xmlNewProp(fix_node, BAD_CAST "reboot", BAD_CAST "True");
 
 	xccdf_level_t complexity = xccdf_fix_get_complexity(fix);
-        if (complexity != 0) {
+        if (complexity != 0)
 	    xmlNewProp(fix_node, BAD_CAST "complexity", BAD_CAST XCCDF_LEVEL_MAP[complexity-1].string);
-        }
 
 	xccdf_level_t disruption = xccdf_fix_get_disruption(fix);
         if (disruption != 0)
@@ -621,8 +578,9 @@ xmlNode *xccdf_check_to_dom(struct xccdf_check *check, xmlDoc *doc, xmlNode *par
 }
 
 #define XCCDF_ITEM_PROCESS_FLAG(reader,flag,attr) \
-	if (xccdf_attribute_has((reader), (attr))) \
-		item->item.flags.flag = xccdf_attribute_get_bool((reader), (attr));
+	if (xccdf_attribute_has((reader), (attr))) { \
+		item->item.flags.flag = xccdf_attribute_get_bool((reader), (attr)); \
+		item->item.defined_flags.flag = true; }
 
 bool xccdf_item_process_attributes(struct xccdf_item *item, xmlTextReaderPtr reader)
 {
@@ -636,8 +594,10 @@ bool xccdf_item_process_attributes(struct xccdf_item *item, xmlTextReaderPtr rea
 	XCCDF_ITEM_PROCESS_FLAG(reader, abstract, XCCDFA_ABSTRACT);
 	XCCDF_ITEM_PROCESS_FLAG(reader, interactive, XCCDFA_INTERACTIVE);
 
-	if (xccdf_attribute_has(reader, XCCDFA_WEIGHT))
+	if (xccdf_attribute_has(reader, XCCDFA_WEIGHT)) {
 		item->item.weight = xccdf_attribute_get_float(reader, XCCDFA_WEIGHT);
+		item->item.defined_flags.weight = true;
+	}
 	if (xccdf_attribute_has(reader, XCCDFA_EXTENDS))
 		item->item.extends = xccdf_attribute_copy(reader, XCCDFA_EXTENDS);
 	item->item.cluster_id = xccdf_attribute_copy(reader, XCCDFA_CLUSTER_ID);
@@ -846,7 +806,7 @@ struct xccdf_model *xccdf_model_clone(const struct xccdf_model *old_model)
 
 	//params maps char * to char * so we will need to oscap_strdup the items.
 	new_model->params = oscap_htable_clone(old_model->params, (oscap_clone_func) oscap_strdup);
-	new_model->params = NULL;
+	//new_model->params = NULL;
 	return new_model;
 }
 
@@ -921,8 +881,8 @@ struct xccdf_warning *xccdf_warning_new(void)
 struct xccdf_warning *xccdf_warning_new_parse(xmlTextReaderPtr reader)
 {
     struct xccdf_warning *w = xccdf_warning_new();
-    w->text = oscap_text_new_parse(XCCDF_TEXT_HTMLSUB, reader);
     w->category = oscap_string_to_enum(XCCDF_WARNING_MAP, xccdf_attribute_get(reader, XCCDFA_CATEGORY));
+    w->text = oscap_text_new_parse(XCCDF_TEXT_HTMLSUB, reader);
     return w;
 }
 
@@ -946,10 +906,11 @@ struct xccdf_reference *xccdf_reference_new(void)
 struct xccdf_reference *xccdf_reference_new_parse(xmlTextReaderPtr reader)
 {
     struct xccdf_reference *ref = xccdf_reference_new();
-    if (xccdf_attribute_has(reader, XCCDFA_OVERRIDE))
-            ref->override = oscap_string_to_enum(OSCAP_BOOL_MAP, xccdf_attribute_get(reader, XCCDFA_OVERRIDE));
+    //if (xccdf_attribute_has(reader, XCCDFA_OVERRIDE))
+    //        ref->override = oscap_string_to_enum(OSCAP_BOOL_MAP, xccdf_attribute_get(reader, XCCDFA_OVERRIDE));
     ref->href = xccdf_attribute_copy(reader, XCCDFA_HREF);
-    ref->content = oscap_element_string_copy(reader);
+	ref->text = oscap_text_new_parse(OSCAP_TEXT_TRAITS_HTML, reader);
+    //ref->content = oscap_element_string_copy(reader);
     // TODO Dublin Core
     return ref;
 }
@@ -957,29 +918,30 @@ struct xccdf_reference *xccdf_reference_new_parse(xmlTextReaderPtr reader)
 void xccdf_reference_free(struct xccdf_reference *ref)
 {
     if (ref != NULL) {
-        oscap_free(ref->lang);
+        //oscap_free(ref->lang);
         oscap_free(ref->href);
-        oscap_free(ref->content);
+		oscap_text_free(ref->text);
+        //oscap_free(ref->content);
         oscap_free(ref);
     }
 }
 
 //clones the specific types of items
-struct xccdf_profile_item * xccdf_profile_item_clone(const struct xccdf_profile_item * item)
+void xccdf_profile_item_clone(struct xccdf_profile_item *clone, const struct xccdf_profile_item * item)
 {
-	struct xccdf_profile_item * clone = oscap_calloc(1, sizeof(struct xccdf_profile_item));
 	clone->note_tag = oscap_strdup(item->note_tag);
 	clone->selects = 	oscap_list_clone(item->selects, (oscap_clone_func) xccdf_select_clone);
 	clone->setvalues = oscap_list_clone(item->setvalues, (oscap_clone_func) xccdf_setvalue_clone);
 	clone->refine_values = oscap_list_clone(item->refine_values, (oscap_clone_func) xccdf_refine_value_clone);
 	clone->refine_rules = oscap_list_clone(item->refine_rules, (oscap_clone_func) xccdf_refine_rule_clone);
-	return clone;
 }
 
-struct xccdf_benchmark_item * xccdf_benchmark_item_clone(const struct xccdf_benchmark_item * item, struct xccdf_item *parent)
+struct xccdf_benchmark_item * xccdf_benchmark_item_clone(struct xccdf_benchmark_item *clone, const struct xccdf_benchmark * bench)
 {
-	struct xccdf_benchmark_item * clone = oscap_calloc(1, sizeof(struct xccdf_benchmark_item));
-	clone->dict = oscap_htable_clone(item->dict, (oscap_clone_func) xccdf_item_clone);	
+    struct xccdf_benchmark_item *item = &XITEM(bench)->sub.benchmark;
+    struct xccdf_item *parent = XITEM(bench);
+
+	clone->dict = oscap_htable_new();
 	clone->notices = oscap_list_clone(item->notices, (oscap_clone_func) xccdf_notice_clone);
 	clone->plain_texts = oscap_list_clone(item->plain_texts, (oscap_clone_func) xccdf_plain_text_clone);
 	
@@ -1001,78 +963,61 @@ struct xccdf_benchmark_item * xccdf_benchmark_item_clone(const struct xccdf_benc
 	return clone;
 }
 
-struct xccdf_rule_item * xccdf_rule_item_clone(const struct xccdf_rule_item * item)
+void xccdf_rule_item_clone(struct xccdf_rule_item *clone, const struct xccdf_rule_item * item)
 {
-	struct xccdf_rule_item * clone = oscap_calloc(1, sizeof(struct xccdf_rule_item));
 	clone->role = item->role;
 	clone->severity = item->severity;
 	clone->idents = oscap_list_clone(item->idents, (oscap_clone_func) xccdf_ident_clone );
 	clone->checks = oscap_list_clone(item->checks, (oscap_clone_func) xccdf_check_clone );
-	//requires is an oscap_list of char * so we should be able to use oscap_strdup for cloning
-	clone->requires = oscap_list_clone(item->requires, (oscap_clone_func) oscap_strdup);
-	//conflicts is also an oscap_list of char *
+	//requires is an oscap_list of oscap_lists of char *
+	clone->requires = oscap_list_clone(item->requires, (oscap_clone_func) oscap_stringlist_clone);
+	//conflicts is an oscap_list of char *
 	clone->conflicts = oscap_list_clone(item->conflicts, (oscap_clone_func) oscap_strdup);
 	clone->profile_notes = oscap_list_clone(item->profile_notes, (oscap_clone_func) xccdf_profile_note_clone);
 	clone->fixes = oscap_list_clone(item->fixes, (oscap_clone_func) xccdf_fix_clone);
 	clone->fixtexts = oscap_list_clone(item->fixtexts, (oscap_clone_func) xccdf_fixtext_clone);
-	return clone;
 }
 
-struct xccdf_group_item * xccdf_group_item_clone(const struct xccdf_group_item * item, struct xccdf_item *parent)
+void xccdf_group_item_clone(struct xccdf_item *parent, const struct xccdf_group_item * item)
 {
-	struct xccdf_group_item * clone = oscap_calloc(1, sizeof(struct xccdf_group_item));;
-	//requires is an oscap_list of char * so we should be able to use oscap_strdup for cloning
-	clone->requires = oscap_list_clone(item->requires, (oscap_clone_func) oscap_strdup);
-	//conflicts is also an oscap_list of char *
+    struct xccdf_group_item *clone = &parent->sub.group;
+	//requires is an oscap_list of oscap_lists of char *
+	clone->requires = oscap_list_clone(item->requires, (oscap_clone_func) oscap_stringlist_clone);
+	//conflicts is an oscap_list of char *
 	clone->conflicts = oscap_list_clone(item->conflicts, (oscap_clone_func) oscap_strdup);
 	clone->values = oscap_list_clone(item->values, (oscap_clone_func) xccdf_value_clone);
 	xccdf_reparent_list(clone->values, parent);
 	clone->content = oscap_list_clone(item->content, (oscap_clone_func) xccdf_item_clone);
 	xccdf_reparent_list(clone->content, parent);
-	return clone;
 }
 
-union xccdf_value_unit xccdf_value_unit_clone_str(const union xccdf_value_unit unit)
+union xccdf_value_unit *xccdf_value_unit_clone_str(const union xccdf_value_unit *unit)
 {
-	union xccdf_value_unit val;
-	val.s = oscap_strdup(unit.s);
+	union xccdf_value_unit *val = oscap_calloc(1, sizeof(union xccdf_value_unit));
+	val->s = oscap_strdup(unit->s);
 	return val;
 }
 
-union xccdf_value_unit xccdf_value_unit_clone_numeric(const union xccdf_value_unit unit)
+union xccdf_value_unit *xccdf_value_unit_clone_numeric(const union xccdf_value_unit *unit)
 {
-	union xccdf_value_unit val;
-	val.n = unit.n;
+	union xccdf_value_unit *val = oscap_calloc(1, sizeof(union xccdf_value_unit));
+	val->n = unit->n;
 	return val;
 }
 
-union xccdf_value_unit xccdf_value_unit_clone_bool(const union xccdf_value_unit unit)
+union xccdf_value_unit *xccdf_value_unit_clone_bool(const union xccdf_value_unit *unit)
 {
-	union xccdf_value_unit val;
-	val.b = unit.b;
+	union xccdf_value_unit *val = oscap_calloc(1, sizeof(union xccdf_value_unit));
+	val->b = unit->b;
 	return val;
 }
 
-struct xccdf_value_val * xccdf_value_val_clone_str(const struct xccdf_value_val * val)
+struct xccdf_value_instance * xccdf_value_instance_clone(const struct xccdf_value_instance * val)
 {
-	return xccdf_value_val_clone(val, XCCDF_TYPE_STRING);
-}
-
-struct xccdf_value_val * xccdf_value_val_clone_numeric(const struct xccdf_value_val * val)
-{
-	return xccdf_value_val_clone(val, XCCDF_TYPE_NUMBER);
-}
-
-struct xccdf_value_val * xccdf_value_val_clone_bool(const struct xccdf_value_val * val)
-{
-	return xccdf_value_val_clone(val, XCCDF_TYPE_BOOLEAN);
-}
-
-struct xccdf_value_val * xccdf_value_val_clone(const struct xccdf_value_val * val, xccdf_value_type_t type)
-{
-	struct xccdf_value_val * clone = oscap_calloc(1, sizeof(struct xccdf_value_val));
+	struct xccdf_value_instance * clone = oscap_calloc(1, sizeof(struct xccdf_value_instance));
+    clone->type = val->type;
 	
-	switch (type) {
+	switch (val->type) {
 	case XCCDF_TYPE_STRING:
 		clone->value.s = oscap_strdup(val->value.s);
 		clone->defval.s = oscap_strdup(val->defval.s);
@@ -1095,37 +1040,23 @@ struct xccdf_value_val * xccdf_value_val_clone(const struct xccdf_value_val * va
 		assert(false);
 	}
 	
-	clone->must_match = val->must_match;
+	clone->flags = val->flags;
+	xccdf_value_instance_set_selector(clone, xccdf_value_instance_get_selector(val));
 	return clone;
 }
 
-struct xccdf_value_item * xccdf_value_item_clone(const struct xccdf_value_item * item)
+void xccdf_value_item_clone(struct xccdf_value_item *clone, const struct xccdf_value_item * item)
 {
-	struct xccdf_value_item * clone = oscap_calloc(1, sizeof(struct xccdf_value_item));
+	//struct xccdf_value_item * clone = oscap_calloc(1, sizeof(struct xccdf_value_item));
+
 	//these three are values which can be directly assigned.
 	clone->type = item->type;
 	clone->interface_hint = item->interface_hint;
 	clone->oper = item->oper;
 	
 	//the rest need deep copy
-	clone->selector = oscap_strdup(item->selector);
-	clone->value = xccdf_value_val_clone(item->value, item->type);
-	//this assumes the values list is a list of xccdf_value_val objects of each type and that the
-	//lists do not have mixed types.
-	switch(clone->type)
-	{
-		case XCCDF_TYPE_STRING:
-			clone->values = oscap_htable_clone(item->values, (oscap_clone_func) xccdf_value_val_clone_str);
-			break;
-		case XCCDF_TYPE_NUMBER:
-			clone->values = oscap_htable_clone(item->values, (oscap_clone_func) xccdf_value_val_clone_numeric);
-			break;
-		case XCCDF_TYPE_BOOLEAN:
-			clone->values = oscap_htable_clone(item->values, (oscap_clone_func) xccdf_value_val_clone_bool);
-			break;
-	}
+	clone->instances = oscap_list_clone(item->instances, (oscap_clone_func) xccdf_value_instance_clone);
 	clone->sources = oscap_list_clone(item->sources, (oscap_clone_func) oscap_strdup);
-	return clone;
 }
 
 struct xccdf_identity * xccdf_identity_clone(const struct xccdf_identity * identity)
@@ -1170,7 +1101,7 @@ struct xccdf_instance * xccdf_instance_clone(const struct xccdf_instance * insta
     struct xccdf_instance * clone = oscap_calloc(1, sizeof(struct xccdf_instance));
     clone->context = oscap_strdup(instance->context);
     clone->parent_context = oscap_strdup(instance->parent_context);
-    clone->context = oscap_strdup(instance->content);
+    clone->content = oscap_strdup(instance->content);
     return clone;
 }
 
@@ -1202,9 +1133,8 @@ struct xccdf_score * xccdf_score_clone(const struct xccdf_score * score)
 	return clone;
 }
 
-struct xccdf_result_item * xccdf_result_item_clone(const struct xccdf_result_item * item)
+void xccdf_result_item_clone(struct xccdf_result_item *clone, const struct xccdf_result_item * item)
 {
-	struct xccdf_result_item * clone = oscap_calloc(1, sizeof(struct xccdf_result_item));
 	clone->start_time = item->start_time;
 	clone->end_time = item->end_time;
 	clone->test_system = oscap_strdup(item->test_system);
@@ -1216,12 +1146,10 @@ struct xccdf_result_item * xccdf_result_item_clone(const struct xccdf_result_ite
 	clone->organizations = oscap_list_clone(item->organizations, (oscap_clone_func) oscap_strdup);
 	clone->remarks = oscap_list_clone(item->remarks, (oscap_clone_func) oscap_text_clone);
 	clone->target_addresses = oscap_list_clone(item->target_addresses, (oscap_clone_func) oscap_strdup);
-	clone->target_facts = oscap_list_clone(item->identities, (oscap_clone_func) xccdf_target_fact_clone);
-	clone->setvalues = oscap_list_clone(item->identities, (oscap_clone_func) xccdf_setvalue_clone);
-	clone->rule_results = oscap_list_clone(item->identities, (oscap_clone_func) xccdf_rule_result_clone);
-	clone->scores = oscap_list_clone(item->identities, (oscap_clone_func) xccdf_score_clone);
-	
-	return clone;
+	clone->target_facts = oscap_list_clone(item->target_facts, (oscap_clone_func) xccdf_target_fact_clone);
+	clone->setvalues = oscap_list_clone(item->setvalues, (oscap_clone_func) xccdf_setvalue_clone);
+	clone->rule_results = oscap_list_clone(item->rule_results, (oscap_clone_func) xccdf_rule_result_clone);
+	clone->scores = oscap_list_clone(item->scores, (oscap_clone_func) xccdf_score_clone);
 }
 
 void xccdf_reparent_list(struct oscap_list * item_list, struct xccdf_item * parent)
@@ -1232,6 +1160,7 @@ void xccdf_reparent_list(struct oscap_list * item_list, struct xccdf_item * pare
 		struct xccdf_item * item = oscap_iterator_next(it);
 		xccdf_reparent_item(item, parent);
 	}
+    oscap_iterator_free(it);
 }
 
 void xccdf_reparent_item(struct xccdf_item * item, struct xccdf_item * parent)
@@ -1239,13 +1168,12 @@ void xccdf_reparent_item(struct xccdf_item * item, struct xccdf_item * parent)
 	if(item != NULL)
 	{
 		item->item.parent = parent;
+		xccdf_benchmark_register_item(xccdf_item_get_benchmark(item), item);
 	}
 }
 
-OSCAP_ACCESSOR_STRING(xccdf_reference, lang)
 OSCAP_ACCESSOR_STRING(xccdf_reference, href)
-OSCAP_ACCESSOR_STRING(xccdf_reference, content)
-OSCAP_ACCESSOR_SIMPLE(bool,        xccdf_reference, override)
+OSCAP_ACCESSOR_TEXT(xccdf_reference, text)
 
 const struct oscap_text_traits XCCDF_TEXT_PLAIN    = { .can_override = true };
 const struct oscap_text_traits XCCDF_TEXT_HTML     = { .html = true, .can_override = true };
