@@ -33,13 +33,26 @@
 OSCAP_HIDDEN_START;
 
 struct xccdf_flags {
-	unsigned selected:1;
-	unsigned hidden:1;
-	unsigned resolved:1;
-	unsigned abstract:1;
-	unsigned prohibit_changes:1;
-	unsigned interactive:1;
-	unsigned multiple:1;
+	bool selected:1;
+	bool hidden:1;
+	bool resolved:1;
+	bool abstract:1;
+	bool prohibit_changes:1;
+	bool interactive:1;
+	bool multiple:1;
+};
+
+struct xccdf_defflags {
+	bool selected:1;
+	bool hidden:1;
+	bool resolved:1;
+	bool abstract:1;
+	bool prohibit_changes:1;
+	bool interactive:1;
+	bool multiple:1;
+	bool weight:1;
+	bool role:1;
+	bool severity:1;
 };
 
 struct xccdf_item;
@@ -54,7 +67,7 @@ struct xccdf_item_base {
 	struct oscap_list *description;
 	struct oscap_list *question;
 	struct oscap_list *rationale;
-        struct oscap_list *warnings;
+	struct oscap_list *warnings;
 
 	char *version;
 	char *version_update;
@@ -66,13 +79,13 @@ struct xccdf_item_base {
 	struct oscap_list *references;
 	struct oscap_list *platforms;
 	struct xccdf_flags flags;
+	struct xccdf_defflags defined_flags;
 };
 
 struct xccdf_rule_item {
 	char *impact_metric;
 	xccdf_role_t role;
 	xccdf_level_t severity;
-	struct xccdf_check *check;
 
 	struct oscap_list *requires;
 	struct oscap_list *conflicts;
@@ -99,11 +112,12 @@ union xccdf_value_unit {
 };
 
 /* This structure is used for multiple-count attributes for Item (0-n) */
-struct xccdf_value_val { 
+struct xccdf_value_instance {
+	char *selector;
+	xccdf_value_type_t type;                    /* Value::type */
 	union xccdf_value_unit value;               /* Value::value   */
 	union xccdf_value_unit defval;              /* Value::default */
 	struct oscap_list *choices;                 /* Value::choices */
-	bool must_match;                            /* */
 	union {
 		struct {
 			xccdf_numeric lower_bound;
@@ -113,17 +127,20 @@ struct xccdf_value_val {
 			char *match;
 		} s;
 	} limits;
+	struct {
+		bool value_given : 1;
+		bool defval_given : 1;
+		bool must_match_given : 1;
+		bool must_match : 1;
+	} flags;
 };
 
 struct xccdf_value_item {
 	xccdf_value_type_t type;                    /* Value::type */
 	xccdf_interface_hint_t interface_hint;      /* Value::interfaceHint */
 	xccdf_operator_t oper;                      /* Value::operator */
-	char *selector;
-
-	struct xccdf_value_val *value;
-	struct oscap_htable *values; // TODO: to list
 
+	struct oscap_list *instances;
 	struct oscap_list *sources;                 /* Value::source */
 };
 
@@ -266,8 +283,8 @@ struct xccdf_check_export {
 };
 
 struct xccdf_profile_note {
-	char *reftag;
 	struct oscap_text *text;
+	char *reftag;
 };
 
 struct xccdf_fix {
@@ -282,19 +299,17 @@ struct xccdf_fix {
 };
 
 struct xccdf_fixtext {
+	struct oscap_text *text;
 	bool reboot;
 	xccdf_strategy_t strategy;
 	xccdf_level_t disruption;
 	xccdf_level_t complexity;
 	char *fixref;
-	char *content;
 };
 
 struct xccdf_reference {
-    bool override;
+	struct oscap_text *text;
     char *href;
-    char *content;
-    char *lang;
 };
 
 struct xccdf_rule_result {
@@ -437,27 +452,25 @@ struct xccdf_reference *xccdf_reference_new_parse(xmlTextReaderPtr reader);
 
 //private methods for cloning items
 //Will clone the item member of a xccdf_item object
-struct xccdf_item_base * xccdf_item_base_clone(const struct xccdf_item_base * item);
+void xccdf_item_base_clone(struct xccdf_item_base *new_base, const struct xccdf_item_base *old_base);
 
 //clones the specific types of items
-struct xccdf_profile_item * xccdf_profile_item_clone(const struct xccdf_profile_item * item);
-struct xccdf_benchmark_item * xccdf_benchmark_item_clone(const struct xccdf_benchmark_item * item, struct xccdf_item * parent);
-struct xccdf_rule_item * xccdf_rule_item_clone(const struct xccdf_rule_item * item);
-struct xccdf_group_item * xccdf_group_item_clone(const struct xccdf_group_item * item, struct xccdf_item * parent);
-union xccdf_value_unit xccdf_value_unit_clone_str(const union xccdf_value_unit unit);
-union xccdf_value_unit xccdf_value_unit_clone_numeric(const union xccdf_value_unit unit);
-union xccdf_value_unit xccdf_value_unit_clone_bool(const union xccdf_value_unit unit);
-struct xccdf_value_val * xccdf_value_val_clone_str(const struct xccdf_value_val * val);
-struct xccdf_value_val * xccdf_value_val_clone_numeric(const struct xccdf_value_val * val);
-struct xccdf_value_val * xccdf_value_val_clone_bool(const struct xccdf_value_val * val);
-struct xccdf_value_val * xccdf_value_val_clone(const struct xccdf_value_val * val, xccdf_value_type_t type);
-struct xccdf_value_item * xccdf_value_item_clone(const struct xccdf_value_item * item);
-struct xccdf_result_item * xccdf_result_item_clone(const struct xccdf_result_item * item);
-struct xccdf_ident * xccdf_ident_clone(const struct xccdf_ident * ident);
+void xccdf_profile_item_clone(struct xccdf_profile_item *clone, const struct xccdf_profile_item * item);
+struct xccdf_benchmark_item * xccdf_benchmark_item_clone(struct xccdf_benchmark_item *clone, const struct xccdf_benchmark * bench);
+void xccdf_rule_item_clone(struct xccdf_rule_item *clone, const struct xccdf_rule_item * item);
+void xccdf_group_item_clone(struct xccdf_item *parent, const struct xccdf_group_item * item);
+void xccdf_value_item_clone(struct xccdf_value_item *clone, const struct xccdf_value_item * item);
+union xccdf_value_unit *xccdf_value_unit_clone_str(const union xccdf_value_unit *unit);
+union xccdf_value_unit *xccdf_value_unit_clone_numeric(const union xccdf_value_unit *unit);
+union xccdf_value_unit *xccdf_value_unit_clone_bool(const union xccdf_value_unit *unit);
+struct xccdf_value_instance * xccdf_value_instance_clone(const struct xccdf_value_instance * val);
+void xccdf_result_item_clone(struct xccdf_result_item *clone, const struct xccdf_result_item * item);
 struct xccdf_profile_note * xccdf_profile_note_clone(const struct xccdf_profile_note * note);
 void xccdf_reparent_list(struct oscap_list * item_list, struct xccdf_item * parent);
 void xccdf_reparent_item(struct xccdf_item * item, struct xccdf_item * parent);
 
+void xccdf_texts_to_dom(struct oscap_text_iterator *texts, xmlNode *parent, const char *elname);
+
 #include "unused.h"
 
 OSCAP_HIDDEN_END;
