@@ -191,9 +191,9 @@ struct xccdf_profile *xccdf_profile_clone (const struct xccdf_profile *old_profi
 {
 	struct xccdf_item *new_profile = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_profile_item));
 	struct xccdf_item *old = XITEM(old_profile);
-	new_profile->item = *(xccdf_item_base_clone(&(old->item)));
+    xccdf_item_base_clone(&new_profile->item, &(old->item));
 	new_profile->type = old->type;
-	new_profile->sub.profile = *(xccdf_profile_item_clone(&(old->sub.profile)));
+    xccdf_profile_item_clone(&new_profile->sub.profile, &old->sub.profile);
 	return XPROFILE(new_profile);
 }
 
@@ -320,12 +320,7 @@ void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node,
 		else
 			xmlNewProp(select_node, BAD_CAST "selected", BAD_CAST "0");
 
-		struct oscap_text_iterator *remarks = xccdf_select_get_remarks(sel);
-		while (oscap_text_iterator_has_more(remarks)) {
-			struct oscap_text *remark = oscap_text_iterator_next(remarks);
-			xccdf_remark_to_dom(remark, doc, select_node);
-		}
-		oscap_text_iterator_free(remarks);
+		xccdf_texts_to_dom(xccdf_select_get_remarks(sel), select_node, "remark");
 	}
 	xccdf_select_iterator_free(selects);
 
@@ -346,18 +341,14 @@ void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node,
 			xmlNewProp(refval_node, BAD_CAST "idref", BAD_CAST idref);
 
 		xccdf_operator_t operator = xccdf_refine_value_get_oper(refine_value);
-		xmlNewProp(refval_node, BAD_CAST "operator", BAD_CAST XCCDF_OPERATOR_MAP[operator - 1].string);
+		if (operator != 0)
+			xmlNewProp(refval_node, BAD_CAST "operator", BAD_CAST XCCDF_OPERATOR_MAP[operator - 1].string);
 
 		const char *selector = xccdf_refine_value_get_selector(refine_value);
 		if (selector)
 			xmlNewProp(refval_node, BAD_CAST "selector", BAD_CAST selector);
 
-		struct oscap_text_iterator *remarks = xccdf_refine_value_get_remarks(refine_value);
-		while (oscap_text_iterator_has_more(remarks)) {
-			struct oscap_text *remark = oscap_text_iterator_next(remarks);
-			xccdf_remark_to_dom(remark, doc, refval_node);
-		}
-		oscap_text_iterator_free(remarks);
+		xccdf_texts_to_dom(xccdf_refine_value_get_remarks(refine_value), refval_node, "remark");
 	}
 	xccdf_refine_value_iterator_free(refine_values);
 
@@ -371,26 +362,23 @@ void xccdf_profile_to_dom(struct xccdf_profile *profile, xmlNode *profile_node,
 			xmlNewProp(refrule_node, BAD_CAST "idref", BAD_CAST idref);
 
 		xccdf_role_t role = xccdf_refine_rule_get_role(refine_rule);
-		xmlNewProp(refrule_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
+		if (role != 0)
+			xmlNewProp(refrule_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
 
 		const char *selector = xccdf_refine_rule_get_selector(refine_rule);
 		if (selector)
 			xmlNewProp(refrule_node, BAD_CAST "selector", BAD_CAST selector);
 
 		xccdf_level_t severity = xccdf_refine_rule_get_severity(refine_rule);
-		xmlNewProp(refrule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity].string);
+		if (severity != 0)
+			xmlNewProp(refrule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity].string);
 
 		float weight = xccdf_refine_rule_get_weight(refine_rule);
 		char weight_str[10];
 		sprintf(weight_str, "%f", weight);
 		xmlNewProp(refrule_node, BAD_CAST "weight", BAD_CAST weight_str);
 		
-		struct oscap_text_iterator *remarks = xccdf_refine_rule_get_remarks(refine_rule);
-		while (oscap_text_iterator_has_more(remarks)) {
-			struct oscap_text *remark = oscap_text_iterator_next(remarks);
-			xccdf_remark_to_dom(remark, doc, refrule_node);
-		}
-		oscap_text_iterator_free(remarks);
+		xccdf_texts_to_dom(xccdf_refine_rule_get_remarks(refine_rule), refrule_node, "remark");
 	}
 	xccdf_refine_rule_iterator_free(refine_rules);
 }
