@@ -24,6 +24,7 @@
 #include "item.h"
 #include "helpers.h"
 #include "xccdf_impl.h"
+#include "common/_error.h"
 #include <math.h>
 #include <text.h>
 
@@ -57,9 +58,9 @@ struct xccdf_result * xccdf_result_clone(const struct xccdf_result * result)
 {
 	struct xccdf_item *new_result = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_result_item));
 	struct xccdf_item *old = XITEM(result);
-	new_result->item = *(xccdf_item_base_clone(&(old->item)));
+    xccdf_item_base_clone(&new_result->item, &(old->item));
 	new_result->type = old->type;
-	new_result->sub.result = *(xccdf_result_item_clone(&(old->sub.result)));
+	xccdf_result_item_clone(&new_result->sub.result, &old->sub.result);
 	return XRESULT(new_result);
 }
 
@@ -83,6 +84,26 @@ static inline void xccdf_result_free_impl(struct xccdf_item *result)
 		xccdf_item_release(result);
 	}
 }
+
+time_t xccdf_result_get_start_time(const struct xccdf_result* result)
+{
+	return ((struct xccdf_item *)result)->sub.result.start_time;
+}
+
+time_t xccdf_result_get_end_time(const struct xccdf_result* result)
+{
+	return ((struct xccdf_item *)result)->sub.result.end_time;
+}
+
+_Bool xccdf_result_set_start_time(struct xccdf_result* result, time_t start)
+{
+	return (((struct xccdf_item *)result)->sub.result.start_time = start);
+}
+
+_Bool xccdf_result_set_end_time(struct xccdf_result* result, time_t end)
+{
+	return (((struct xccdf_item *)result)->sub.result.end_time = end);
+}
 XCCDF_FREE_GEN(result)
 
 XCCDF_ACCESSOR_STRING(result, test_system)
@@ -134,7 +155,7 @@ OSCAP_ACCESSOR_STRING(xccdf_rule_result, version)
 OSCAP_ACCESSOR_STRING(xccdf_rule_result, idref)
 OSCAP_IGETINS(xccdf_ident, xccdf_rule_result, idents, ident)
 OSCAP_IGETINS(xccdf_fix, xccdf_rule_result, fixes, fix)
-OSCAP_IGETTER(xccdf_check, xccdf_rule_result, checks)
+OSCAP_IGETINS(xccdf_check, xccdf_rule_result, checks, check)
 OSCAP_IGETINS_GEN(xccdf_override, xccdf_rule_result, overrides, override)
 OSCAP_IGETINS_GEN(xccdf_message, xccdf_rule_result, messages, message)
 OSCAP_IGETINS_GEN(xccdf_instance, xccdf_rule_result, instances, instance)
@@ -324,7 +345,7 @@ struct xccdf_result *xccdf_result_new_parse(xmlTextReaderPtr reader)
 	if (!xccdf_item_process_attributes(res, reader)) goto fail;
 
 	if (xccdf_attribute_has(reader, XCCDFA_END_TIME))
-		res->sub.result.start_time = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_END_TIME));
+		res->sub.result.end_time = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_END_TIME));
 	else goto fail;
 	res->sub.result.start_time = oscap_get_datetime(xccdf_attribute_get(reader, XCCDFA_START_TIME));
 	res->item.version = xccdf_attribute_copy(reader, XCCDFA_VERSION);
@@ -382,20 +403,63 @@ fail:
 	return NULL;
 }
 
+int xccdf_result_export(struct xccdf_result *result, const char *file)
+{
+	__attribute__nonnull__(file);
+
+	int retcode = 0;
+
+	LIBXML_TEST_VERSION;
+
+	xmlDocPtr doc = xmlNewDoc(BAD_CAST "1.0");
+	if (doc == NULL) {
+		oscap_setxmlerr(xmlGetLastError());
+		return -1;
+	}
+
+	xccdf_result_to_dom(result, NULL, doc, NULL);
+
+	retcode = xmlSaveFormatFileEnc(file, doc, "UTF-8", 1);
+	if (retcode < 1)
+		oscap_setxmlerr(xmlGetLastError());
+
+	xmlFreeDoc(doc);
+
+	return retcode;
+}
+
 void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlDoc *doc, xmlNode *parent)
 {
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+        xmlNs *ns_xccdf = NULL;
+	if (parent) {
+	        ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
+	} else {
+		if (!result_node) result_node = xccdf_item_to_dom(XITEM(result), doc, NULL);
+                ns_xccdf = xmlNewNs(result_node, XCCDF_BASE_NAMESPACE, NULL);
+		xmlDocSetRootElement(doc, result_node);
+	}
 
 	/* Handle attributes */
-	/* start-time and end-time do not appear to be implemented in OpenSCAP */
+	time_t start = xccdf_result_get_start_time(result);
+	if (start) {
+		struct tm *lt = localtime(&start);
+		char timestamp[] = "yyyy-mm-ddThh:mm:ss";
+		snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
+			 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
+		xmlNewProp(result_node, BAD_CAST "start-time", BAD_CAST timestamp);
+	}
+
+	time_t end = xccdf_result_get_end_time(result);
+	if (end) {
+		struct tm *lt = localtime(&end);
+		char timestamp[] = "yyyy-mm-ddThh:mm:ss";
+		snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
+			 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
+		xmlNewProp(result_node, BAD_CAST "end-time", BAD_CAST timestamp);
+	}
 
 	/* Handle children */
-	struct oscap_text_iterator *remarks = xccdf_result_get_remarks(result);
-	while (oscap_text_iterator_has_more(remarks)) {
-		struct oscap_text *remark = oscap_text_iterator_next(remarks);
-		xccdf_remark_to_dom(remark, doc, result_node);
-	}
-	oscap_text_iterator_free(remarks);
+	xccdf_texts_to_dom(xccdf_result_get_remarks(result), result_node, "remark");
 
 	struct oscap_string_iterator *orgs = xccdf_result_get_organizations(result);
 	while (oscap_string_iterator_has_more(orgs)) {
@@ -449,19 +513,21 @@ void xccdf_result_to_dom(struct xccdf_result *result, xmlNode *result_node, xmlD
 	oscap_string_iterator_free(target_addresses);
 
 	struct xccdf_target_fact_iterator *target_facts = xccdf_result_get_target_facts(result);
-	while (xccdf_target_fact_iterator_has_more(target_facts)) {
-		struct xccdf_target_fact *target_fact = xccdf_target_fact_iterator_next(target_facts);
-		xmlNode *target_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "target_fact", BAD_CAST target_fact->value);
-		xmlNode *fact_node = xmlNewChild(target_node, ns_xccdf, BAD_CAST "fact", NULL);
+        if (xccdf_target_fact_iterator_has_more(target_facts)) {
+                xmlNode *target_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "target_facts", NULL);
+                while (xccdf_target_fact_iterator_has_more(target_facts)) {
+                        struct xccdf_target_fact *target_fact = xccdf_target_fact_iterator_next(target_facts);
+                        xmlNode *fact_node = xmlNewChild(target_node, ns_xccdf, BAD_CAST "fact", BAD_CAST target_fact->value);
 
-		const char *name = xccdf_target_fact_get_name(target_fact);
-		if (name)
-			xmlNewProp(fact_node, BAD_CAST "name", BAD_CAST name);
+                        const char *name = xccdf_target_fact_get_name(target_fact);
+                        if (name)
+                                xmlNewProp(fact_node, BAD_CAST "name", BAD_CAST name);
 
-		xccdf_value_type_t value = xccdf_target_fact_get_type(target_fact);
-		xmlNewProp(fact_node, BAD_CAST "type", BAD_CAST XCCDF_FACT_TYPE_MAP[value - 1].string);
-	}
-	xccdf_target_fact_iterator_free(target_facts);
+                        xccdf_value_type_t value = xccdf_target_fact_get_type(target_fact);
+                        xmlNewProp(fact_node, BAD_CAST "type", BAD_CAST XCCDF_FACT_TYPE_MAP[value - 1].string);
+                }
+                xccdf_target_fact_iterator_free(target_facts);
+        }
 
 	struct xccdf_rule_result_iterator *rule_results = xccdf_result_get_rule_results(result);
 	while (xccdf_rule_result_iterator_has_more(rule_results)) {
@@ -583,17 +649,21 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 		xmlNewProp(result_node, BAD_CAST "idref", BAD_CAST idref);
 
 	xccdf_role_t role = xccdf_rule_result_get_role(result);
-	xmlNewProp(result_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
+	if (role != 0) 
+            xmlNewProp(result_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
 
 	time_t date = xccdf_rule_result_get_time(result);
-	struct tm *lt = localtime(&date);
-	char timestamp[] = "yyyy-mm-ddThh:mm:ss";
-	snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
-		 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
-	xmlNewProp(result_node, BAD_CAST "date", BAD_CAST timestamp);
+	if (date) {
+		struct tm *lt = localtime(&date);
+		char timestamp[] = "yyyy-mm-ddThh:mm:ss";
+		snprintf(timestamp, sizeof(timestamp), "%4d-%02d-%02dT%02d:%02d:%02d",
+			 1900 + lt->tm_year, 1 + lt->tm_mon, lt->tm_mday, lt->tm_hour, lt->tm_min, lt->tm_sec);
+		xmlNewProp(result_node, BAD_CAST "time", BAD_CAST timestamp);
+	}
 
 	xccdf_level_t severity = xccdf_rule_result_get_severity(result);
-	xmlNewProp(result_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
+	if (severity != 0)
+            xmlNewProp(result_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
 
 	const char *version = xccdf_rule_result_get_version(result);
 	if (version)
@@ -606,7 +676,8 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 
 	/* Handle children */
 	xccdf_test_result_type_t test_res = xccdf_rule_result_get_result(result);
-	xmlNewChild(result_node, ns_xccdf, BAD_CAST "result", BAD_CAST XCCDF_RESULT_MAP[test_res - 1].string);
+	if (test_res != 0)
+            xmlNewChild(result_node, ns_xccdf, BAD_CAST "result", BAD_CAST XCCDF_RESULT_MAP[test_res - 1].string);
 
 	struct xccdf_override_iterator *overrides = xccdf_rule_result_get_overrides(result);
 	while (xccdf_override_iterator_has_more(overrides)) {
@@ -627,7 +698,10 @@ xmlNode *xccdf_rule_result_to_dom(struct xccdf_rule_result *result, xmlDoc *doc,
 		struct xccdf_message *message = xccdf_message_iterator_next(messages);
 		const char *content = xccdf_message_get_content(message);
 		xmlNode *message_node = xmlNewChild(result_node, ns_xccdf, BAD_CAST "message", BAD_CAST content);
-		xmlNewProp(message_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
+
+                xccdf_level_t message_severity = xccdf_message_get_severity(message);
+		if (message_severity != 0)
+                    xmlNewProp(message_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[message_severity - 1].string);
 	}
 	xccdf_message_iterator_free(messages);
 
@@ -713,30 +787,18 @@ xmlNode *xccdf_override_to_dom(struct xccdf_override *override, xmlDoc *doc, xml
 		xmlNewProp(override_node, BAD_CAST "authority", BAD_CAST authority);
 
 	xccdf_test_result_type_t old = xccdf_override_get_old_result(override);
-	xmlNewProp(override_node, BAD_CAST "old-result", BAD_CAST XCCDF_RESULT_MAP[old - 1].string);
+	if (old != 0)
+		xmlNewProp(override_node, BAD_CAST "old-result", BAD_CAST XCCDF_RESULT_MAP[old - 1].string);
 
 	xccdf_test_result_type_t new = xccdf_override_get_new_result(override);
-	xmlNewProp(override_node, BAD_CAST "new-result", BAD_CAST XCCDF_RESULT_MAP[new - 1].string);
+	if (new != 0)
+		xmlNewProp(override_node, BAD_CAST "new-result", BAD_CAST XCCDF_RESULT_MAP[new - 1].string);
 
-	struct oscap_text *remark = xccdf_override_get_remark(override);
-	if (remark)
-		xccdf_remark_to_dom(remark, doc, override_node);
+	oscap_text_to_dom(xccdf_override_get_remark(override), override_node, "remark");
 
 	return override_node;
 }
 
-xmlNode *xccdf_remark_to_dom(struct oscap_text *remark, xmlDoc *doc, xmlNode *parent)
-{
-	xmlNs *ns_xccdf = xmlSearchNsByHref(doc, parent, XCCDF_BASE_NAMESPACE);
-	xmlNode *remark_node = xmlNewChild(parent, ns_xccdf, BAD_CAST "remark", BAD_CAST oscap_text_get_text(remark));
-
-	const char *lang = oscap_text_get_lang(remark);
-	if (lang)
-		xmlNewProp(remark_node, BAD_CAST "xml:lang", BAD_CAST lang);
-
-	return remark_node;
-}
-
 static struct xccdf_message *xccdf_message_new_parse(xmlTextReaderPtr reader)
 {
 	XCCDF_ASSERT_ELEMENT(reader, XCCDFE_MESSAGE);
@@ -759,6 +821,29 @@ static struct xccdf_instance *xccdf_instance_new_parse(xmlTextReaderPtr reader)
 	return inst;
 }
 
+const char * xccdf_test_result_type_get_text(xccdf_test_result_type_t id)
+{
+    if (id == 0) return NULL;
+
+    return XCCDF_RESULT_MAP[id - 1].string;
+}
+
+struct xccdf_rule_result * xccdf_result_get_rule_result_by_id(struct xccdf_result * result, const char * id)
+{
+        struct xccdf_rule_result * rule_result = NULL;
+    
+        struct xccdf_rule_result_iterator * rr_it = xccdf_result_get_rule_results(result);
+        while (xccdf_rule_result_iterator_has_more(rr_it)) {
+            rule_result = xccdf_rule_result_iterator_next(rr_it);
+            if (oscap_streq(xccdf_rule_result_get_idref(rule_result), id))
+                break;
+            else rule_result = NULL;
+        }
+        xccdf_rule_result_iterator_free(rr_it);
+
+        return rule_result;
+}
+
 /* --------  DUMP ---------- */
 
 static void xccdf_rule_result_dump(struct xccdf_rule_result *res, int depth)
