@@ -185,9 +185,9 @@ struct xccdf_group * xccdf_group_clone(const struct xccdf_group * group)
 {
 	struct xccdf_item *new_group = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_group_item));
 	struct xccdf_item *old = XITEM(group);
-	new_group->item = *(xccdf_item_base_clone(&(old->item)));
+    xccdf_item_base_clone(&new_group->item, &(old->item));
 	new_group->type = old->type;
-	new_group->sub.group = *(xccdf_group_item_clone(&(old->sub.group), new_group));
+    xccdf_group_item_clone(new_group, &(old->sub.group));
 	return XGROUP(new_group);
 }
 
@@ -260,8 +260,8 @@ void xccdf_group_free(struct xccdf_item *group)
 struct xccdf_item *xccdf_rule_new_internal(struct xccdf_item *parent)
 {
 	struct xccdf_item *rule = xccdf_item_new(XCCDF_RULE, parent);
-	rule->sub.rule.role = XCCDF_ROLE_FULL;
-	rule->sub.rule.severity = XCCDF_UNKNOWN;
+	//rule->sub.rule.role = XCCDF_ROLE_FULL;
+	//rule->sub.rule.severity = XCCDF_UNKNOWN;
 	rule->sub.rule.idents = oscap_list_new();
 	rule->sub.rule.checks = oscap_list_new();
 	rule->sub.rule.requires = oscap_list_new();
@@ -281,9 +281,9 @@ struct xccdf_rule *xccdf_rule_clone(const struct xccdf_rule * rule)
 {
 	struct xccdf_item *new_rule = oscap_calloc(1, sizeof(struct xccdf_item) + sizeof(struct xccdf_rule_item));
 	struct xccdf_item *old = XITEM(rule);
-	new_rule->item = *(xccdf_item_base_clone(&(old->item)));
+    xccdf_item_base_clone(&new_rule->item, &(old->item));
 	new_rule->type = old->type;
-	new_rule->sub.rule = *(xccdf_rule_item_clone(&(old->sub.rule)));
+    xccdf_rule_item_clone(&new_rule->sub.rule, &old->sub.rule);
 	return XRULE(new_rule);
 }
 
@@ -297,11 +297,15 @@ struct xccdf_item *xccdf_rule_parse(xmlTextReaderPtr reader, struct xccdf_item *
 		xccdf_rule_free(rule);
 		return NULL;
 	}
-	if (xccdf_attribute_has(reader, XCCDFA_ROLE))
+	if (xccdf_attribute_has(reader, XCCDFA_ROLE)) {
 		rule->sub.rule.role = oscap_string_to_enum(XCCDF_ROLE_MAP, xccdf_attribute_get(reader, XCCDFA_ROLE));
-	if (xccdf_attribute_has(reader, XCCDFA_SEVERITY))
+		rule->item.defined_flags.role = true;
+	}
+	if (xccdf_attribute_has(reader, XCCDFA_SEVERITY)) {
 		rule->sub.rule.severity =
 		    oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_SEVERITY));
+		rule->item.defined_flags.severity = true;
+	}
 
 	int depth = oscap_element_depth(reader) + 1;
 
@@ -321,12 +325,11 @@ struct xccdf_item *xccdf_rule_parse(xmlTextReaderPtr reader, struct xccdf_item *
 				oscap_list_add(rule->sub.rule.profile_notes, note);
 				break;
 			}
+                case XCCDFE_COMPLEX_CHECK:
 		case XCCDFE_CHECK:{
 				struct xccdf_check *check = xccdf_check_parse(reader);
 				if (check == NULL)
 					break;
-				if (check->selector == NULL || strcmp(check->selector, "") == 0)
-					rule->sub.rule.check = check;
 				oscap_list_add(rule->sub.rule.checks, check);
 				break;
 			}
@@ -378,11 +381,6 @@ void xccdf_rule_free(struct xccdf_item *rule)
 	}
 }
 
-struct xccdf_ident *xccdf_ident_new(void)
-{
-    return oscap_calloc(1, sizeof(struct xccdf_ident));
-}
-
 struct xccdf_ident * xccdf_ident_clone(const struct xccdf_ident * ident)
 {
 	struct xccdf_ident * clone = xccdf_ident_new();
@@ -391,6 +389,11 @@ struct xccdf_ident * xccdf_ident_clone(const struct xccdf_ident * ident)
 	return clone;
 }
 
+struct xccdf_ident *xccdf_ident_new(void)
+{
+    return oscap_calloc(1, sizeof(struct xccdf_ident));
+}
+
 struct xccdf_ident *xccdf_ident_new_fill(const char *id, const char *sys)
 {
 	struct xccdf_ident *ident = xccdf_ident_new();
@@ -679,14 +682,6 @@ const struct oscap_string_map XCCDF_STRATEGY_MAP[] = {
 	{XCCDF_STRATEGY_UNKNOWN, NULL}
 };
 
-#define XCCDF_FIXCOMMON_PARSE(reader, fix) do { \
-	fix->reboot     = xccdf_attribute_get_bool(reader, XCCDFA_REBOOT); \
-	fix->strategy   = oscap_string_to_enum(XCCDF_STRATEGY_MAP, xccdf_attribute_get(reader, XCCDFA_STRATEGY)); \
-	fix->disruption = oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_DISRUPTION)); \
-	fix->complexity = oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_COMPLEXITY)); \
-	fix->content    = oscap_element_string_copy(reader); \
-	} while (false)
-
 struct xccdf_fix *xccdf_fix_new(void)
 {
         return oscap_calloc(1, sizeof(struct xccdf_fix));
@@ -718,7 +713,11 @@ struct xccdf_fix *xccdf_fix_parse(xmlTextReaderPtr reader)
 	fix->id = xccdf_attribute_copy(reader, XCCDFA_ID);
 	fix->system = xccdf_attribute_copy(reader, XCCDFA_SYSTEM);
 	fix->platform = xccdf_attribute_copy(reader, XCCDFA_PLATFORM);
-	XCCDF_FIXCOMMON_PARSE(reader, fix);
+	fix->reboot     = xccdf_attribute_get_bool(reader, XCCDFA_REBOOT);
+	fix->strategy   = oscap_string_to_enum(XCCDF_STRATEGY_MAP, xccdf_attribute_get(reader, XCCDFA_STRATEGY));
+	fix->disruption = oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_DISRUPTION));
+	fix->complexity = oscap_string_to_enum(XCCDF_LEVEL_MAP, xccdf_attribute_get(reader, XCCDFA_COMPLEXITY));
+	fix->content    = oscap_get_xml(reader);
 	return fix;
 }
 
@@ -735,7 +734,7 @@ struct xccdf_fixtext * xccdf_fixtext_clone(const struct xccdf_fixtext * fixtext)
 	clone->disruption = fixtext->disruption;
 	clone->complexity = fixtext->complexity;
 	clone->fixref = oscap_strdup(fixtext->fixref);
-	clone->content = oscap_strdup(fixtext->content);
+	clone->text = oscap_text_clone(fixtext->text);
 	return clone;	
 }
 
@@ -743,14 +742,14 @@ struct xccdf_fixtext *xccdf_fixtext_parse(xmlTextReaderPtr reader)
 {
 	struct xccdf_fixtext *fix = xccdf_fixtext_new();
 	fix->fixref = xccdf_attribute_copy(reader, XCCDFA_FIXREF);
-	XCCDF_FIXCOMMON_PARSE(reader, fix);
+	fix->text = oscap_text_new_parse(XCCDF_TEXT_HTMLSUB, reader);
 	return fix;
 }
 
 void xccdf_fixtext_free(struct xccdf_fixtext *item)
 {
 	if (item) {
-		oscap_free(item->content);
+		oscap_text_free(item->text);
 		oscap_free(item->fixref);
 		oscap_free(item);
 	}
@@ -809,23 +808,26 @@ void xccdf_rule_to_dom(struct xccdf_rule *rule, xmlNode *rule_node, xmlDoc *doc,
 		xmlNewProp(rule_node, BAD_CAST "extends", BAD_CAST extends);
 
 	if (xccdf_rule_get_multiple(rule))
-		xmlNewProp(rule_node, BAD_CAST "multiple", BAD_CAST "True");
+		xmlNewProp(rule_node, BAD_CAST "multiple", BAD_CAST "true");
 
 	if (xccdf_rule_get_selected(rule))
-		xmlNewProp(rule_node, BAD_CAST "selected", BAD_CAST "True");
+		xmlNewProp(rule_node, BAD_CAST "selected", BAD_CAST "true");
 	else
-		xmlNewProp(rule_node, BAD_CAST "selected", BAD_CAST "False");
+		xmlNewProp(rule_node, BAD_CAST "selected", BAD_CAST "false");
 
-	float weight = xccdf_rule_get_weight(rule);
-	char weight_str[10];
-	sprintf(weight_str, "%f", weight);
-	xmlNewProp(rule_node, BAD_CAST "weight", BAD_CAST weight_str);
+	if (XITEM(rule)->item.defined_flags.weight) {
+		char *weight_str = oscap_sprintf("%f", xccdf_rule_get_weight(rule));
+		xmlNewProp(rule_node, BAD_CAST "weight", BAD_CAST weight_str);
+		oscap_free(weight_str);
+	}
 
 	xccdf_role_t role = xccdf_rule_get_role(rule);
-	xmlNewProp(rule_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
+	if (role != 0)
+		xmlNewProp(rule_node, BAD_CAST "role", BAD_CAST XCCDF_ROLE_MAP[role - 1].string);
 
 	xccdf_level_t severity = xccdf_rule_get_severity(rule);
-	xmlNewProp(rule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
+	if (severity != 0)
+		xmlNewProp(rule_node, BAD_CAST "severity", BAD_CAST XCCDF_LEVEL_MAP[severity - 1].string);
 
 	/* Handle Child Nodes */
 	struct oscap_text_iterator *rationales = xccdf_rule_get_rationale(rule);
@@ -907,24 +909,24 @@ void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *
 	if (extends)
 		xmlNewProp(group_node, BAD_CAST "extends", BAD_CAST extends);
 
-	if (xccdf_group_get_selected(group))
-		xmlNewProp(group_node, BAD_CAST "selected", BAD_CAST "True");
-	else
-		xmlNewProp(group_node, BAD_CAST "selected", BAD_CAST "False");
+	if (XITEM(group)->item.defined_flags.selected) {
+		if (xccdf_group_get_selected(group))
+			xmlNewProp(group_node, BAD_CAST "selected", BAD_CAST "True");
+		else
+			xmlNewProp(group_node, BAD_CAST "selected", BAD_CAST "False");
+	}
 
-	float weight = xccdf_group_get_weight(group);
-	char weight_str[10];
-	sprintf(weight_str, "%f", weight);
-	xmlNewProp(group_node, BAD_CAST "weight", BAD_CAST weight_str);
+	xmlNewProp(group_node, BAD_CAST "hidden", BAD_CAST (xccdf_group_get_hidden(group) ? "true" : "false"));
+
+	if (XITEM(group)->item.defined_flags.weight) {
+		float weight = xccdf_group_get_weight(group);
+		char weight_str[10];
+		sprintf(weight_str, "%f", weight);
+		xmlNewProp(group_node, BAD_CAST "weight", BAD_CAST weight_str);
+	}
 
 	/* Handle Child Nodes */
-	struct oscap_text_iterator *rationales = xccdf_group_get_rationale(group);
-	while (oscap_text_iterator_has_more(rationales)) {
-		struct oscap_text *rationale = oscap_text_iterator_next(rationales);
-		xmlNode * child = xmlNewChild(group_node, ns_xccdf, BAD_CAST "rationale", BAD_CAST oscap_text_get_text(rationale));
-                if (oscap_text_get_lang(rationale) != NULL) xmlNewProp(child, BAD_CAST "xml:lang", BAD_CAST oscap_text_get_lang(rationale));
-	}
-	oscap_text_iterator_free(rationales);
+	xccdf_texts_to_dom(xccdf_group_get_rationale(group), group_node, "rationale");
 
 	struct oscap_string_iterator *platforms = xccdf_group_get_platforms(group);
 	while (oscap_string_iterator_has_more(platforms)) {
@@ -965,6 +967,7 @@ void xccdf_group_to_dom(struct xccdf_group *group, xmlNode *group_node, xmlDoc *
 XCCDF_STATUS_CURRENT(rule)
 XCCDF_STATUS_CURRENT(group)
 XCCDF_GROUP_IGETTER(item, content)
+XCCDF_GROUP_IGETTER(value, values)
 
 XCCDF_ITEM_ADDER_REG(group, rule, content)
 XCCDF_ITEM_ADDER_REG(group, group, content)
@@ -984,8 +987,6 @@ bool xccdf_group_add_content(struct xccdf_group *rule, struct xccdf_item *item)
 XCCDF_ACCESSOR_STRING(rule, impact_metric)
 XCCDF_ACCESSOR_SIMPLE(rule, xccdf_role_t, role)
 XCCDF_ACCESSOR_SIMPLE(rule, xccdf_level_t, severity)
-XCCDF_RULE_GETTER(struct xccdf_check *, check)
-XCCDF_SETTER_GENERIC(rule, check, struct xccdf_check *, xccdf_check_free,)
 XCCDF_LISTMANIP(rule, ident, idents)
 XCCDF_LISTMANIP(rule, check, checks)
 XCCDF_LISTMANIP(rule, profile_note, profile_notes)
@@ -1014,7 +1015,7 @@ OSCAP_ACCESSOR_SIMPLE(xccdf_strategy_t, xccdf_fixtext, strategy)
 OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fixtext, disruption)
 OSCAP_ACCESSOR_SIMPLE(xccdf_level_t, xccdf_fixtext, complexity)
 OSCAP_ACCESSOR_SIMPLE(bool, xccdf_fixtext, reboot)
-OSCAP_ACCESSOR_STRING(xccdf_fixtext, content)
+OSCAP_ACCESSOR_TEXT(xccdf_fixtext, text)
 OSCAP_ACCESSOR_STRING(xccdf_fixtext, fixref) XCCDF_ITERATOR_GEN_S(fixtext)
 
 OSCAP_ACCESSOR_SIMPLE(xccdf_strategy_t, xccdf_fix, strategy)
