@@ -35,7 +35,6 @@
 #include <time.h>
 #include <oscap.h>
 #include <xccdf.h>
-#include <oval_definitions.h>
  
 /**
  * @struct xccdf_policy_model
@@ -55,6 +54,8 @@ struct xccdf_policy;
  */
 struct xccdf_value_binding;
 
+struct xccdf_value_binding_iterator;
+
 /**
  * @struct xccdf_policy_iterator
  * Iterate through policies
@@ -80,18 +81,12 @@ struct xccdf_policy_model *xccdf_policy_model_new(struct xccdf_benchmark *benchm
  */
 struct xccdf_policy * xccdf_policy_new(struct xccdf_policy_model * model, struct xccdf_profile * profile);
 
-/**
- * Constructor for Value binding
- * @memberof xccdf_value_binding
- */
-struct xccdf_value_binding * xccdf_value_binding_new(void);
-
 /**
  * Constructor of structure with profile bindings - refine_rules, refine_values and set_values
  * @memberof xccdf_value_binding
  * @return new structure of xccdf_value_binding
  */
-struct xccdf_value_binding * xccdf_value_binding_new();
+struct xccdf_value_binding * xccdf_value_binding_new(void);
 
 /** 
  * Destructor of Policy Model structure
@@ -116,28 +111,29 @@ void xccdf_value_binding_free(struct xccdf_value_binding *);
  * Not yet implemented
  * @memberof xccdf_policy
  */
-void xccdf_policy_export_variables(struct xccdf_policy *, char *export_namespace, struct oscap_export_target *);
+void xccdf_policy_export_variables(struct xccdf_policy *, char *export_namespace, const char *file);
 /**
  * Import/Export function
  * Not yet implemented 
  * @memberof xccdf_policy
  */
-void xccdf_policy_export_controls (struct xccdf_policy *, char *export_namespace, struct oscap_export_target *);
+void xccdf_policy_export_controls (struct xccdf_policy *, char *export_namespace, const char *file);
 /**
  * Import/Export function
  * Not yet implemented
  * @memberof xccdf_policy
  */
-void xccdf_policy_import_results(struct xccdf_policy *, char *import_namespace, struct oscap_import_source *);
+void xccdf_policy_import_results(struct xccdf_policy *, char *import_namespace, const char *file);
 /**
  * Import/Export function
  * Not yet implemented
  * @memberof xccdf_policy
  */
-void xccdf_policy_export_results(struct xccdf_policy *, char *scoring_model_namespace, struct oscap_export_target *);
+void xccdf_policy_export_results(struct xccdf_policy *, char *scoring_model_namespace, const char *file);
 
 /**
  * Function to register callback for checking system
+ * For Python use xccdf_policy_model_register_callback_py
  * @param model XCCDF Policy Model
  * @param sys String representing given checking system
  * @param func Callback - pointer to function called by XCCDF Policy system when rule parsed
@@ -147,6 +143,33 @@ void xccdf_policy_export_results(struct xccdf_policy *, char *scoring_model_name
  */
 bool xccdf_policy_model_register_callback(struct xccdf_policy_model * model, char * sys, void * func, void * usr);
 
+/**
+ * Function to register output callback for checking system that will be called after each rule evaluation.
+ * For Python use xccdf_policy_model_register_rule_callback_py
+ * @param model XCCDF Policy Model
+ * @param func Callback - pointer to function called by XCCDF Policy system when rule parsed
+ * @param usr optional parameter for passing user data to callback
+ * @memberof xccdf_policy_model
+ * @return true if callback registered succesfully, false otherwise
+ * \par Example
+ * With the first function below (register output callback) user registers the callback that will be called after
+ * wach rule evalution is done. Second callback is registered as callback for evaluation itself and will be called
+ * during the evaluation.
+ * \code
+ * xccdf_policy_model_register_output_callback(policy_model, callback, NULL);
+ * xccdf_policy_model_register_callback(policy_model, "http://oval.mitre.org/XMLSchema/oval-definitions-5", oval_agent_eval_rule, (void *) usr);
+ * \endcode
+ * The example of callback:
+ * \code
+ * static int callback(const char *id, int result, void *arg)
+ * {
+ *      printf("The result of rule \"%s\" is: %s\n", id, xccdf_test_result_type_get_text(result));
+ *      return 0;
+ * }
+ * \endcode
+ */
+bool xccdf_policy_model_register_output_callback(struct xccdf_policy_model * model, void * func, void * usr);
+
 /************************************************************/
 /**
  * @name Getters
@@ -208,45 +231,52 @@ struct xccdf_profile * xccdf_policy_get_profile(const struct xccdf_policy *);
 struct xccdf_select_iterator * xccdf_policy_get_rules(const struct xccdf_policy *);
 
 /**
- * Get refine value from value bindings
+ * Get variable name from value bindings
  * @memberof xccdf_value_binding
- * @return xccdf_refine_value
+ * @return String
  */
-struct xccdf_refine_value * xccdf_value_binding_get_refine_value(const struct xccdf_value_binding *);
+char * xccdf_value_binding_get_name(const struct xccdf_value_binding *);
 
 /**
- * Get set value from value bindings
+ * Get value from value bindings
  * @memberof xccdf_value_binding
- * @return xccdf_setvalue
+ * @return String
  */
-struct xccdf_setvalue * xccdf_value_binding_get_set_value(const struct xccdf_value_binding *);
+char * xccdf_value_binding_get_value(const struct xccdf_value_binding *);
 
 /**
- * get value (element value) from value bindings
+ * get variable type from value bindings
  * @memberof xccdf_value_binding
- * @return xccdf_value
+ * @return xccdf_value_type_t
  */
-struct xccdf_value * xccdf_value_binding_get_rule_value(const struct xccdf_value_binding *);
+xccdf_value_type_t xccdf_value_binding_get_type(const struct xccdf_value_binding *);
 
 /**
- * get check export elements from value bindings
+ * get Value operator from value bindings
  * @memberof xccdf_value_binding
- * @return xccdf_check_export_iterator
+ * @return xccdf_operator_t
  */
-struct xccdf_check_export_iterator * xccdf_value_binding_get_check_exports(const struct xccdf_value_binding *);
+xccdf_operator_t xccdf_value_binding_get_operator(const struct xccdf_value_binding *);
+
+/**
+ * get Set Value from value bindings
+ * @memberof xccdf_value_binding
+ * @return String
+ */
+char * xccdf_value_binding_get_setvalue(const struct xccdf_value_binding *);
 
 /**
  * Get results of all XCCDF Policy results
  * @memberof xccdf_policy_model
  */
-struct xccdf_result_iterator * xccdf_policy_model_get_results(const struct xccdf_policy_model * model);
+struct xccdf_result_iterator * xccdf_policy_get_results(const struct xccdf_policy * policy);
 
 /**
  * Get XCCDF Result structure by it's idetificator if there is one
  * @memberof xccdf_policy_model
  * @return structure xccdf_result if found, NULL otherwise
  */
-struct xccdf_result * xccdf_policy_model_get_result_by_id(struct xccdf_policy_model * model, const char * id);
+struct xccdf_result * xccdf_policy_get_result_by_id(struct xccdf_policy * policy, const char * id);
 
 /**
  * Get ID of XCCDF Profile that is implemented by XCCDF Policy
@@ -265,16 +295,6 @@ const char * xccdf_policy_get_id(struct xccdf_policy * policy);
  */
 struct xccdf_policy * xccdf_policy_model_get_policy_by_id(struct xccdf_policy_model * policy_model, const char * id);
 
-/**
- * Get external variables from XCCDF for given Policy and export them to OVAL Variable model
- * @param policy XCCDF Policy
- * @param def_model OVAL definition model
- * @return OVAL Variable model
- * @see oval_variable_model
- * @memberof xccdf_policy
- */
-struct oval_variable_model * xccdf_policy_get_variables(struct xccdf_policy * policy, struct oval_definition_model * def_model);
-
 /************************************************************/
 /** @} End of Getters group */
 
@@ -300,27 +320,6 @@ bool xccdf_policy_model_add_policy(struct xccdf_policy_model *, struct xccdf_pol
  */
 bool xccdf_policy_add_rule(struct xccdf_policy *, struct xccdf_select *);
 
-/**
- * Add refine rule to the Value Binding structure
- * @memberof xccdf_value_binding
- * @return true if rule has been added succesfully
- */
-bool xccdf_value_binding_add_refine_rule(struct xccdf_value_binding *, struct xccdf_refine_rule *);
-
-/**
- * Add refine value to the Value Binding structure
- * @memberof xccdf_value_binding
- * @return true if rule has been added succesfully
- */
-bool xccdf_value_binding_add_refine_value(struct xccdf_value_binding *, struct xccdf_refine_value *);
-
-/**
- * Add value to the Value Binding structure
- * @memberof xccdf_value_binding
- * @return true if rule has been added succesfully
- */
-bool xccdf_value_binding_add_setvalue(struct xccdf_value_binding *, struct xccdf_setvalue *);
-
 /**
  * Set a new selector to the Policy structure
  * @memberof xccdf_policy
@@ -332,7 +331,7 @@ bool xccdf_policy_set_selected(struct xccdf_policy * policy, char * idref);
  * Add result to XCCDF Policy Model
  * @memberof xccdf_policy_model
  */
-bool xccdf_policy_model_add_result(struct xccdf_policy_model * model, struct xccdf_result * item);
+bool xccdf_policy_add_result(struct xccdf_policy * policy, struct xccdf_result * item);
 
 /**
  * Add value binding to the Policy structure
@@ -346,7 +345,7 @@ bool xccdf_policy_add_value(struct xccdf_policy *, struct xccdf_value_binding *)
  * @memberof xccdf_value_binding
  * @return true if rule has been added succesfully
  */
-bool xccdf_value_binding_add_check_export(struct xccdf_value_binding *, struct xccdf_check_export *);
+//bool xccdf_value_binding_add_check_export(struct xccdf_value_binding *, struct xccdf_check_export *);
 
 /************************************************************/
 /** @} End of Setters group */
@@ -362,8 +361,24 @@ bool xccdf_value_binding_add_check_export(struct xccdf_value_binding *, struct x
  * @param policy given Policy to evaluate
  * @memberof xccdf_policy
  * @return true if evaluation pass or false in case of error
- */
-bool xccdf_policy_evaluate(struct xccdf_policy * policy);
+ * \par Example
+ * Before each policy evaluation user has to register callback that will be called for each check.
+ * Every checking engine must have registered callback or the particular check will be skipped.
+ * In the code below is used the predefined function \ref oval_agent_eval_rule for evaluation OVAL checks:
+ * \code
+ * xccdf_policy_model_register_callback(policy_model, "http://oval.mitre.org/XMLSchema/oval-definitions-5", oval_agent_eval_rule, (void *) usr);
+ * \endcode
+ * \par
+ * If you use this predefined OVAL callback, user data structure (last parameter of register function) \b MUST be of type \ref\a oval_agent_cb_data:
+ * \code
+ * struct oval_agent_cb_data *usr = oval_agent_cb_data_new();
+ * oval_agent_cb_data_set_session(usr, sess);
+ * oval_agent_cb_data_set_usr(usr, (void *) policy_model);
+ * \endcode
+ * Optionally you can use \code oval_agent_cb_data_set_callback(usr, callback); \endcode with callback which will be called inside the
+ * evaluation callback (useful for debugging checking engines) and has same parameters as output callback.
+ * */
+struct xccdf_result *  xccdf_policy_evaluate(struct xccdf_policy * policy);
 
 /**
  * Resolve benchmark by applying all refine_rules and refine_values to rules / values
@@ -420,6 +435,21 @@ struct xccdf_value_binding * xccdf_value_binding_iterator_next(struct xccdf_valu
  */
 void xccdf_value_binding_iterator_free(struct xccdf_value_binding_iterator *it);
 
+/**
+ * Reset the iterator structure (it will point to the first item in the list)
+ * @memberof xccdf_value_binding_iterator
+ */
+void xccdf_value_binding_iterator_reset(struct xccdf_value_binding_iterator *it);
+
+/**
+ * Get score of the XCCDF Benchmark
+ * @param policy XCCDF Policy
+ * @param test_result Test Result model
+ * @param system Score system
+ * @return XCCDF Score
+ */
+struct xccdf_score * xccdf_policy_get_score(struct xccdf_policy * policy, struct xccdf_result * test_result, const char * system);
+
 /************************************************************/
 /** @} End of Iterators group */
 
