@@ -61,6 +61,33 @@ bool oscap_list_add(struct oscap_list * list, void *value)
 	return true;
 }
 
+bool oscap_list_push(struct oscap_list *list, void *value)
+{
+	return oscap_list_add(list,value);
+}
+
+bool oscap_list_pop(struct oscap_list *list, oscap_destruct_func destructor)
+{
+	if (list == NULL || list->first == NULL) return false;
+	struct oscap_list_item *cur = list->first, *prev = NULL;
+
+	while (cur != list->last) {
+		prev = cur;
+		cur = cur->next;
+	}
+
+	if (destructor) destructor(cur->data);
+	oscap_free(cur);
+
+	list->last = prev;
+	if (prev) prev->next = NULL;
+	else list->first = NULL;
+
+	--list->itemcount;
+
+	return true;
+}
+
 struct oscap_list * oscap_list_clone(const struct oscap_list * list, oscap_clone_func cloner)
 {
         if (list == NULL) 
@@ -73,13 +100,24 @@ struct oscap_list * oscap_list_clone(const struct oscap_list * list, oscap_clone
         while (item != NULL) {
                 if (cloner)
                     oscap_list_add(copy, cloner(item->data));
-                else oscap_list_add(copy, NULL);
+                else oscap_list_add(copy, item->data);
                 item = item->next;
         }
 
         return copy;
 }
 
+struct oscap_list *oscap_list_destructive_join(struct oscap_list *list1, struct oscap_list *list2)
+{
+	if (list1->first == NULL)
+		list1->first = list2->first;
+	else list1->last->next = list2->first;
+	if (list2->last != NULL) list1->last = list2->last;
+	list1->itemcount += list2->itemcount;
+	oscap_free(list2);
+	return list1;
+}
+
 int oscap_list_get_itemcount(struct oscap_list *list)
 {
 	__attribute__nonnull__(list);
@@ -107,6 +145,35 @@ void oscap_list_free0(struct oscap_list *list)
 	oscap_list_free(list, NULL);
 }
 
+bool oscap_ptr_cmp(void *node1, void *node2)
+{
+	return node1 == node2;
+}
+
+void* oscap_list_find(struct oscap_list *list, void *what, oscap_cmp_func compare)
+{
+	if (list == NULL) return false;
+	if (compare == NULL) compare = oscap_ptr_cmp;
+
+	struct oscap_iterator *it = oscap_iterator_new(list);
+
+	while (oscap_iterator_has_more(it)) {
+		void *item = oscap_iterator_next(it);
+		if (compare(item, what)) {
+			oscap_iterator_free(it);
+			return item;
+		}
+	}
+
+	oscap_iterator_free(it);
+	return NULL;
+}
+
+bool oscap_list_contains(struct oscap_list *list, void *what, oscap_cmp_func compare)
+{
+	return oscap_list_find(list, what, compare) != NULL;
+}
+
 void oscap_list_dump(struct oscap_list *list, oscap_dump_func dumper, int depth)
 {
 	if (list == NULL) {
@@ -134,10 +201,6 @@ static inline void oscap_iterator_find_nearest(struct oscap_iterator *it)
 	do {
 		it->cur = (it->cur ? it->cur->next : it->list->first);
 	} while (!it->filter(it->cur->data, it->user_data) && oscap_iterator_has_more(it));
-	/*
-	   while (it->cur && !it->filter(it->cur->data, it->user_data))
-	   it->cur = it->cur->next;
-	 */
 }
 
 void *oscap_iterator_new(struct oscap_list *list)
@@ -218,10 +281,23 @@ void *oscap_iterator_next(struct oscap_iterator *it)
 
 bool oscap_iterator_has_more(struct oscap_iterator *it)
 {
-	__attribute__nonnull__(it);
+	if (!it) return false;
 	return (!it->cur && it->list->first) || (it->cur && it->cur->next);
 }
 
+void oscap_iterator_reset(struct oscap_iterator * it)
+{
+    it->cur = NULL;
+}
+
+struct oscap_stringlist *oscap_stringlist_clone(struct oscap_stringlist *list)
+{
+    void *clone = oscap_list_new(); // oscap_stringlist (or oscap_list)
+    OSCAP_FOR_STR(str, oscap_stringlist_get_strings(list))
+        oscap_list_add(clone, oscap_strdup(str));
+    return clone;
+}
+
 struct oscap_string_iterator *oscap_stringlist_get_strings(const struct oscap_stringlist* list)
 {
 	return oscap_iterator_new((struct oscap_list *) list);
@@ -276,14 +352,12 @@ struct oscap_htable * oscap_htable_clone(const struct oscap_htable * table, osca
 	if (t == NULL)
 		return NULL;
 
-	int i;
-	for (i = 0; i < table->hsize; ++i) {
+	for (size_t i = 0; i < table->hsize; ++i) {
 		struct oscap_htable_item *item = table->table[i];
 		while (item != NULL) {
 			oscap_htable_add(t, item->key, (void *) cloner(item->value));
 			item = item->next;
 		}
-		
 	}
 	
 	return t;
