@@ -96,10 +96,11 @@ struct oscap_text *oscap_text_new_parse(struct oscap_text_traits traits, xmlText
     // new text
     struct oscap_text *text = oscap_text_new_full(traits, NULL, NULL);
 
-    // extract 'overrides' attribute
+    // extract 'override' attribute
     if (text->traits.can_override) {
-        xmlTextReaderMoveToAttribute(reader, BAD_CAST "overrides");
+        xmlTextReaderMoveToAttribute(reader, BAD_CAST "override");
         text->traits.overrides = oscap_string_to_enum(OSCAP_BOOL_MAP, (const char *) xmlTextReaderConstValue(reader));
+		if (xmlTextReaderConstValue(reader) != NULL) text->traits.override_given = true;
     }
 
     // extract language
@@ -108,7 +109,8 @@ struct oscap_text *oscap_text_new_parse(struct oscap_text_traits traits, xmlText
     xmlTextReaderMoveToElement(reader);
 
     // extract content
-    if (text->traits.html) text->text = oscap_get_xml(reader);
+    if (text->traits.html || text->traits.can_substitute)
+		text->text = oscap_get_xml(reader);
     else text->text = oscap_element_string_copy(reader);
 
     // TODO substitution support
@@ -116,4 +118,24 @@ struct oscap_text *oscap_text_new_parse(struct oscap_text_traits traits, xmlText
     return text;
 }
 
+xmlNode *oscap_text_to_dom(struct oscap_text *text, xmlNode *parent, const char *elname)
+{
+	if (!text) return NULL;
+
+	xmlNode *text_node = NULL;
+
+	if (text->traits.html || text->traits.can_substitute)
+		text_node = oscap_xmlstr_to_dom(parent, elname, text->text);
+	else text_node = xmlNewChild(parent, NULL, BAD_CAST elname, BAD_CAST text->text);
+
+	if (text_node == NULL) return NULL;
+
+	if (text->lang)
+		xmlNodeSetLang(text_node, BAD_CAST text->lang);
+	if (text->traits.can_override && text->traits.override_given)
+		xmlNewProp(text_node, BAD_CAST "override", BAD_CAST (text->traits.overrides ? "true" : "false"));
+
+
+	return text_node;
+}
 
