@@ -12,33 +12,30 @@
 #include <string.h>
 #include <oval_agent_api.h>
 #include <assume.h>
+#include <error.h>
 
 int main(int argc, char **argv) {
-  
-  /* definition model populate */
-  struct oscap_import_source *def_in = oscap_import_source_new_file(argv[1], NULL);
-  assume(def_in != NULL);
-  
-  struct oval_definition_model *def_model = oval_definition_model_new();
-  assume(def_model != NULL);
 
-  int ret = oval_definition_model_import(def_model, def_in);
-  assume(ret != -1);
-  
-  oscap_import_source_free(def_in);
+  /* definition model populate */
+  struct oval_definition_model *def_model=NULL;
+  def_model = oval_definition_model_import(argv[1]);
+  assume(def_model != NULL);
 
   assume(oval_definition_model_is_valid(def_model));
 
   /* create syschar model */
   struct oval_syschar_model *sys_model = oval_syschar_model_new(def_model);
   assume(sys_model != NULL);
-      
+
+  /* create probe session */
+  oval_probe_session_t *sess = oval_probe_session_new(sys_model);
+
   /* probe sysinfo */
-  oval_syschar_model_probe_sysinfo(sys_model);
+  assume(oval_probe_session_query_sysinfo(sess) == 0);
 
   /* call probes */
-  assume(oval_syschar_model_probe_objects(sys_model) == 0);
-  
+  assume(oval_probe_session_query_objects(sess) == 0);
+
   /* create result model */
   struct oval_syschar_model *sys_models[] = {sys_model, NULL};
   struct oval_results_model* res_model = oval_results_model_new( def_model, sys_models );
@@ -50,8 +47,7 @@ int main(int argc, char **argv) {
   /* set up directives */
   struct oval_result_directives * res_direct = oval_result_directives_new(res_model);
   assume(res_direct != NULL);
-  
-  oval_result_directives_set_reported(res_direct, OVAL_RESULT_INVALID, true);
+
   oval_result_directives_set_reported(res_direct, OVAL_RESULT_TRUE, true);
   oval_result_directives_set_reported(res_direct, OVAL_RESULT_FALSE, true);
   oval_result_directives_set_reported(res_direct, OVAL_RESULT_UNKNOWN, true);
@@ -60,26 +56,23 @@ int main(int argc, char **argv) {
   oval_result_directives_set_reported(res_direct, OVAL_RESULT_NOT_APPLICABLE , true);
   oval_result_directives_set_content(res_direct,OVAL_RESULT_FALSE, OVAL_DIRECTIVE_CONTENT_FULL);
   oval_result_directives_set_content(res_direct,OVAL_RESULT_TRUE, OVAL_DIRECTIVE_CONTENT_FULL);
-  oval_result_directives_set_content(res_direct,OVAL_RESULT_INVALID, OVAL_DIRECTIVE_CONTENT_FULL);
   oval_result_directives_set_content(res_direct,OVAL_RESULT_UNKNOWN, OVAL_DIRECTIVE_CONTENT_FULL);
   oval_result_directives_set_content(res_direct,OVAL_RESULT_ERROR, OVAL_DIRECTIVE_CONTENT_FULL);
   oval_result_directives_set_content(res_direct,OVAL_RESULT_NOT_EVALUATED, OVAL_DIRECTIVE_CONTENT_FULL);
   oval_result_directives_set_content(res_direct,OVAL_RESULT_NOT_APPLICABLE, OVAL_DIRECTIVE_CONTENT_FULL);
-  
+
   /* report results */
-  struct oscap_export_target *result_out  = oscap_export_target_new_file(argv[2], "UTF-8");
-  assume(result_out != NULL);
+  oval_results_model_export(res_model, res_direct, argv[2]);
 
-  oval_results_model_export(res_model, res_direct, result_out);
-
-  oscap_export_target_free(result_out);
-  
+  /* free */
+  oval_probe_session_destroy(sess);
   oval_definition_model_free(def_model);
-
   oval_syschar_model_free(sys_model);
-
   oval_results_model_free(res_model);
-  
+  oval_result_directives_free(res_direct);
+  oscap_cleanup();
+  oscap_clearerr();
+
   return 0;
 }
 
@@ -110,7 +103,7 @@ int main(int argc, char **argv) {
 /*                         c = oscap_err_code (), */
 /*                         d = oscap_err_desc ()); */
 /*         } */
-                
+
 /* 	return 0; */
 /* } */
 
