@@ -6,13 +6,13 @@
 
 int main (void)
 {
-  
-  /* 
+
+  /*
    *  Create empty models
    */
   struct oval_definition_model *def_model = oval_definition_model_new();
   assume(def_model != NULL);
-  
+
   struct oval_syschar_model *sys_model = oval_syschar_model_new(def_model);
   assume(sys_model != NULL);
 
@@ -25,7 +25,7 @@ int main (void)
   /*
    *  Call the sysinfo probe.
    */
-  struct oval_sysinfo *info = oval_probe_sysinf_eval (sess);
+  struct oval_sysinfo *info = oval_probe_sysinfo_query (sess);
   assume(info != NULL);
 
   if (info != NULL) {
@@ -35,22 +35,22 @@ int main (void)
 	    "       os_version: %s\n"
 	    "  os_architecture: %s\n"
 	    "primary_host_name: %s\n",
-            
+
 	    a = oval_sysinfo_get_os_name (info),
 	    b = oval_sysinfo_get_os_version (info),
 	    c = oval_sysinfo_get_os_architecture (info),
 	    d = oval_sysinfo_get_primary_host_name (info));
-    
+
     assume(a != NULL);
     assume(b != NULL);
     assume(c != NULL);
     assume(d != NULL);
-        
+
     struct oval_sysint_iterator *ifit = oval_sysinfo_get_interfaces (info);
-    
+
     if (ifit != NULL) {
       printf ("Interfaces:\n");
-      
+
       while (oval_sysint_iterator_has_more (ifit)) {
 	struct oval_sysint *ife = oval_sysint_iterator_next (ifit);
         assume(ife != NULL);
@@ -59,24 +59,24 @@ int main (void)
 		oval_sysint_get_name (ife),
 		oval_sysint_get_ip_address (ife),
 		oval_sysint_get_mac_address (ife));
-	
+
 	/* oval_sysint_free (ife); */
       }
-      
+
       oval_sysint_iterator_free (ifit);
     }
-    
+
     oval_sysinfo_free (info);
     oval_probe_session_destroy(sess);
 
     return (0);
   }
-  
+
   /*
    *  Free the probe context. This also terminates
    *  all running probes executed under this context.
    */
   oval_probe_session_destroy(sess);
-  
+
   return (1);
 }
